'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.setToolbarHotkey = undefined;

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _section = require('./section');

var _section2 = _interopRequireDefault(_section);

var _hotkey = require('./hotkey');

var _hotkey2 = _interopRequireDefault(_hotkey);

var _emoji = require('./emoji');

var _emoji2 = _interopRequireDefault(_emoji);

var _string = require('../utils/string');

var _command = require('../utils/command');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var getSections = function getSections(engine) {
  var locale = engine.locale;
  var sectionMap = {};
  (0, _section2.default)(engine).forEach(function (item) {
    sectionMap[item.name] = item;
  });
  // 分组形式的Section
  var groups = [];
  // 普通Section
  groups.push({
    title: locale.section.normal,
    items: [sectionMap.image, sectionMap.table, sectionMap.file, sectionMap.math,
    // sectionMap.codeblock,
    // sectionMap.mindmap,
    // sectionMap.diagram,
    //sectionMap.mxgraph,
    // sectionMap.lockedtext,
    sectionMap.label, sectionMap.date]
  });
  // 媒体Section
  var media = [sectionMap.website,
  //sectionMap.onlinedoc,
  sectionMap.localdoc, sectionMap.video, sectionMap.youku];

  // groups.push({
  //   title: locale.section.embed,
  //   items: media,
  // });
  return groups;
};

var setToolbarHotkey = exports.setToolbarHotkey = function setToolbarHotkey(customList, keyList) {
  var hotkeyMap = {};
  var hotkeyGroupMap = {};
  if (!keyList) {
    keyList = (0, _hotkey2.default)();
  }
  keyList.forEach(function (item) {
    item.keyText = (0, _string.getHotkeyText)(item.keys);
    item.keyValue = item.keys.join('').toLowerCase();
    hotkeyMap[item.name] = item;
    if (item.group) {
      if (hotkeyGroupMap[item.group]) hotkeyGroupMap[item.group].push(item.name);else hotkeyGroupMap[item.group] = [item.name];
    }
  });

  return customList.map(function (group) {
    return group.map(function (item) {
      if ((typeof item === 'undefined' ? 'undefined' : _typeof(item)) === 'object') {
        if (_typeof(item.items) === 'object' && item.items.length > 0) {
          item.items = item.items.map(function (childItem) {
            var childName = (typeof childItem === 'undefined' ? 'undefined' : _typeof(childItem)) === 'object' ? childItem.name : childItem;
            var childConfig = hotkeyMap[childName];
            if (!childItem.hotkey && childConfig && childConfig.keyValue) {
              if ((typeof childItem === 'undefined' ? 'undefined' : _typeof(childItem)) === 'object') childItem.hotkey = childConfig.keyValue;else childItem = {
                name: childName,
                hotkey: childConfig.keyValue
              };
            }
            return childItem;
          });
        } else {
          if (hotkeyGroupMap[item.name]) {
            item.items = [];
            hotkeyGroupMap[item.name].forEach(function (child) {
              var config = hotkeyMap[child];
              var childObj = {
                name: child
              };
              if (config && config.keyValue) {
                childObj.hotkey = config.keyValue;
              }
              item.items.push(childObj);
            });
          } else if (!item.hotkey && hotkeyMap[item.name]) {
            item.hotkey = hotkeyMap[item.name].keyValue;
          }
        }
      } else {
        if (hotkeyGroupMap[item]) {
          item = {
            name: item,
            items: []
          };
          hotkeyGroupMap[item.name].forEach(function (child) {
            var config = hotkeyMap[child];
            var childObj = {
              name: child
            };
            if (config && config.keyValue) {
              childObj.hotkey = config.keyValue;
            }
            item.items.push(childObj);
          });
        } else {
          var config = hotkeyMap[item];
          if (config && config.keyValue) {
            item = {
              name: item,
              hotkey: config.keyValue
            };
          }
        }
      }
      return item;
    });
  });
};

var getToolbarConfig = function getToolbarConfig(engine) {
  var locale = engine.locale;
  // 快捷键

  var hotkeyMap = {};
  (0, _hotkey2.default)(locale).forEach(function (item) {
    item.keyText = (0, _string.getHotkeyText)(item.keys);
    item.keyValue = item.keys.join('').toLowerCase();
    hotkeyMap[item.name] = item;
  });
  // 标题
  var headingMap = {
    p: locale.heading.normal,
    h1: locale.heading.heading_1,
    h2: locale.heading.heading_2,
    h3: locale.heading.heading_3,
    h4: locale.heading.heading_4,
    h5: locale.heading.heading_5,
    h6: locale.heading.heading_6
  };
  var fontnumberArray = [{
    key: '小五',
    value: '小五',
    ptSize: 9
  }, {
    key: '五号',
    value: '五号',
    ptSize: 11
  }, {
    key: '小四',
    value: '小四',
    ptSize: 12
  }, {
    key: '四号',
    value: '四号',
    ptSize: 14
  }, {
    key: '三号',
    value: '三号',
    ptSize: 16
  }, {
    key: '小二',
    value: '小二',
    ptSize: 18
  }, {
    key: '二号',
    value: '二号',
    ptSize: 22
  }, {
    key: '小一',
    value: '小一',
    ptSize: 24
  }, {
    key: '小初',
    value: '小初',
    ptSize: 36
  }];
  var fontnumberMap = {};
  var reverseFontnumberMap = {};
  var fontnumberKeyMap = {};
  fontnumberArray.forEach(function (fontnumber) {
    fontnumberMap[fontnumber.value] = fontnumber.ptSize;
    reverseFontnumberMap[fontnumber.ptSize] = fontnumber.value;
    fontnumberKeyMap[fontnumber.value] = fontnumber.value;
  });
  return [{
    name: 'section',
    title: _react2.default.createElement('span', {
      dangerouslySetInnerHTML: {
        __html: locale.section.buttonTitle
      }
    }),
    type: 'collapse',
    icon: '<span class="zacheditor-icon zacheditor-icon-section" />',
    data: getSections(engine),
    description: locale.section.description,
    onClick: function onClick(name) {
      engine.command.execute(name);
    }
  }, {
    name: 'save',
    title: locale.editor.save,
    hotkey: hotkeyMap.save.keyText,
    hotkeyVal: hotkeyMap.save.keyValue,
    icon: '<span class="zacheditor-icon zacheditor-icon-save" />',
    onClick: function onClick() {
      engine.command.execute('save');
    }
  }, {
    name: 'undo',
    title: locale.history.undo,
    hotkey: hotkeyMap.undo.keyText,
    hotkeyVal: hotkeyMap.undo.keyValue,
    icon: '<span class="zacheditor-icon zacheditor-icon-undo" />',
    getDisabled: function getDisabled() {
      return !engine.command.queryState('undo');
    },
    onClick: function onClick() {
      engine.command.execute('undo');
    }
  }, {
    name: 'redo',
    title: locale.history.redo,
    hotkey: hotkeyMap.redo.keyText,
    hotkeyVal: hotkeyMap.redo.keyValue,
    icon: '<span class="zacheditor-icon zacheditor-icon-redo" />',
    getDisabled: function getDisabled() {
      return !engine.command.queryState('redo');
    },
    onClick: function onClick() {
      engine.command.execute('redo');
    }
  }, {
    name: 'paintformat',
    title: locale.paintformat.buttonTitle,
    icon: '<span class="zacheditor-icon zacheditor-icon-paintformat" />',
    getActive: function getActive() {
      return engine.command.queryState('paintformat');
    },
    onClick: function onClick() {
      engine.command.execute('paintformat');
    }
  }, {
    name: 'removeformat',
    title: locale.clearFormat.buttonTitle,
    icon: '<span class="zacheditor-icon zacheditor-icon-clean" />',
    onClick: function onClick() {
      engine.command.execute('removeformat');
    }
  }, {
    name: 'emoji',
    type: 'dropdown',
    title: locale.emoji.buttonTitle,
    icon: '<span class="zacheditor-icon zacheditor-icon-emoji" />',
    data: _emoji2.default,
    onClick: function onClick(value) {
      engine.command.execute('emoji', value);
    }
  }, {
    name: 'heading',
    type: 'dropdown',
    title: locale.heading.buttonTitle,
    className: 'zacheditor-button-heading',
    data: [{
      key: 'p',
      value: headingMap.p,
      className: ''
    }, {
      key: 'h1',
      value: headingMap.h1,
      className: 'zacheditor-button-set-list-item-heading1'
    }, {
      key: 'h2',
      value: headingMap.h2,
      className: 'zacheditor-button-set-list-item-heading2'
    }, {
      key: 'h3',
      value: headingMap.h3,
      className: 'zacheditor-button-set-list-item-heading3'
    }, {
      key: 'h4',
      value: headingMap.h4,
      className: 'zacheditor-button-set-list-item-heading4'
    }, {
      key: 'h5',
      value: headingMap.h5,
      className: 'zacheditor-button-set-list-item-heading5'
    }, {
      key: 'h6',
      value: headingMap.h6,
      className: 'zacheditor-button-set-list-item-heading6'
    }],
    getActive: function getActive() {
      return engine.command.queryState('heading') || 'p';
    },
    getCurrentText: function getCurrentText(active) {
      return headingMap[active] || locale.heading.normal;
    },
    onClick: function onClick(value) {
      engine.command.execute('heading', value);
    }
  }, {
    name: 'h1',
    title: headingMap.h1,
    hotkey: hotkeyMap.h1.keyText,
    hotkeyVal: hotkeyMap.h1.keyValue,
    icon: '<span class="zacheditor-icon zacheditor-icon-h1" />',
    getActive: function getActive() {
      return engine.command.queryState('heading') === 'h1';
    },
    onClick: function onClick() {
      engine.command.execute('heading', 'h1');
    }
  }, {
    name: 'h2',
    title: headingMap.h2,
    hotkey: hotkeyMap.h2.keyText,
    hotkeyVal: hotkeyMap.h2.keyValue,
    icon: '<span class="zacheditor-icon zacheditor-icon-h2" />',
    getActive: function getActive() {
      return engine.command.queryState('heading') === 'h2';
    },
    onClick: function onClick() {
      engine.command.execute('heading', 'h2');
    }
  }, {
    name: 'h3',
    title: headingMap.h3,
    hotkey: hotkeyMap.h3.keyText,
    hotkeyVal: hotkeyMap.h3.keyValue,
    icon: '<span class="zacheditor-icon zacheditor-icon-h3" />',
    getActive: function getActive() {
      return engine.command.queryState('heading') === 'h3';
    },
    onClick: function onClick() {
      engine.command.execute('heading', 'h3');
    }
  }, {
    name: 'h4',
    title: headingMap.h4,
    hotkey: hotkeyMap.h4.keyText,
    hotkeyVal: hotkeyMap.h4.keyValue,
    icon: '<span class="zacheditor-icon zacheditor-icon-h4" />',
    getActive: function getActive() {
      return engine.command.queryState('heading') === 'h4';
    },
    onClick: function onClick() {
      engine.command.execute('heading', 'h4');
    }
  }, {
    name: 'h5',
    title: headingMap.h5,
    hotkey: hotkeyMap.h5.keyText,
    hotkeyVal: hotkeyMap.h5.keyValue,
    icon: '<span class="zacheditor-icon zacheditor-icon-h5" />',
    getActive: function getActive() {
      return engine.command.queryState('heading') === 'h5';
    },
    onClick: function onClick() {
      engine.command.execute('heading', 'h5');
    }
  }, {
    name: 'h6',
    title: headingMap.h6,
    hotkey: hotkeyMap.h6.keyText,
    hotkeyVal: hotkeyMap.h6.keyValue,
    icon: '<span class="zacheditor-icon zacheditor-icon-h6" />',
    getActive: function getActive() {
      return engine.command.queryState('heading') === 'h6';
    },
    onClick: function onClick() {
      engine.command.execute('heading', 'h6');
    }
  }, {
    name: 'bold',
    title: locale.bold.text,
    hotkey: hotkeyMap.bold.keyText,
    hotkeyVal: hotkeyMap.bold.keyValue,
    icon: '<span class="zacheditor-icon zacheditor-icon-bold" />',
    getActive: function getActive() {
      return engine.command.queryState('bold');
    },
    getDisabled: function getDisabled() {
      var tag = engine.command.queryState('heading') || 'p';
      var isCodeblock = engine.command.queryState('codeblock') === 'codeblock';
      return (/^h\d$/.test(tag) || isCodeblock
      );
    },
    onClick: function onClick() {
      engine.command.execute('bold');
    }
  }, {
    name: 'italic',
    title: locale.italic.text,
    hotkey: hotkeyMap.italic.keyText,
    hotkeyVal: hotkeyMap.italic.keyValue,
    icon: '<span class="zacheditor-icon zacheditor-icon-italic" />',
    getActive: function getActive() {
      return engine.command.queryState('italic');
    },
    onClick: function onClick() {
      engine.command.execute('italic');
    }
  }, {
    name: 'strikethrough',
    title: locale.strikethrough.text,
    hotkey: hotkeyMap.strikethrough.keyText,
    hotkeyVal: hotkeyMap.strikethrough.keyValue,
    icon: '<span class="zacheditor-icon zacheditor-icon-strikethrough" />',
    getActive: function getActive() {
      return engine.command.queryState('strikethrough');
    },
    onClick: function onClick() {
      engine.command.execute('strikethrough');
    }
  }, {
    name: 'underline',
    title: locale.underline.text,
    hotkey: hotkeyMap.underline.keyText,
    hotkeyVal: hotkeyMap.underline.keyValue,
    icon: '<span class="zacheditor-icon zacheditor-icon-underline" />',
    getActive: function getActive() {
      return engine.command.queryState('underline');
    },
    onClick: function onClick() {
      engine.command.execute('underline');
    }
  }, {
    name: 'fontsize',
    type: 'dropdown',
    title: locale.fontsize.buttonTitle,
    className: 'zacheditor-button-fontsize',
    data: fontnumberArray,
    getActive: function getActive() {
      return reverseFontnumberMap[engine.command.queryState('fontsize')] || '小四';
    },
    getDisabled: function getDisabled() {
      var tag = engine.command.queryState('heading') || 'p';
      var isCodeblock = engine.command.queryState('codeblock') === 'codeblock';
      return (/^h\d$/.test(tag) || isCodeblock
      );
    },
    getCurrentText: function getCurrentText(active) {
      return fontnumberKeyMap[active] || '小四';
    },
    onClick: function onClick(value) {
      engine.command.execute('fontsize', value);
    }
  }, {
    name: 'fontcolor',
    type: 'color',
    title: locale.color.fontColor,
    moreTitle: locale.color.moreColor,
    defaultColor: '#262626',
    currentColor: '#F5222D',
    getActive: function getActive() {
      return engine.command.queryState('fontcolor') || '';
    },
    onClick: function onClick(value) {
      engine.command.execute('fontcolor', value, '#262626');
    }
  }, {
    name: 'highlight',
    type: 'color',
    title: locale.color.bgColor,
    moreTitle: locale.color.moreColor,
    defaultColor: '#FFFFFF',
    currentColor: '#FADB14',
    getActive: function getActive() {
      return engine.command.queryState('backcolor') || '';
    },
    onClick: function onClick(value) {
      engine.command.execute('backcolor', value, '#FFFFFF');
    }
  }, {
    name: 'moremark',
    type: 'dropdown',
    title: locale.moremark.buttonTitle,
    icon: '<span class="zacheditor-icon zacheditor-icon-moremark" />',
    className: 'zacheditor-button-icon-list',
    data: [{
      key: 'sup',
      icon: '<span class="zacheditor-icon zacheditor-icon-sup" />',
      title: locale.sup.text,
      className: 'zacheditor-button-set-list-item-icon'
    }, {
      key: 'sub',
      icon: '<span class="zacheditor-icon zacheditor-icon-sub" />',
      title: locale.sub.text,
      className: 'zacheditor-button-set-list-item-icon'
    }, {
      key: 'code',
      icon: '<span class="zacheditor-icon zacheditor-icon-code" />',
      title: locale.code.text,
      className: 'zacheditor-button-set-list-item-icon'
    }],
    getActive: function getActive() {
      return engine.command.queryState('moremark');
    },
    onClick: function onClick(value) {
      engine.command.execute('moremark', value);
    }
  }, {
    name: 'sup',
    title: locale.sup.text,
    hotkey: hotkeyMap.sup.keyText,
    hotkeyVal: hotkeyMap.sup.keyValue,
    icon: '<span class="zacheditor-icon zacheditor-icon-sup" />',
    getActive: function getActive() {
      return engine.command.queryState('sup');
    },
    getDisabled: function getDisabled() {
      var tag = engine.command.queryState('heading') || 'p';
      return (/^h\d$/.test(tag)
      );
    },
    onClick: function onClick() {
      engine.command.execute('moremark', 'sup');
    }
  }, {
    name: 'sub',
    title: locale.sub.text,
    hotkey: hotkeyMap.sub.keyText,
    hotkeyVal: hotkeyMap.sub.keyValue,
    icon: '<span class="zacheditor-icon zacheditor-icon-sub" />',
    getActive: function getActive() {
      return engine.command.queryState('sub');
    },
    getDisabled: function getDisabled() {
      var tag = engine.command.queryState('heading') || 'p';
      return (/^h\d$/.test(tag)
      );
    },
    onClick: function onClick() {
      engine.command.execute('moremark', 'sub');
    }
  }, {
    name: 'code',
    title: locale.code.text,
    hotkey: hotkeyMap.code.keyText,
    hotkeyVal: hotkeyMap.code.keyValue,
    icon: '<span class="zacheditor-icon zacheditor-icon-code" />',
    getActive: function getActive() {
      return engine.command.queryState('code');
    },
    getDisabled: function getDisabled() {
      var tag = engine.command.queryState('heading') || 'p';
      return (/^h\d$/.test(tag)
      );
    },
    onClick: function onClick() {
      engine.command.execute('moremark', 'code');
    }
  }, {
    name: 'alignment',
    type: 'dropdown',
    title: locale.alignment.buttonTitle,
    icon: '<span class="zacheditor-icon zacheditor-icon-align-left" />',
    className: 'zacheditor-button-icon-list',
    data: [{
      key: 'left',
      icon: '<span class="zacheditor-icon zacheditor-icon-align-left" />',
      title: locale.alignment.alignLeft,
      className: 'zacheditor-button-set-list-item-icon',
      shortcut: hotkeyMap.left.keyText,
      hotkeyVal: hotkeyMap.left.keyValue
    }, {
      key: 'center',
      icon: '<span class="zacheditor-icon zacheditor-icon-align-center" />',
      title: locale.alignment.alignCenter,
      className: 'zacheditor-button-set-list-item-icon',
      shortcut: hotkeyMap.center.keyText,
      hotkeyVal: hotkeyMap.center.keyValue
    }, {
      key: 'right',
      icon: '<span class="zacheditor-icon zacheditor-icon-align-right" />',
      title: locale.alignment.alignRight,
      className: 'zacheditor-button-set-list-item-icon',
      shortcut: hotkeyMap.right.keyText,
      hotkeyVal: hotkeyMap.right.keyValue
    }, {
      key: 'justify',
      icon: '<span class="zacheditor-icon zacheditor-icon-align-justify" />',
      title: locale.alignment.alignJustify,
      className: 'zacheditor-button-set-list-item-icon',
      shortcut: hotkeyMap.justify.keyText,
      hotkeyVal: hotkeyMap.justify.keyValue
    }],
    getActive: function getActive() {
      return engine.command.queryState('alignment');
    },
    onClick: function onClick(value) {
      engine.command.execute('alignment', value);
    }
  }, {
    name: 'left',
    title: locale.alignment.alignLeft,
    hotkey: hotkeyMap.left.keyText,
    hotkeyVal: hotkeyMap.left.keyValue,
    icon: '<span class="zacheditor-icon zacheditor-icon-align-left" />',
    getActive: function getActive() {
      return engine.command.queryState('alignment') === 'left';
    },
    onClick: function onClick() {
      engine.command.execute('alignment', 'left');
    }
  }, {
    name: 'center',
    title: locale.alignment.alignCenter,
    hotkey: hotkeyMap.center.keyText,
    hotkeyVal: hotkeyMap.center.keyValue,
    icon: '<span class="zacheditor-icon zacheditor-icon-align-center" />',
    getActive: function getActive() {
      return engine.command.queryState('alignment') === 'center';
    },
    onClick: function onClick() {
      engine.command.execute('alignment', 'center');
    }
  }, {
    name: 'right',
    title: locale.alignment.alignRight,
    hotkey: hotkeyMap.right.keyText,
    hotkeyVal: hotkeyMap.right.keyValue,
    icon: '<span class="zacheditor-icon zacheditor-icon-align-right" />',
    getActive: function getActive() {
      return engine.command.queryState('alignment') === 'right';
    },
    onClick: function onClick() {
      engine.command.execute('alignment', 'right');
    }
  }, {
    name: 'justify',
    title: locale.alignment.alignJustify,
    hotkey: hotkeyMap.justify.keyText,
    hotkeyVal: hotkeyMap.justify.keyValue,
    icon: '<span class="zacheditor-icon zacheditor-icon-align-justify" />',
    getActive: function getActive() {
      return engine.command.queryState('alignment') === 'justify';
    },
    onClick: function onClick() {
      engine.command.execute('alignment', 'justify');
    }
  }, {
    name: 'list',
    type: 'dropdown',
    title: locale.list.buttonTitle,
    icon: '<span class="zacheditor-icon zacheditor-icon-ordered-list" />',
    className: 'zacheditor-button-icon-list',
    data: [{
      key: 'orderedlist',
      icon: '<span class="zacheditor-icon zacheditor-icon-ordered-list" />',
      title: locale.list.orderedList,
      className: 'zacheditor-button-set-list-item-icon'
    }, {
      key: 'unorderedlist',
      icon: '<span class="zacheditor-icon zacheditor-icon-unordered-list" />',
      title: locale.list.unorderedList,
      className: 'zacheditor-button-set-list-item-icon'
    }],
    getActive: function getActive() {
      return engine.command.queryState('tasklist');
    },
    onClick: function onClick(value) {
      engine.command.execute('tasklist', value);
    }
  }, {
    name: 'orderedlist',
    title: locale.list.orderedList,
    hotkey: hotkeyMap.orderedlist.keyText,
    hotkeyVal: hotkeyMap.orderedlist.keyValue,
    icon: '<span class="zacheditor-icon zacheditor-icon-ordered-list" />',
    getActive: function getActive() {
      return engine.command.queryState('tasklist') === 'orderedlist';
    },
    onClick: function onClick() {
      engine.command.execute('tasklist', 'orderedlist');
    }
  }, {
    name: 'unorderedlist',
    title: locale.list.unorderedList,
    hotkey: hotkeyMap.unorderedlist.keyText,
    hotkeyVal: hotkeyMap.unorderedlist.keyValue,
    icon: '<span class="zacheditor-icon zacheditor-icon-unordered-list" />',
    getActive: function getActive() {
      return engine.command.queryState('tasklist') === 'unorderedlist';
    },
    onClick: function onClick() {
      engine.command.execute('tasklist', 'unorderedlist');
    }
  }, {
    name: 'indent-list',
    type: 'dropdown',
    title: locale.indent.buttonTitle,
    icon: '<span class="zacheditor-icon zacheditor-icon-indent" />',
    className: 'zacheditor-button-icon-list',
    data: [{
      key: 'indent',
      icon: '<span class="zacheditor-icon zacheditor-icon-indent" />',
      title: locale.indent.increase,
      className: 'zacheditor-button-set-list-item-icon',
      shortcut: hotkeyMap.indent.keyText
    }, {
      key: 'outdent',
      icon: '<span class="zacheditor-icon zacheditor-icon-outdent" />',
      title: locale.indent.decrease,
      className: 'zacheditor-button-set-list-item-icon',
      shortcut: hotkeyMap.outdent.keyText
    }],
    onClick: function onClick(value) {
      engine.command.execute(value);
    }
  }, {
    name: 'indent',
    title: locale.indent.increase,
    icon: '<span class="zacheditor-icon zacheditor-icon-indent" />',
    onClick: function onClick() {
      engine.command.execute('indent');
    }
  }, {
    name: 'outdent',
    title: locale.indent.decrease,
    icon: '<span class="zacheditor-icon zacheditor-icon-outdent" />',
    onClick: function onClick() {
      engine.command.execute('outdent');
    }
  }, {
    name: 'tasklist',
    title: locale.list.taskList,
    icon: '<span class="zacheditor-icon zacheditor-icon-task-list" />',
    getActive: function getActive() {
      return engine.command.queryState('tasklist') === 'tasklist';
    },
    onClick: function onClick() {
      engine.command.execute('tasklist', 'tasklist');
    }
  }, {
    name: 'link',
    title: locale.link.buttonTitle,
    type: 'link',
    className: 'zacheditor-button-link',
    hotkey: hotkeyMap.link.keyText,
    hotkeyVal: hotkeyMap.link.keyValue,
    icon: '<span class="zacheditor-icon zacheditor-icon-link" />',
    onClick: function onClick() {
      engine.command.execute('link', '', '链接', true);
    }
  }, {
    name: 'codeblock',
    icon: '<span class="zacheditor-icon zacheditor-icon-codeblock" />',
    title: locale.section.codeblock,
    onClick: function onClick() {
      engine.command.execute('codeblock', '');
    }
  }, {
    name: 'table',
    icon: '<span class="zacheditor-icon zacheditor-icon-table" />',
    title: locale.section.table,
    onClick: function onClick(opts) {
      opts = opts !== undefined ? opts : {
        rows: 3,
        cols: 3
      };
      engine.command.execute('table', opts);
    }
  }, {
    name: 'math',
    icon: '<span class="zacheditor-icon zacheditor-icon-math" />',
    title: locale.section.math,
    onClick: function onClick() {
      engine.command.execute('math');
    }
  }, {
    name: 'image',
    type: 'upload',
    title: locale.image.buttonTitle,
    icon: '<span class="zacheditor-icon zacheditor-icon-image" />'
  }, {
    name: 'video',
    type: 'video',
    title: locale.section.video,
    icon: '<span class="zacheditor-icon zacheditor-icon-video" />',
    setOnline: function setOnline(value) {
      (0, _command.addIframeSection)(engine, 'youku', value);
    }
  }, {
    name: 'file',
    type: 'upload',
    title: locale.file.buttonTitle,
    icon: '<span class="zacheditor-icon zacheditor-icon-attachment" />'
  }, {
    name: 'quote',
    title: locale.blockquote.buttonTitle,
    icon: '<span class="zacheditor-icon zacheditor-icon-quote" />',
    getActive: function getActive() {
      return engine.command.queryState('quote');
    },
    onClick: function onClick() {
      engine.command.execute('quote');
    }
  }, {
    name: 'hr',
    title: locale.hr.buttonTitle,
    icon: '<span class="zacheditor-icon zacheditor-icon-hr" />',
    onClick: function onClick() {
      engine.command.execute('hr');
    }
  }, {
    name: 'divider',
    title: locale.divider.buttonTitle,
    icon: '<span class="zacheditor-svg-icon zacheditor-svg-icon-divider" />',
    onClick: function onClick() {
      engine.command.execute('divider');
    }
  }, {
    name: 'subdivider',
    title: locale.subdivider.buttonTitle,
    icon: '<span class="zacheditor-svg-icon zacheditor-svg-icon-subdivider" />',
    onClick: function onClick() {
      engine.command.execute('subdivider');
    }
  }, {
    name: 'toc',
    title: locale.toc.title,
    icon: '<span class="zacheditor-icon zacheditor-icon-toc" accessbilityid="toc-button" />',
    getActive: function getActive() {
      return engine.command.queryState('toc');
    },
    onClick: function onClick() {
      engine.command.execute('toc');
    }
  }, {
    name: 'search',
    title: locale.search.search + ' & ' + locale.search.replace,
    icon: '<span class="zacheditor-icon zacheditor-icon-searchreplace" accessbilityid="searchreplace-button" />',
    getActive: function getActive() {
      return engine.command.queryState('search');
    },
    onClick: function onClick() {
      engine.command.execute('search');
    }
  }, {
    name: 'translate',
    title: locale.translate.title,
    icon: '<span class="zacheditor-icon zacheditor-icon-translate" accessbilityid="translate-button" />',
    getActive: function getActive() {
      return engine.command.queryState('translate');
    },
    onClick: function onClick() {
      engine.command.execute('translate');
    }
  }, {
    name: 'label',
    icon: '<span class="zacheditor-icon zacheditor-icon-label" />',
    title: locale.section.label,
    onClick: function onClick() {
      engine.command.execute('label');
    }
  }, {
    name: 'question',
    icon: '<span class="zacheditor-svg-icon zacheditor-svg-icon-question" />',
    title: locale.blockquote.question,
    pinyin: 'xiaoti,question',
    description: locale.blockquote.questionDesc,
    getActive: function getActive() {
      return engine.command.queryState('quote');
    },
    onClick: function onClick() {
      engine.command.execute('quote', 'question');
    }
  }, {
    name: 'answer',
    icon: '<span class="zacheditor-svg-icon zacheditor-svg-icon-answer" />',
    title: locale.blockquote.answer,
    pinyin: 'daan,answer',
    description: locale.blockquote.answerDesc,
    getActive: function getActive() {
      return engine.command.queryState('quote');
    },
    onClick: function onClick() {
      engine.command.execute('quote', 'answer');
    }
  }, {
    name: 'analysis',
    icon: '<span class="zacheditor-svg-icon zacheditor-svg-icon-analysis" />',
    title: locale.blockquote.analysis,
    pinyin: 'jiexi,analysis',
    description: locale.blockquote.analysisDesc,
    getActive: function getActive() {
      return engine.command.queryState('quote');
    },
    onClick: function onClick() {
      engine.command.execute('quote', 'analysis');
    }
  }, {
    name: 'blankmark',
    icon: '<span class="zacheditor-svg-icon zacheditor-svg-icon-blankmark" />',
    title: locale.blankmark.title,
    pinyin: 'tiankongbiaoji,blankmark',
    description: locale.blankmark.description,
    onClick: function onClick() {
      engine.command.execute('blankmark');
    }
  }];
};
exports.default = getToolbarConfig;