'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _eventemitter = require('eventemitter2');

var _tinyCanvas = require('./tiny-canvas');

var _tinyCanvas2 = _interopRequireDefault(_tinyCanvas);

var _dom = require('../utils/dom');

var _type = require('../utils/type');

var _tinsoEditorEngine = require('tinso-editor-engine');

var _tinsoEditorEngine2 = _interopRequireDefault(_tinsoEditorEngine);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

/**
 * @fileOverview 查找 && 替换
 */

var Search = function (_EventEmitter) {
  _inherits(Search, _EventEmitter);

  function Search(options) {
    _classCallCheck(this, Search);

    var _this = _possibleConstructorReturn(this, (Search.__proto__ || Object.getPrototypeOf(Search)).call(this, options));

    _this._filter = function (e) {
      return !(e.getAttribute('data-section-key') && !_this.engine.section.getComponent(_tinsoEditorEngine2.default.$(e)).constructor.canSearch);
    };
    if (!options.target) throw new Error('please set a dom element as target!');
    _this.options = _extends({
      /**
       * dom element
       * @type {Object}
       */
      target: null,

      /**
       * style
       * @type {Object}
       */
      style: {
        highlight: {
          fill: 'rgba(255, 230, 0, 0.5)'
        },
        select: {
          fill: 'rgba(255, 140, 0, 0.5)'
        }
      },

      /**
       * filter
       * @type {Function|Null}
       */
      filter: null
    }, options);
    _this.ranges = []; // cache finded ranges

    _this._initTarget();

    _this._createCanvas();
    _this.engine = options.engine;
    return _this;
  }

  _createClass(Search, [{
    key: '_saveSectionValue',
    value: function _saveSectionValue(range) {
      var root = this.engine.section.closest(range.commonAncestorContainer);
      if (root) {
        this.engine.section.getComponent(root).saveValue();
      }
    }
  }, {
    key: '_initTarget',
    value: function _initTarget() {
      var target = this.options.target;
      if ((0, _dom.getStyle)(target, 'position') === 'static') {
        target.style.position = 'relative';
      }
    }
  }, {
    key: '_createCanvas',
    value: function _createCanvas() {
      var target = this.options.target;
      var container = document.createElement('div');
      container.style.position = 'absolute';
      container.style.top = 0;
      container.style.left = 0;
      container.style['pointer-events'] = 'none';
      target.appendChild(container);

      var canvas = new _tinyCanvas2.default({
        container: container
      });
      this.container = container;
      this.canvas = canvas;
    }
  }, {
    key: '_resizeCanvas',
    value: function _resizeCanvas() {
      var target = this.options.target;
      this.canvas.resize((0, _dom.getWidth)(target), (0, _dom.getHeight)(target));
    }
  }, {
    key: '_getRelativeRect',
    value: function _getRelativeRect(range) {
      var container = this.container;
      var rangeBoundingClientRect = range.getBoundingClientRect();
      var containerBoundingClientRect = container.getBoundingClientRect();
      return {
        x: rangeBoundingClientRect.left - containerBoundingClientRect.left,
        y: rangeBoundingClientRect.top - containerBoundingClientRect.top,
        width: rangeBoundingClientRect.right - rangeBoundingClientRect.left,
        height: rangeBoundingClientRect.bottom - rangeBoundingClientRect.top
      };
    }
  }, {
    key: '_cacheRange',
    value: function _cacheRange(range) {
      return {
        startOffset: range.startOffset,
        endOffset: range.endOffset,
        startContainer: range.startContainer,
        endContainer: range.endContainer,
        commonAncestorContainer: range.commonAncestorContainer
      };
    }
  }, {
    key: '_isRangeWrap',
    value: function _isRangeWrap(range) {
      var cache = this._cacheRange(range);

      range.setEnd(cache.startContainer, cache.startOffset + 1);
      var startBox = range.getBoundingClientRect();
      range.setEnd(cache.startContainer, cache.endOffset - 1);
      range.setStart(cache.endContainer, cache.endOffset);
      var endBox = range.getBoundingClientRect();
      range.setStart(cache.startContainer, cache.startOffset);
      range.setEnd(cache.endContainer, cache.endOffset);
      return startBox.bottom !== endBox.bottom;
    }
  }, {
    key: '_drawOneByOne',
    value: function _drawOneByOne(range, options) {
      var cache = this._cacheRange(range);
      var cursor = cache.startOffset;
      while (cursor < cache.endOffset) {
        range.setStart(cache.commonAncestorContainer, cursor);
        range.setEnd(cache.commonAncestorContainer, cursor + 1);

        var rect = this._getRelativeRect(range);
        this.canvas.clearRect(rect);
        this.canvas.draw('Rect', Object.assign({}, rect, options));
        cursor++;
      }

      range.setStart(cache.startContainer, cache.startOffset);
      range.setEnd(cache.endContainer, cache.endOffset);
    }
  }, {
    key: '_drawRangeBackground',
    value: function _drawRangeBackground(range, _ref) {
      var isWrap = this._isRangeWrap(range);
      if (isWrap) return this._drawOneByOne(range, _ref);
      var rect = this._getRelativeRect(range);
      this.canvas.clearRect(rect);
      this.canvas.draw('Rect', Object.assign({}, rect, _ref));
    }
  }, {
    key: '_highlightRanges',
    value: function _highlightRanges(ranges) {
      var _this2 = this;

      ranges.forEach(function (range) {
        _this2._highlightRange(range);
      });
    }
  }, {
    key: '_clearRange',
    value: function _clearRange(range) {
      var rect = this._getRelativeRect(range);
      this.canvas.clearRect(rect);
    }
  }, {
    key: '_highlightRange',
    value: function _highlightRange(range) {
      var highlight = this.options.style.highlight;
      this._drawRangeBackground(range, highlight);
    }
  }, {
    key: '_selectRange',
    value: function _selectRange(range) {
      var select = this.options.style.select;
      this._drawRangeBackground(range, select);
    }
  }, {
    key: '_clearCanvas',
    value: function _clearCanvas() {
      this.canvas.clear();
    }
  }, {
    key: '_move',
    value: function _move(offset) {
      var selectedRange = this.selectedRange,
          ranges = this.ranges;


      if (!selectedRange) {
        return;
      }

      var selectedRangeIndex = ranges.indexOf(selectedRange);
      var nextRange = ranges[selectedRangeIndex + offset];
      if (!nextRange) {
        if (offset > 0) {
          nextRange = ranges[offset - 1];
        } else {
          nextRange = ranges[ranges.length + offset];
        }
      }
      this._highlightRange(selectedRange);
      this.select(nextRange);
      this.scrollIntoView();
    }
  }, {
    key: '_replace',
    value: function _replace(range, replacement) {
      var node = range.commonAncestorContainer;
      var parentNode = node.parentNode;
      var textNode = document.createTextNode(replacement);

      if ('A' === parentNode.tagName) {
        var href = parentNode.getAttribute('href');
        var text = parentNode.innerText;
        if (href === text) parentNode.setAttribute('href', replacement);
      }
      range.deleteContents();
      range.insertNode(textNode);
    }
  }, {
    key: 'replace',
    value: function replace(replacement) {
      if ((0, _type.isNil)(replacement)) {
        return;
      }

      var selectedRange = this.selectedRange,
          ranges = this.ranges;


      if (ranges.length > 0) {
        var evObj = {
          range: selectedRange
        };
        this.emit('before:replace', evObj);
        this._replace(selectedRange, replacement);
        this._clearRange(selectedRange);
        this.emit('after:replace', evObj);
        this._saveSectionValue(selectedRange);
        this.engine.history.save();
      }
    }
  }, {
    key: 'replaceAll',
    value: function replaceAll(replacement) {
      var _this3 = this;

      if ((0, _type.isNil)(replacement)) {
        return;
      }

      var ranges = this.ranges;
      var evObj = {
        ranges: ranges
      };
      this.emit('before:replaceAll', evObj);
      ranges.forEach(function (range) {
        _this3._replace(range, replacement);
      });
      this.emit('after:replaceAll', evObj);
      ranges.forEach(function (range) {
        _this3._saveSectionValue(range);
      });
      this.engine.history.save();
      this.clear();
    }
  }, {
    key: 'show',
    value: function show() {
      this.container.style.display = '';
    }
  }, {
    key: 'hide',
    value: function hide() {
      this.container.style.display = 'none';
    }

    /**
     * set aim text in target dom
     * @param  {String} keyword - search key word
     * @param  {String} filter - filter need search child element
     * @return {Array} ranges
     */

  }, {
    key: 'find',
    value: function find(keyword, filter) {
      var ranges = [];
      this.clear();

      if (keyword && (0, _type.isString)(keyword) && keyword.length > 0) {
        filter = (0, _type.isFunction)(filter) ? filter : this._filter;

        this._resizeCanvas();
        var target = this.options.target;
        var textNodes = (0, _dom.getTextNodes)(target, filter);

        if (textNodes.length > 0) {
          ranges = (0, _dom.getRangesByText)(textNodes, keyword);
          this._highlightRanges(ranges);
          this.ranges = ranges;
          this.keyword = keyword;
        }
      }

      this.ranges = ranges;
      return ranges;
    }
  }, {
    key: 'reFind',
    value: function reFind() {
      var selectedRangeIndex = this.selectedRangeIndex;
      var ranges = this.find(this.keyword);

      if (ranges.length > 0) {
        if (!(0, _type.isNil)(selectedRangeIndex)) {
          var range = ranges[selectedRangeIndex] ? ranges[selectedRangeIndex] : ranges[0];
          this.select(range);
        }
      }
      return ranges;
    }
  }, {
    key: 'scrollIntoView',
    value: function scrollIntoView() {
      if (this.selectedRange) {
        var element = this.selectedRange.commonAncestorContainer.parentElement;
        if (!(0, _dom.isInViewport)(element)) element.scrollIntoView({
          block: 'center'
        });
      }
    }
  }, {
    key: 'select',
    value: function select(param) {
      if ((0, _type.isNil)(param)) {
        return;
      }
      var ranges = this.ranges;
      var range = param;

      if ((0, _type.isNumber)(param)) {
        range = ranges[param];
      }

      var evObj = {
        range: range
      };
      this.emit('before:select', evObj);
      this._selectRange(range);
      this.selectedRange = range;
      this.selectedRangeIndex = ranges.indexOf(range);
      this.emit('after:select', evObj);
    }
  }, {
    key: 'next',
    value: function next() {
      this._move(1);
    }
  }, {
    key: 'prev',
    value: function prev() {
      this._move(-1);
    }
  }, {
    key: 'clear',
    value: function clear() {
      this._clearCanvas();
      this.selectedRange = undefined;
      this.ranges = [];
      this.keyword = undefined;
    }
  }, {
    key: 'count',
    value: function count() {
      var ranges = this.ranges;
      if (ranges.length === 0) {
        return 0;
      }
      var selectedRange = this.selectedRange;
      return [ranges.indexOf(selectedRange) + 1, ranges.length];
    }
  }, {
    key: 'destroy',
    value: function destroy() {
      this.options = null;
      this.container.parentElement.removeChild(this.container);
    }
  }]);

  return Search;
}(_eventemitter.EventEmitter2);

exports.default = Search;