"use strict";

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var TinyCanvas = function () {
    function TinyCanvas(config) {
        _classCallCheck(this, TinyCanvas);

        if (!config.container) throw new Error("please set a dom cantainer!");
        this.options = Object.assign({
            container: null,
            limitHeight: 5e3,
            canvasCache: [],
            canvasCount: 0
        }, config);
        config.container.style["line-height"] = "0px";
    }

    _createClass(TinyCanvas, [{
        key: "_removeCanvas",
        value: function _removeCanvas() {
            this.options.canvasCache.forEach(function (item) {
                item.parentElement.removeChild(item);
            });
            this.options.canvasCache = [];
        }
    }, {
        key: "_getCanvas",
        value: function _getCanvas(index) {
            index = index > 0 ? index - 1 : index;
            return this.options.canvasCache[index];
        }
    }, {
        key: "resize",
        value: function resize(width, height) {
            var _options = this.options,
                limitHeight = _options.limitHeight,
                canvasCount = _options.canvasCount,
                container = _options.container,
                canvasCache = _options.canvasCache;

            var count = Math.ceil(height / limitHeight);
            if (count !== canvasCount) {
                this._removeCanvas();
                canvasCache = [];
                for (var l = 0; l < count; l++) {
                    var canvas = document.createElement("canvas");
                    canvas.style["vertical-align"] = "bottom";
                    canvas.setAttribute("width", width);
                    if (l === count - 1) {
                        canvas.setAttribute("height", height % limitHeight);
                    } else {
                        canvas.setAttribute("height", limitHeight);
                    }
                    container.appendChild(canvas);
                    canvasCache.push(canvas);
                }
                this.options.canvasCache = canvasCache;
            } else {
                var lastCanvas = this._getCanvas(count);
                canvasCache.forEach(function (canvas) {
                    canvas.setAttribute("width", width);
                });
                if (lastCanvas) {
                    lastCanvas.setAttribute("height", height % limitHeight);
                }
            }
        }
    }, {
        key: "_handleSingleRect",
        value: function _handleSingleRect(options) {
            var x = options.x,
                y = options.y,
                index = options.index,
                width = options.width,
                height = options.height,
                callback = options.callback;
            var limitHeight = this.options.limitHeight;

            var canvas = this._getCanvas(index);
            if (canvas) {
                var context = canvas.getContext("2d");
                callback({
                    x: x,
                    y: y - limitHeight * (index - 1),
                    width: width,
                    height: height,
                    context: context
                });
            }
        }
    }, {
        key: "_drawRect",
        value: function _drawRect(options) {
            var x = options.x,
                y = options.y,
                fill = options.fill,
                width = options.width,
                height = options.height,
                stroke = options.stroke;

            var callback = function callback(opt) {
                opt.context.fillStyle = fill === undefined ? "#FFEC3D" : fill;
                opt.context.strokeStyle = stroke === undefined ? "#FFEC3D" : stroke;
                opt.context.fillRect(opt.x, opt.y, opt.width, opt.height);
            };
            this._handleRect({
                x: x,
                y: y,
                width: width,
                height: height,
                callback: callback
            });
        }
    }, {
        key: "_handleRect",
        value: function _handleRect(options) {
            var x = options.x,
                y = options.y,
                width = options.width,
                height = options.height,
                callback = options.callback;
            var limitHeight = this.options.limitHeight;

            var min = {
                x: x,
                y: y
            };
            var max = {
                x: x + width,
                y: y + height
            };
            var minCount = Math.ceil(min.y / limitHeight);
            var maxCount = Math.ceil(max.y / limitHeight);
            if (minCount === maxCount) {
                this._handleSingleRect(Object.assign({}, min, {
                    index: minCount,
                    width: width,
                    height: height,
                    callback: callback
                }));
            } else {
                this._handleSingleRect(Object.assign({}, min, {
                    index: minCount,
                    width: width,
                    height: height,
                    callback: callback
                }));

                this._handleSingleRect(Object.assign({}, min, {
                    index: maxCount,
                    width: width,
                    height: height,
                    callback: callback
                }));
            }
        }
    }, {
        key: "getImageData",
        value: function getImageData(options) {
            var x = options.x,
                y = options.y,
                width = options.width,
                height = options.height;
            var limitHeight = this.options.limitHeight;

            var count = Math.ceil(y / limitHeight);
            var lastCanvas = this._getCanvas(count);
            var context = lastCanvas.getContext("2d");
            return context.getImageData(x, y, width, height);
        }
    }, {
        key: "draw",
        value: function draw(name, options) {
            this["_draw" + name](options);
        }
    }, {
        key: "clearRect",
        value: function clearRect(options) {
            var x = options.x,
                y = options.y,
                width = options.width,
                height = options.height;

            var callback = function callback(opt) {
                opt.context.clearRect(opt.x, opt.y, opt.width, opt.height);
            };
            this._handleRect({
                x: x,
                y: y,
                width: width,
                height: height,
                callback: callback
            });
        }
    }, {
        key: "clear",
        value: function clear() {
            this.options.canvasCache.forEach(function (canvas) {
                var context = canvas.getContext("2d");
                var width = Number(canvas.getAttribute("width"));
                var height = Number(canvas.getAttribute("height"));
                context.clearRect(0, 0, width, height);
            });
        }
    }, {
        key: "destroy",
        value: function destroy() {
            this._removeCanvas();
        }
    }]);

    return TinyCanvas;
}();

exports.default = TinyCanvas;