'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _eventemitter = require('eventemitter2');

var _tinsoEditorEngine = require('tinso-editor-engine');

var _tinsoEditorEngine2 = _interopRequireDefault(_tinsoEditorEngine);

var _ajax = require('../utils/ajax');

var _ajax2 = _interopRequireDefault(_ajax);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var NodeUtils = _tinsoEditorEngine2.default.NodeUtils,
    $ = _tinsoEditorEngine2.default.$;

var Translate = function (_EventEmitter) {
  _inherits(Translate, _EventEmitter);

  function Translate(_ref2) {
    var engine = _ref2.engine;

    _classCallCheck(this, Translate);

    var _this = _possibleConstructorReturn(this, (Translate.__proto__ || Object.getPrototypeOf(Translate)).call(this));

    _this.engine = engine;
    return _this;
  }

  // 获取能保持段落格式的最小段落元素
  // <p></p> : <p></p>
  // <ul><li></li></ul> : <li></li>


  _createClass(Translate, [{
    key: '_getMiniParagraphElements',
    value: function _getMiniParagraphElements(childNodes) {
      var rst = [];
      childNodes.forEach(function (p) {
        if (p.tagName === 'UL') {
          p.childNodes.forEach(function (li) {
            rst.push(li);
          });
        } else {
          rst.push(p);
        }
      });
      return rst;
    }
  }, {
    key: '_getSourceText',
    value: function _getSourceText(_ref) {
      var childNodes = _ref.childNodes,
          isCrossRow = _ref.isCrossRow;

      var rst = '';

      if (isCrossRow) {
        var paragraphElements = this._getMiniParagraphElements(childNodes);
        paragraphElements.forEach(function (node) {
          rst += node.innerText + '\n';
        });
      } else {
        var div = document.createElement('div');
        childNodes.forEach(function (node) {
          div.append(node);
        });
        rst = div.innerText;
      }
      return rst;
    }
  }, {
    key: '_translateFragment',
    value: function _translateFragment(_ref) {
      var isCrossRow = _ref.isCrossRow,
          targetPlainText = _ref.targetPlainText,
          fragment = _ref.fragment;


      if (isCrossRow) {
        var textSplit = targetPlainText.split('\n');
        var paragraphElements = this._getMiniParagraphElements(fragment.childNodes);

        paragraphElements.forEach(function (node, index) {
          node.innerHTML = _tinsoEditorEngine2.default.StringUtils.escape(textSplit[index]);
        });
      } else {
        fragment.append(document.createTextNode(targetPlainText));
      }
    }
  }, {
    key: '_getChildNodes',
    value: function _getChildNodes(fragment) {
      var _this2 = this;

      var childNodes = Array.from(fragment.childNodes).filter(function (node) {
        // 只取非Section内的元素
        return !_this2.engine.section.closest(node);
      });
      childNodes.forEach(function (node) {
        // 删除里面的Section元素
        NodeUtils.walkTree(node, function (sub) {
          if (_this2.engine.section.closest(sub)) {
            sub.parentNode.removeChild(sub);
          }
        });
      });
      return childNodes;
    }
  }, {
    key: 'translate',
    value: function translate(range, source, target) {
      var _this3 = this;

      if (!range || !source || !target || this.engine.section.closest(range.commonAncestorContainer) || // 在Section里的内容不翻译
      $(range.commonAncestorContainer).closest('.sub-editor-content')[0] // TODO：在表格子编辑器的内容暂时不翻译
      ) {
          return;
        }

      var fragment = range.cloneContents();
      var isCrossRow = $(range.commonAncestorContainer).isRoot() || range.commonAncestorContainer.tagName === 'UL';
      var childNodes = this._getChildNodes(fragment);
      var q = this._getSourceText({
        childNodes: childNodes,
        isCrossRow: isCrossRow
      });

      if (!q || q === '\n\n' || q === ' ') {
        // 双击行尾会触发换行，过滤双换行符
        return;
      }

      var options = this.engine.options.translate;
      (0, _ajax2.default)({
        url: options.action,
        dataType: 'json',
        data: {
          text: q,
          source: source,
          target: target
        },
        success: function success(res) {
          if (res.result) {
            var targetPlainText = res.data;
            _this3._translateFragment({
              isCrossRow: isCrossRow,
              targetPlainText: targetPlainText,
              fragment: fragment
            });

            _this3.emit('success', {
              targetPlainText: targetPlainText,
              // 翻文
              fragment: fragment
            });
          } else {
            _this3.emit('fail', {
              errorMsg: res.message
            });
          }
        },
        error: function error(err) {
          _this3.emit('fail', {
            errorMsg: err.message
          });
        },
        method: 'POST'
      });
    }
  }]);

  return Translate;
}(_eventemitter.EventEmitter2);

exports.default = Translate;