'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _tinsoEditorEngine = require('tinso-editor-engine');

var _tinsoEditorEngine2 = _interopRequireDefault(_tinsoEditorEngine);

var _codeblock = require('../section/codeblock');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var ChangeUtils = _tinsoEditorEngine2.default.ChangeUtils,
    NodeUtils = _tinsoEditorEngine2.default.NodeUtils,
    StringUtils = _tinsoEditorEngine2.default.StringUtils,
    HTMLParser = _tinsoEditorEngine2.default.HTMLParser,
    $ = _tinsoEditorEngine2.default.$;

// 缩写替换

var INPUT_MODE_ALIAS = {
  text: 'plain',
  sh: 'bash',
  ts: 'typescript',
  js: 'javascript',
  py: 'python',
  puml: 'plantuml',
  uml: 'plantuml',
  vb: 'basic',
  md: 'markdown',
  'c++': 'cpp'
};

var PLUGIN_NAME = 'codeblock';
exports.default = {
  initialize: function initialize() {
    var _this = this;

    // 创建命令
    this.command.add(PLUGIN_NAME, {
      queryState: function queryState() {
        var _change = _this.change,
            section = _change.section,
            activeSection = _change.activeSection;

        if (section && activeSection) {
          return section.getName(activeSection);
        }
        return '';
      },
      execute: function execute(mode, code) {
        var sectionRoot = _this.change.insertSection(PLUGIN_NAME, {
          mode: mode,
          code: code
        });
        _this.change.focusSection(sectionRoot);
        var component = _this.section.getComponent(sectionRoot);
        window.setTimeout(function () {
          component.focusToCodeEditor();
        }, 10);
      }
    });
    // Markdown 快捷键
    this.on('keydown:enter', function (e) {
      var range = _this.change.getRange();
      if (!range.collapsed) {
        return;
      }
      var block = ChangeUtils.getClosestBlock(range.startContainer);
      if (!block.isHeading()) {
        return;
      }
      var options = _this.options.markdown || {
        items: ['codeblcok']
      };

      if (options.items.indexOf('codeblcok') < 0) return;

      var chars = ChangeUtils.getBlockLeftText(block, range);
      var match = /^```(.*){0,10}$/.exec(chars);

      if (match) {
        var mode = (undefined === match[1] ? '' : match[1]).toLowerCase();
        mode = INPUT_MODE_ALIAS[mode] || mode;

        if (_codeblock.MODE_NAME_MAP[mode] || mode === '') {
          e.preventDefault();
          ChangeUtils.removeBlockLeftText(block, range);
          _this.command.execute(PLUGIN_NAME, mode, '');
          block.remove();
          return false;
        }
      }
    });

    // 转换页面粘贴过来的代码块
    this.on('paste:origin', function (root) {
      NodeUtils.walkTree(root, function (node) {
        node = $(node);
        // 编辑页面
        if (node.isBlock() && node.hasClass('bi-code')) {
          var preNode = node.find('pre');
          if (preNode.length > 0) {
            var match = /^bi-code bi-code-(\w+)/.exec(node[0].className);
            var mode = match ? match[1] : '';
            var code = new HTMLParser(preNode).toText();
            code = StringUtils.unescape(code);
            _this.section.replaceNode(node, PLUGIN_NAME, {
              mode: mode,
              code: code
            });
            return;
          }
        }
      });
    });
    // 转换粘贴过来的代码块
    this.on('paste:schema', function (schema) {
      schema.add({
        pre: {
          'data-syntax': '*'
        }
      });
    });

    this.on('paste:each', function (node) {
      // 阅读页面
      if (node.name === 'pre' && node.attr('data-syntax')) {
        var code = new HTMLParser(node).toText();
        code = StringUtils.unescape(code);

        _this.section.replaceNode(node, PLUGIN_NAME, {
          mode: node.attr('data-syntax'),
          code: code
        });

        return;
      }
      // 其它页面
      if (node.name === 'pre' && node.first() && node.first().name === 'code') {
        var _code = new HTMLParser(node).toText();
        _code = StringUtils.unescape(_code);
        _this.section.replaceNode(node, PLUGIN_NAME, {
          mode: 'plain',
          code: _code
        });
      }
    });

    // 快捷键
    var options = this.options[PLUGIN_NAME] || {
      hotkey: 'mod+shift+c'
    };

    if (!!options.hotkey) {
      this.hotkey.set(options.hotkey, PLUGIN_NAME);
    }
  }
};