'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _engine = require('../editor/engine');

var _engine2 = _interopRequireDefault(_engine);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

exports.default = {
    initialize: function initialize() {
        var _this = this;

        // 拖拽上传文件
        this.on('drop:files', function (files) {
            if (_this.uploader.post("file", files)) return false;
        });

        // 转换页面粘贴过来的文件
        this.on('paste:origin', function (root) {
            _engine2.default.NodeUtils.walkTree(root, function (node) {
                node = _engine2.default.$(node);
                // 编辑页面
                if (node.isBlock() && node.hasClass('bi-file')) {
                    var fileNode = node.find('.bi-file-content');
                    if (fileNode.length > 0) {
                        var src = fileNode.attr('data-src') || '';
                        var name = fileNode.attr('data-name');
                        var ext = src.split('.').pop();
                        var sizeNode = fileNode.find('.size');
                        if (!src || !ext || sizeNode.length === 0) {
                            return;
                        }
                        var size = sizeNode.text().replace(/^.+?([\d\.]+)\s*(kb|mb).*$/i, function (match0, size, unit) {
                            // 注意：这里必须用 1000，不能用 1024
                            if (unit.toLowerCase() === 'kb') {
                                return size * 1000;
                            }

                            if (unit.toLowerCase() === 'mb') {
                                return size * 1000 * 1000;
                            }
                        });
                        var value = {
                            src: src,
                            name: name,
                            size: parseFloat(size) || 0,
                            ext: ext
                        };
                        _this.section.replaceNode(node, 'file', value);
                        return;
                    }
                }
            });
        });
        this.on('paste:schema', function (schema) {
            schema.add([{
                p: {
                    'data-type': '*',
                    'data-filetype': '*',
                    'data-name': '*',
                    'data-size': '*',
                    'data-src': '*'
                }
            }]);
        });
        this.on('paste:each', function (node) {
            // 阅读页面
            if (node.attr('data-type') === 'file') {
                var src = node.attr('data-src');
                if (src) {
                    var value = {
                        src: src,
                        name: node.attr('data-name'),
                        size: parseInt(node.attr('data-size'), 10) || 0,
                        ext: node.attr('data-filetype')
                    };

                    _this.section.replaceNode(node, 'file', value);
                    return;
                }
            }
        });
    }
};