'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.handleKeydownEnter = undefined;

var _string = require('../../utils/string');

var _engine = require('../../editor/engine');

var _engine2 = _interopRequireDefault(_engine);

var _request = require('../../utils/request');

var _compress = require('./compress');

var _compress2 = _interopRequireDefault(_compress);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// 上传图片 URL
function uploadUrl(sectionRoot, url) {
  var _this = this;

  var locale = this.locale.image || {};
  var imageConfig = this.options.image || {};
  var uploadAction = imageConfig.action || '';
  (0, _request.post)(uploadAction, {
    url: url
  }).then(function (res) {
    var data = res.data;
    var params = {
      status: 'done',
      src: data.url
    };
    // 如果返回了大小，也更新一下
    if (data.size) {
      params.size = data.size;
    }

    if (data.width) {
      params.width = data.width;
    }

    if (data.height) {
      params.height = data.height;
    }

    _engine2.default.UploadUtils.updateSection(_this, sectionRoot, params, true);
  }).catch(function () {
    _engine2.default.UploadUtils.updateSection(_this, sectionRoot, {
      status: 'error',
      message: locale.copyFailedTips
    }, true);
  });
}
// 自己页面
function fromSelfPage(node) {
  var value = this.section.getValue(node);
  var src = value.src || '';

  if (!(0, _string.isRemoteImage)(src)) {
    if ('uploading' === value.status) {
      delete value.status;
      this.section.setValue(node, value);
    }
    return;
  }

  value.status = 'uploading';
  value.percent = 0;
  this.section.replaceNode(node, 'image', value);
}
// 查看页面
function fromView(node) {
  var linkNode = void 0;
  var linkUrl = void 0;

  if (node.name === 'a') {
    linkUrl = node.attr('href');
    linkNode = node;
    node = linkNode.find('[data-type=image]');
  }

  var img = node.find('img');
  var src = img.attr('src');
  var width = node.attr('data-width') || img.css('width') || img.attr('width') || '';
  var height = node.attr('data-height') || img.css('height') || img.attr('height') || '';

  if (!/%/.test(width)) {
    width = parseInt(width, 10);
  }

  if (!/%/.test(height)) {
    height = parseInt(height, 10);
  }

  var display = node.attr('data-display');
  var align = node.attr('data-align');
  var value = {
    src: src,
    display: display,
    align: align
  };

  if (width) {
    value.width = width;
  }

  if (height) {
    value.height = height;
  }

  if (linkUrl !== undefined) {
    value.link = linkUrl;
  }

  if ((0, _string.isBase64Image)(src) || (0, _string.isRemoteImage)(src)) {
    value.status = 'uploading';
    value.percent = 0;
  }
  this.section.replaceNode(linkNode || node, 'image', value);
}
// 普通页面
function fromOtherPage(node) {
  var linkNode = void 0;
  var linkUrl = void 0;

  if (node.name === 'a') {
    linkUrl = node.attr('href');
    linkNode = node;
    node = linkNode.find('img');
  }

  var src = node.attr('src');
  var width = node.css('width') || node.attr('width') || '';
  var height = node.css('height') || node.attr('height') || '';

  if (!/%/.test(width)) {
    width = parseInt(width, 10);
  }

  if (!/%/.test(height)) {
    height = parseInt(height, 10);
  }

  var value = {
    src: src
  };

  if (width) {
    value.width = width;
  }

  if (height) {
    value.height = height;
  }

  if (linkUrl !== undefined) {
    value.link = linkUrl;
  }

  if ((0, _string.isBase64Image)(src) || (0, _string.isRemoteImage)(src)) {
    value.status = 'uploading';
    value.percent = 0;
  }
  this.section.replaceNode(linkNode || node, 'image', value);
}

function handleKeydownEnter(e) {
  var range = this.change.getRange();
  if (!range.collapsed) {
    return;
  }
  var block = _engine2.default.ChangeUtils.getClosestBlock(range.startContainer);
  if (block.name !== 'p') {
    return;
  }
  var chars = _engine2.default.ChangeUtils.getBlockLeftText(block, range); // ![](url)
  // ![name|center|250x200](url)
  // ![|center](url)

  var match = /^!\[([^\]]{0,})\]\(([http|https]?:\/\/[^\)]{5,})\)$/.exec(chars);
  if (!match) return;
  e.preventDefault();
  _engine2.default.ChangeUtils.removeBlockLeftText(block, range);
  this.history.save();
  var splits = match[1].split('|');
  var src = match[2];
  var value = {
    src: src,
    name: splits[0],
    progress: {
      percent: 0
    }
  };
  var alignment = splits[1];

  if (alignment === 'center' || alignment === 'right') {
    this.command.execute('alignment', alignment);
  }
  if (!(0, _string.isRemoteImage)(src)) {
    value.status = 'done';
    delete value.progress;
    this.change.insertSection('image', value);
    return;
  }
  var sectionRoot = _engine2.default.UploadUtils.insertSection(this, 'image', value);
  uploadUrl.call(this, sectionRoot, src);
}

exports.handleKeydownEnter = handleKeydownEnter;

var imageCompress = new _compress2.default({
  isOverSize: function isOverSize(file) {
    var ext = 'string' == typeof file.ext ? file.ext.toLowerCase() : undefined;
    return ('jpg' === ext || 'jpeg' === ext) && !_engine2.default.UploadUtils.isSizeLimit('image', file.size);
  }
});
exports.default = {
  initialize: function initialize() {
    var _this2 = this;

    // 拖拽上传图片
    this.on('drop:files', function (files) {
      if (_this2.uploader.post('image', files)) return false;
    });
    // 上传粘贴板里的图片
    this.on('paste:files', function (files) {
      imageCompress.compress(files).then(function (files) {
        _this2.uploader.post('image', files, {
          onBeforeInsertSection: function onBeforeInsertSection(file, options) {
            if (!file.ext) {
              options.scaleByPixelRatio = true;
            }
          }
        });
      });
    });

    // 转换粘贴过来的图片
    this.on('paste:schema', function (schema) {
      schema.add({
        p: {
          'data-type': '*',
          'data-src': '*',
          'data-align': '*',
          'data-display': '*',
          'data-width': '*',
          'data-height': '*'
        }
      });
    });
    this.on('paste:each', function (node) {
      // 自己页面
      if (node.isSection() && node.attr('data-ready-section') === 'image') {
        fromSelfPage.call(_this2, node);
        return;
      }
      // 阅读页面
      if (node.attr('data-type') === 'image' && node.find('img').length > 0) {
        fromView.call(_this2, node);
        return;
      }
      // 阅读页面，带链接
      if (node.name === 'a' && node.find('[data-type=image]').length > 0 && node.find('img').length > 0) {
        fromView.call(_this2, node);
        return;
      }
      // 外部页面，带链接
      if (node.name === 'a' && node.find('img').length > 0) {
        fromOtherPage.call(_this2, node);
        return;
      }
      // 外部页面，纯图片
      if (node.name === 'img' && node.attr('src')) {
        fromOtherPage.call(_this2, node);
      }
    });
    this.on('paste:after', function () {
      // 转存 base64 图片、第三方图片
      var options = _this2.options['image'] || {};
      _this2.container.find('[data-section-key=image]').each(function (node, key) {
        var sectionRoot = _engine2.default.$(node);
        var component = _this2.section.getComponent(sectionRoot);
        var value = component.value;
        if (value.status !== 'uploading') {
          return;
        }

        var src = component.imageNode.src || '';
        // 转存 base64 图片

        if ((0, _string.isBase64Image)(src)) {
          var file = _engine2.default.ImageUtils.dataURIToFile(src);
          var ext = _engine2.default.UploadUtils.getFileExtname(file);
          var name = ext ? 'image.'.concat(ext) : 'image';
          var fileObject = new File([file], name);
          component.value = {
            status: 'uploading',
            src: src,
            name: name,
            size: file.size,
            type: file.type,
            percent: 0
          };
          _this2.section.updateNode(sectionRoot, component);
          fileObject.uid = _this2.uploader.getUid(key);
          _this2.uploader.addUploadingSection(fileObject.uid, sectionRoot);
          _this2.uploader.post('image', [fileObject]);
          return;
        }
        // 转存第三方图片
        if (options.domain) {
          options.domain.forEach(function (domain) {
            if (src.indexOf(domain) > -1) {
              return;
            }
          });
        }
        if ((0, _string.isRemoteImage)(src)) {
          _this2.section.updateNode(sectionRoot, component);
          uploadUrl.call(_this2, sectionRoot, src);
        }
      });
    });
    // markdown 快捷方式
    this.on('keydown:enter', function (e) {
      handleKeydownEnter.call(_this2, e);
    });
  }
};