'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _reactDom = require('react-dom');

var _reactDom2 = _interopRequireDefault(_reactDom);

var _antd = require('antd');

var _engine = require('../../editor/engine');

var _engine2 = _interopRequireDefault(_engine);

var _range = require('tinso-editor-engine/lib/utils/range');

var _string = require('tinso-editor-engine/lib/utils/string');

var _editor = require('./editor');

var _editor2 = _interopRequireDefault(_editor);

require('./index.css');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var PLUGIN_NAME = 'link';
var TAG_NAME = 'a';

function shouldTargetBlank(href) {
  return href && ['http:', 'https:', 'data:', 'ftp:'].some(function (protocol) {
    return href.startsWith(protocol);
  });
}

function getLink(linkUrl) {
  var _window = window,
      location = _window.location;

  var url = (0, _string.escapeRegExp)(location.origin + location.pathname.replace('/edit', ''));
  var reg = RegExp('^'.concat(url, '#'));
  linkUrl.replace(reg, '#').trim();
  if (/^(\w+\.)+\w+(\/|$)/.test(linkUrl)) {
    linkUrl = 'http://' + linkUrl;
  }
  return linkUrl;
}

function go(node) {
  var _this = this;

  var type = arguments.length > 1 && void 0 !== arguments[1] ? arguments[1] : 'preview';
  var isRmove = arguments.length > 2 && void 0 !== arguments[2] && arguments[2];
  var container = document.createElement('div');
  var removeLink = function removeLink() {
    _engine2.default.$(node).each(function (child) {
      var range = _this.change.getRange();
      range.selectNode(child);
      _this.change.select(range);
      _this.change.unwrapInline('<a />');
    });
  };
  var _document = document,
      body = _document.body;

  body.appendChild(container);
  _reactDom2.default.render(_react2.default.createElement(
    _antd.ConfigProvider,
    { autoInsertSpaceInButton: false },
    _react2.default.createElement(_editor2.default, {
      target: node,
      shouldTargetBlank: shouldTargetBlank,
      getLink: getLink,
      type: type,
      container: container,
      onDeleteLink: function onDeleteLink() {
        _this.history.save();
        removeLink();
        _reactDom2.default.unmountComponentAtNode(container);
        body.removeChild(container);
      },
      onConfirm: function onConfirm(text, href) {
        if (shouldTargetBlank(href)) {
          node.setAttribute('target', '_blank');
        } else {
          node.removeAttribute('target');
        }
        node.setAttribute('href', (0, _string.sanitizeUrl)(href));
        node.innerText = text;
        _reactDom2.default.unmountComponentAtNode(container);
        body.removeChild(container);
        _this.history.save();
      },
      onCancel: function onCancel(options) {
        if (!options.link && isRmove) {
          removeLink();
        }
        _reactDom2.default.unmountComponentAtNode(container);
        body.removeChild(container);
      }
    })
  ), container);
}

exports.default = {
  initialize: function initialize() {
    var _this2 = this;

    this.container.on('click', function (event) {
      var link = _engine2.default.$(event.target).closest(TAG_NAME);
      if (link.length > 0) {
        var section = _this2.section.closest(link);
        if (section) {
          var component = _this2.section.getComponent(section);
          if (component && 'table' === component.name) {
            event.preventDefault();
            event.stopPropagation();
          }
        } else {
          event.preventDefault();
          event.stopImmediatePropagation();
          event.stopPropagation();
          if (link[0]) {
            go.call(_this2, link[0]);
          }
        }
      }
    });
    this.command.add(PLUGIN_NAME, {
      execute: function execute() {
        var range = _this2.change.getRange();
        if (range.collapsed) {
          var linkNode = _engine2.default.$('<a>'.concat(_this2.lang.link.linkText, '</a>'));
          _this2.change.insertInline(linkNode);
          if (linkNode[0]) {
            go.call(_this2, linkNode[0], 'edit', true);
          }
          return linkNode;
        }
        _this2.history.stop();
        _this2.change.unwrapInline('<a />');
        _this2.change.wrapInline('<a href="" target="_blank" class="zacheditor-link-marker" />');
        (0, _range.shrinkToElementNode)(range);
        _this2.change.select(range);
        _this2.history.save();
        var root = _engine2.default.$(range.commonAncestorContainer);
        if (TAG_NAME !== root.name) {
          root = root.find('.zacheditor-link-marker');
          if (root.length === 0) {
            root = _engine2.default.$(range.commonAncestorContainer).closest(TAG_NAME);
          }
        }
        root.removeClass('zacheditor-link-marker');
        if (root[0]) {
          go.call(_this2, root[0], 'edit', true);
        }
        return root;
      }
    });

    this.on('paste:each', function (node) {
      if (TAG_NAME === node.name) {
        if (shouldTargetBlank(node.attr('href') || '')) {
          node.attr('target', '_blank');
        } else {
          node.removeAttr('target');
        }
      }
    });
    // 快捷键
    var options = this.options[PLUGIN_NAME] || {
      hotkey: 'mod+k'
    };

    if (!!options.hotkey) {
      this.hotkey.set(options.hotkey, PLUGIN_NAME);
    }
  }
};