'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.handleKeydownEnter = undefined;

var _tinsoEditorEngine = require('tinso-editor-engine');

var _tinsoEditorEngine2 = _interopRequireDefault(_tinsoEditorEngine);

var _utils = require('../section/table/utils');

var _schema = require('../section/table/schema');

var _schema2 = _interopRequireDefault(_schema);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var ChangeUtils = _tinsoEditorEngine2.default.ChangeUtils,
    StringUtils = _tinsoEditorEngine2.default.StringUtils,
    NodeUtils = _tinsoEditorEngine2.default.NodeUtils,
    $ = _tinsoEditorEngine2.default.$;


function handleKeydownEnter(e) {
  var range = this.change.getRange();
  if (!range.collapsed) {
    return;
  }
  var block = ChangeUtils.getClosestBlock(range.startContainer);
  if (block.name !== 'p') {
    return;
  }

  var options = this.options.markdown || {
    items: ['table']
  };

  if (options.items.indexOf('table') < 0) return;

  var chars = ChangeUtils.getBlockLeftText(block, range);
  var match = /^\|(?:(?:[^\|]+?)\|){2,}$/.exec(chars);

  if (match) {
    e.preventDefault();
    ChangeUtils.removeBlockLeftText(block, range);
    var textList = match[0].split('|').filter(function (n) {
      return n;
    });
    var rows = 3;
    var cols = textList.length;
    var html = '<table>';

    for (var i = 0; i <= rows - 1; i++) {
      html += '<tr>';
      for (var j = 0; j <= cols - 1; j++) {
        var text = i === 0 ? StringUtils.escape(textList[j]) : '<br />';
        html += '<td><p>'.concat(text, '</p></td>');
      }
      html += '</tr>';
    }

    html += '</table>';
    var node = $(html);
    var table = (0, _utils.normalizeTable)(node[0]);
    var tableModel = (0, _utils.getTableModel)(node[0]);
    var value = {
      html: table.outerHTML,
      rows: tableModel.rows,
      cols: tableModel.cols
    };
    this.command.execute('table', value);
    return value;
  }
}

exports.handleKeydownEnter = handleKeydownEnter;
exports.default = {
  initialize: function initialize() {
    var _this = this;

    // 添加被允许的标签
    this.schema.add(_schema2.default);
    // 创建命令
    this.command.add('table', {
      execute: function execute(value) {
        var sectionRoot = _this.change.insertSection('table', value || {
          rows: 8,
          cols: 8
        });
        var component = _this.section.getComponent(sectionRoot);
        _this.change.activateSection(sectionRoot);
        component.selection.selectFirstCell();
        return component;
      }
    });
    this.on('paste:origin', _utils.unWrapperTableBI);
    this.on('paste:before', function (fragment) {
      NodeUtils.fetchAllChildren(fragment).forEach(function (node) {
        node = $(node);
        if (node.name === 'table') {
          var table = (0, _utils.normalizeTable)(node[0]);
          var tableModel = (0, _utils.getTableModel)(node[0]);
          var value = {
            html: table.outerHTML,
            rows: tableModel.rows,
            cols: tableModel.cols
          };
          _this.section.replaceNode(node, 'table', value);
        }
      });
    });
    // markdown 快捷方式
    this.on('keydown:enter', function (e) {
      handleKeydownEnter.call(_this, e);
    });
  }
};