'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _eventemitter = require('eventemitter2');

var _engine = require('../editor/engine');

var _engine2 = _interopRequireDefault(_engine);

var _userAgent = require('tinso-editor-engine/lib/utils/user-agent');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var Scrollbar = function (_EventEmitter) {
  _inherits(Scrollbar, _EventEmitter);

  /**
   * @param {nativeNode} container 需要添加滚动条的容器元素
   * @param {boolean} x 横向滚动条
   * @param {boolean} y 竖向滚动条
   * @param {boolean} needShadow 是否显示阴影
   */
  function Scrollbar(container, x, y, needShadow) {
    _classCallCheck(this, Scrollbar);

    var _this = _possibleConstructorReturn(this, (Scrollbar.__proto__ || Object.getPrototypeOf(Scrollbar)).call(this));

    _this.init = function () {
      if (!_userAgent.firefox && !_userAgent.mobile) {
        var children = _this.container.children();
        var hasScrollbar = false;
        children.each(function (child) {
          if (_engine2.default.$(child).hasClass('zacheditor-scrollbar')) {
            hasScrollbar = true;
          }
        });
        if (!hasScrollbar) {
          _this.container.css('position', 'relative');
          _this.container.addClass('zacheditor-scrollable');
          if (_this.option.x) {
            _this.scrollBarX = _engine2.default.$('<div class="zacheditor-scrollbar zacheditor-scrollbar-x "><div class="zacheditor-scrollbar-trigger"></div></div>');
            _this.slideX = _this.scrollBarX.find('.zacheditor-scrollbar-trigger');
            _this.container.append(_this.scrollBarX);
            _this.container.addClass('scroll-x');
          }
          if (_this.option.y) {
            _this.scrollBarY = _engine2.default.$('<div class="zacheditor-scrollbar zacheditor-scrollbar-y "><div class="zacheditor-scrollbar-trigger"></div></div>');
            _this.slideY = _this.scrollBarY.find('.zacheditor-scrollbar-trigger');
            _this.container.append(_this.scrollBarY);
            _this.container.addClass('scroll-y');
          }
          if (_this.option.needShadow) {
            _this.shadowLeft = _engine2.default.$('<div class="scrollbar-shadow-left"></div>');
            _this.shadowRight = _engine2.default.$('<div class="scrollbar-shadow-right"></div>');
            _this.container.append(_this.shadowLeft);
            _this.container.append(_this.shadowRight);
          }
          _this.refresh();
          _this.bindEvents();
        }
      }
    };

    _this.scrollX = function (event) {
      if (_this.slideX.dragging) {
        var left = _this.slideX.startScrollLeft + (event.clientX - _this.slideX.startX);
        left = Math.max(0, Math.min(left, _this.oWidth - _this.xWidth));
        _this.slideX.css('left', left + 'px');
        var min = left / (_this.oWidth - _this.xWidth);
        min = Math.min(1, min);
        _this.container[0].scrollLeft = (_this.sWidth - _this.oWidth) * min;
      }
    };

    _this.scrollY = function (event) {
      if (_this.slideY.dragging) {
        var top = _this.slideY.startScrollTop + (event.clientY - _this.slideY.startY);
        top = Math.max(0, Math.min(top, _this.oHeight - _this.yHeight));
        _this.slideY.css('top', top + 'px');
        var min = top / (_this.oHeight - _this.yHeight);
        min = Math.min(1, min);
        _this.container[0].scrollTop = (_this.sHeight - _this.oHeight) * min;
      }
    };

    _this.scrollXEnd = function () {
      _this.slideX.dragging = false;
      document.body.removeEventListener('mousemove', _this.scrollX);
      document.body.removeEventListener('mouseup', _this.scrollXEnd);
      _this.container.removeClass('scrolling');
    };

    _this.scrollYEnd = function () {
      _this.slideY.dragging = false;
      document.body.removeEventListener('mousemove', _this.scrollY);
      document.body.removeEventListener('mouseup', _this.scrollYEnd);
      _this.container.removeClass('scrolling');
    };

    _this.bindXScrollEvent = function () {
      if (_this.option.x) {
        _this.slideX.on('mousedown', function (event) {
          _this.container.addClass('scrolling');
          _this.slideX.dragging = true;
          _this.slideX.startX = event.clientX;
          _this.slideX.startScrollLeft = parseInt(_this.slideX.css('left'));
          document.body.addEventListener('mousemove', _this.scrollX);
          document.body.addEventListener('mouseup', _this.scrollXEnd);
        });
      }
    };

    _this.bindYScrollEvent = function () {
      if (_this.option.y) {
        _this.slideY.on('mousedown', function (event) {
          _this.container.addClass('scrolling');
          _this.slideY.dragging = true;
          _this.slideY.startY = event.clientY;
          _this.slideY.startScrollTop = parseInt(_this.slideY.css('top'));
          document.body.addEventListener('mousemove', _this.scrollY);
          document.body.addEventListener('mouseup', _this.scrollYEnd);
        });
      }
    };

    _this.reRenderShadow = function (width) {
      if (_this.option.needShadow) {
        _this.shadowLeft.css('left', width + 'px');
        _this.shadowRight.css('left', width + _this.oWidth - 4 + 'px');
        _this.shadowLeft.css('display', 'none');
        _this.shadowRight.css('display', 'none');
        _this.shadowTimer && clearTimeout(_this.shadowTimer);
        _this.shadowTimer = setTimeout(function () {
          if (0 !== width) {
            _this.shadowLeft.css('display', 'block');
          }
          if (width !== _this.maxScrollLeft) {
            _this.shadowRight.css('display', 'block');
          }
        }, 100);
      }
    };

    _this.reRenderX = function (left) {
      if (_this.option.x) {
        _this.scrollBarX.css('left', left + 'px');
        var min = left / (_this.sWidth - _this.oWidth);
        min = Math.min(1, min);
        _this.slideX.css('left', (_this.oWidth - _this.xWidth) * min + 'px');
        _this.reRenderShadow(left);
      }
    };

    _this.reRenderY = function (top) {
      if (_this.option.y) {
        _this.scrollBarY.css('top', top + 'px');
        var min = top / (_this.sHeight - _this.oHeight);
        min = Math.min(1, min);
        _this.slideY.css('top', (_this.oHeight - _this.yHeight) * min + 'px');
      }
    };

    _this.bindEvents = function () {
      _this.container.on('scroll', function (event) {
        var target = event.target;
        var scrollTop = target.scrollTop,
            scrollLeft = target.scrollLeft;

        _this.reRenderX(scrollLeft);
        _this.reRenderY(scrollTop);
      });
      _this.bindXScrollEvent();
      _this.bindYScrollEvent();
    };

    _this.refresh = function () {
      if (!_userAgent.firefox && !_userAgent.mobile) {
        var _this$container$ = _this.container[0],
            offsetWidth = _this$container$.offsetWidth,
            offsetHeight = _this$container$.offsetHeight,
            scrollWidth = _this$container$.scrollWidth,
            scrollHeight = _this$container$.scrollHeight,
            scrollLeft = _this$container$.scrollLeft,
            scrollTop = _this$container$.scrollTop;


        _this.oWidth = offsetWidth;
        _this.oHeight = offsetHeight;
        _this.sWidth = scrollWidth;
        _this.sHeight = scrollHeight;
        _this.xWidth = Math.floor(offsetWidth * offsetWidth / scrollWidth);
        _this.yHeight = Math.floor(offsetHeight * offsetHeight / scrollHeight);
        _this.maxScrollLeft = scrollWidth - offsetWidth;
        if (_this.option.x) {
          _this.slideX.css('width', _this.xWidth + 'px');
          _this.slideX.css('display', _this.oWidth === _this.sWidth ? 'none' : 'block');
        }
        if (_this.option.y) {
          _this.slideY.css('height', _this.yHeight + 'px');
          _this.slideY.css('display', _this.oHeight === _this.sHeight ? 'none' : 'block');
        }
        _this.reRenderX(scrollLeft);
        _this.reRenderY(scrollTop);
      }
    };

    _this.container = _engine2.default.$(container);
    _this.option = {
      x: x,
      y: y,
      needShadow: needShadow
    };
    _this.init();
    return _this;
  }

  return Scrollbar;
}(_eventemitter.EventEmitter2);

exports.default = Scrollbar;