'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.setMode = exports.template = exports.MODE_SYNTAX_MAP = exports.MODE_NAME_MAP = undefined;

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _reactDom = require('react-dom');

var _reactDom2 = _interopRequireDefault(_reactDom);

var _modeConfig = require('./mode-config');

var _modeConfig2 = _interopRequireDefault(_modeConfig);

var _tinsoEditorEngine = require('tinso-editor-engine');

var _tinsoEditorEngine2 = _interopRequireDefault(_tinsoEditorEngine);

var _header = require('./header');

var _header2 = _interopRequireDefault(_header);

var _codemirror = require('codemirror');

var _codemirror2 = _interopRequireDefault(_codemirror);

var _runMode = require('./run-mode');

var _runMode2 = _interopRequireDefault(_runMode);

var _debounce = require('lodash/debounce');

var _debounce2 = _interopRequireDefault(_debounce);

var _antd = require('antd');

require('antd/lib/message/style');

var _base = require('../base');

var _base2 = _interopRequireDefault(_base);

require('./index.css');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var StringUtils = _tinsoEditorEngine2.default.StringUtils,
    $ = _tinsoEditorEngine2.default.$;
// Map: 代码块支持的模式 -> 名称

var qa = ['c', 'cpp', 'csharp', 'erlang', 'go', 'groovy', 'java', 'kotlin', 'makefile', 'objectivec', 'perl', 'python', 'rust', 'swift', 'vbnet'];

var MODE_NAME_MAP = exports.MODE_NAME_MAP = {};
// Map: 代码块支持的模式 -> 在 codemirror 中的语法 mode
var MODE_SYNTAX_MAP = exports.MODE_SYNTAX_MAP = {};
_modeConfig2.default.forEach(function (item) {
  MODE_NAME_MAP[item.value] = item.name;
  MODE_SYNTAX_MAP[item.value] = item.syntax;
});
// 传给 CodeMirror 的 mode
var getCodeMirrorMode = function getCodeMirrorMode(mode) {
  return MODE_SYNTAX_MAP[mode] || mode;
};

var template = exports.template = function template(data) {
  var escape = StringUtils.escape;
  var mode = data.mode;
  return '\n  <div class="zacheditor-codeblock zacheditor-codeblock-'.concat(escape(mode), '">\n    <div class="zacheditor-codeblock-header"></div>\n    <div class="zacheditor-codeblock-content"></div>\n  </div>\n  ');
};

var setMode = exports.setMode = function setMode(root, mode, code) {
  mode = getCodeMirrorMode(mode);
  var stage = $('<div class="CodeMirror"><pre class="cm-s-default" /></div>');
  root.append(stage);
  var pre = stage.find('pre')[0];
  (0, _runMode2.default)(code || '', mode, pre, {
    tabSize: 2
  });
};

var CodeBlock = function (_SectionBase) {
  _inherits(CodeBlock, _SectionBase);

  function CodeBlock(engine) {
    _classCallCheck(this, CodeBlock);

    var _this = _possibleConstructorReturn(this, (CodeBlock.__proto__ || Object.getPrototypeOf(CodeBlock)).call(this));

    _this.adaptMode = function (value) {
      // 是指定语言类型
      if (MODE_NAME_MAP[value.mode]) {
        return value.mode;
      }
      return _this.mode;
    };

    _this.saveValue = function () {
      // 中文输入过程需要判断
      if (_this.engine.domEvent.isComposing) {
        return;
      }
      var code = _this.codeMirror.getValue();
      _this.section.setValue(_this.sectionRoot, {
        mode: _this.mode,
        code: code
      });
    };

    _this.focusToCodeEditor = function () {
      _this.codeMirror.focus();
    };

    _this.engine = engine;
    if (!engine) {
      return _possibleConstructorReturn(_this);
    }
    var viewEngine = _this.getViewEngine();
    _this.section = engine.section;
    _this.locale = viewEngine.locale.codeBlock;

    var options = _this.loadOptions();
    _this.mode = MODE_NAME_MAP[options.mode] ? options.mode : 'plain';
    return _this;
  }

  _createClass(CodeBlock, [{
    key: 'loadOptions',
    value: function loadOptions() {
      var options = localStorage.getItem('zacheditor-codeblock');
      if (options) try {
        return JSON.parse(options);
      } catch (e) {}
      return {};
    }
  }, {
    key: 'setOptions',
    value: function setOptions(options) {
      localStorage.setItem('zacheditor-codeblock', JSON.stringify(options));
    }
  }, {
    key: 'embedToolbar',
    value: function embedToolbar() {
      var options = this.engine ? this.engine.options : this.contentView.options;
      var config = options.codeblock || {};
      var embed = options.type === 'mini' || options.type === 'mobile' ? [{
        type: 'copy'
      }, {
        type: 'separator'
      }, {
        type: 'copyContent',
        tips: this.getViewEngine().locale.codeBlock.copyCode
      }, {
        type: 'delete'
      }] : [{
        type: 'dnd'
      }, {
        type: 'copy'
      }, {
        type: 'separator'
      }, {
        type: 'copyContent',
        tips: this.getViewEngine().locale.codeBlock.copyCode
      }, {
        type: 'delete'
      }];

      if (Array.isArray(config.embed)) {
        return config.embed;
      } else if (_typeof(config.embed) === 'object') {
        var embedArray = [];
        embed.forEach(function (item) {
          if (config.embed[item.type] !== false) {
            embedArray.push(item);
          }
        });
        return embedArray;
      } else {
        return embed;
      }
    }
  }, {
    key: 'copyContent',
    value: function copyContent() {
      var successText = this.getViewEngine().locale.codeBlock.copySuccess;
      _tinsoEditorEngine2.default.ClipboardUtils.copyText(this.codeArea.text());
      _antd.message.success(successText);
    }
  }, {
    key: 'destroy',
    value: function destroy() {}
  }, {
    key: 'activate',
    value: function activate() {
      if (!this.state.readonly) {
        this.resizeController.show();
      }
    }
  }, {
    key: 'unactivate',
    value: function unactivate() {
      this.setOptions({
        mode: this.mode
      });
      this.saveValue();
      if (!this.state.readonly) {
        this.resizeController.hide();
      }
    }

    /**
     * 适配 section 取值为当前支持的标准模式
     *
     * @param {object} value section 取值
     * @param {string} value.mode section 存储的代码格式
     * @return {string} 归一化后的合法代码格式
     */

  }, {
    key: 'renderToolbar',
    value: function renderToolbar() {
      var _this2 = this;

      this.blockHeader = this.container.find('.zacheditor-codeblock-header');

      if (!this.engine) {
        this.blockHeader.remove();
        return;
      }

      var headerDom = this.blockHeader[0];
      _reactDom2.default.render(_react2.default.createElement(_header2.default, {
        defaultValue: this.mode,
        onSelect: function onSelect(value) {
          _this2.mode = value;
          _this2.updateMode(_this2.mode);
          _this2.saveValue();
          _this2.focusToCodeEditor();
        },
        locale: this.locale,
        getPopupContainer: function getPopupContainer() {
          return headerDom;
        }
      }), headerDom);
    }

    /**
     *
     * @param {string} mode codemirror 中的代码模式 https://codemirror.net/mode/
     * @param {string} code 代码取值
     */

  }, {
    key: 'createCodeEditor',
    value: function createCodeEditor(mode, code) {
      var _this3 = this;

      var config = arguments.length > 2 && void 0 !== arguments[2] ? arguments[2] : {};
      this.codeMirror = (0, _codemirror2.default)(this.codeArea[0], Object.assign({
        mode: mode,
        value: code,
        lineNumbers: true,
        lineWrapping: false,
        styleActiveLine: false,
        matchBrackets: true,
        autofocus: false,
        dragDrop: false,
        tabSize: 2,
        readOnly: false,
        scrollbarStyle: 'null',
        viewportMargin: 1 / 0
      }, config));

      this.codeMirror.on('focus', function () {
        _this3.engine.toolbar.disable(true);
      });

      this.codeMirror.on('blur', function () {
        _this3.engine.toolbar.disable(false);
      });

      this.codeMirror.on('change', (0, _debounce2.default)(function () {
        _this3.engine.change.onChange(_this3.engine.change.getValue());
        _this3.saveValue();
      }, 1e3));

      this.codeMirror.setOption('extraKeys', {
        Enter: function Enter(mirror) {
          var cfg = _this3.getCfg(mirror.getValue());
          for (var item in cfg) {
            mirror.setOption(item, cfg[item]);
          }mirror.execCommand('newlineAndIndent');
        }
      });
    }
  }, {
    key: 'getCfg',
    value: function getCfg(value) {
      var mode = arguments.length > 1 && void 0 !== arguments[1] ? arguments[1] : this.mode;
      var tabSize = this.codeMirror ? this.codeMirror.getOption('indentUnit') : qa.includes(mode) ? 4 : 2;
      var reg = void 0;
      if (value) {
        reg = value.match(/^ {2,4}(?=[^\s])/gm);
      }
      if (reg) {
        tabSize = reg.reduce(function (val1, val2) {
          return Math.min(val1, val2.length);
        }, 1 / 0);
      }
      return {
        tabSize: tabSize,
        indentUnit: tabSize
      };
    }
  }, {
    key: 'updateMode',
    value: function updateMode(mode) {
      var syntax = getCodeMirrorMode(mode);
      var code = this.codeMirror.getValue();
      var cfg = this.getCfg(code, mode);
      this.codeArea.empty();
      this.createCodeEditor(syntax, code || '', cfg);
    }
    // 阅读模式渲染代码

  }, {
    key: 'renderCode',
    value: function renderCode(code) {
      this.codeArea = this.container.find('.zacheditor-codeblock-content');
      setMode(this.codeArea, this.mode, code);
    }
    // 编辑模式渲染编辑器

  }, {
    key: 'renderEditor',
    value: function renderEditor(code) {
      var _this4 = this;

      this.sectionRoot = this.section.closest(this.container);
      this.codeArea = this.container.find('.zacheditor-codeblock-content');
      var mode = getCodeMirrorMode(this.mode);
      var cfg = this.getCfg(code, mode);
      this.createCodeEditor(mode, code || '', cfg);

      this.container.on('click', function (e) {
        var hasSelect = $(e.target).closest('.ant-select');
        if (!hasSelect || hasSelect.length === 0) _this4.focusToCodeEditor();
      });
      this.addResizeController(this.codeArea);
    }
  }, {
    key: 'render',
    value: function render(container, value) {
      value = value || {};
      this.mode = this.adaptMode(value);
      // 初始化 section 节点
      container.append(template({
        mode: this.mode
      }));
      // 渲染工具条
      this.renderToolbar();
      // 渲染主体
      if (this.engine) {
        this.renderEditor(value.code);
      } else {
        this.renderCode(value.code);
      }
      var height = this.value.height;
      if (height) {
        this.codeArea.css('height', height + 'px');
      }
    }
  }]);

  return CodeBlock;
}(_base2.default);

CodeBlock.type = 'block';
exports.default = CodeBlock;