'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _codemirror = require('codemirror');

var _codemirror2 = _interopRequireDefault(_codemirror);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * 代码来自 runmode addon
 * 支持行号需要考虑复制粘贴问题
 *
 * runmode 本身不支持行号，见 https://github.com/codemirror/CodeMirror/issues/3364
 * 可参考的解法  https://stackoverflow.com/questions/14237361/use-codemirror-for-standalone-syntax-highlighting-with-line-numbers
 *
 * ref:
 * - https://codemirror.net/doc/manual.html#addons
 * - https://codemirror.net/addon/runmode/runmode.js
 */
exports.default = function (string, modespec, callback, options) {
    var mode = _codemirror2.default.getMode(_codemirror2.default.defaults, modespec);
    var ie = /MSIE \d/.test(navigator.userAgent);
    var ie_lt9 = ie && (document.documentMode == null || document.documentMode < 9);

    if (callback.appendChild) {
        var tabSize = options && options.tabSize || _codemirror2.default.defaults.tabSize;
        var node = callback;
        var col = 0;
        node.innerHTML = '';

        callback = function callback(text, style) {
            if (text === '\n') {
                // Emitting LF or CRLF on IE8 or earlier results in an incorrect display.
                // Emitting a carriage return makes everything ok.
                node.appendChild(document.createTextNode(ie_lt9 ? '\r' : text));
                col = 0;
                return;
            }

            var content = '';
            // replace tabs

            for (var pos = 0;;) {
                var idx = text.indexOf('\t', pos);

                if (idx === -1) {
                    content += text.slice(pos);
                    col += text.length - pos;
                    break;
                } else {
                    col += idx - pos;
                    content += text.slice(pos, idx);
                    var size = tabSize - col % tabSize;
                    col += size;

                    for (var i = 0; i < size; ++i) {
                        content += ' ';
                    }

                    pos = idx + 1;
                }
            }

            if (style) {
                var sp = node.appendChild(document.createElement('span'));
                sp.className = 'cm-' + style.replace(/ +/g, ' cm-');
                sp.appendChild(document.createTextNode(content));
            } else {
                node.appendChild(document.createTextNode(content));
            }
        };
    }

    var lines = _codemirror2.default.splitLines(string);
    var state = options && options.state || _codemirror2.default.startState(mode);

    for (var i = 0, e = lines.length; i < e; ++i) {
        if (i) callback('\n');
        var stream = new _codemirror2.default.StringStream(lines[i]);
        if (!stream.string && mode.blankLine) mode.blankLine(state);

        while (!stream.eol()) {
            var style = mode.token(stream, state);
            callback(stream.current(), style, i, stream.start, state);
            stream.start = stream.pos;
        }
    }
};