'use strict';

var _command = require('./command');

var _command2 = _interopRequireDefault(_command);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

_command2.default.registerCommand('collapseExpand', {
    getItem: function getItem(editor) {
        var page = editor.getCurrentPage();
        var graph = page.getGraph();
        if (this.itemId) {
            return graph.find(this.itemId);
        }
        return page.getSelected()[0];
    },
    enable: function enable(editor) {
        var item = this.getItem(editor);
        return item && item.collapseExpand !== false && item.getChildren().length > 0;
    },
    execute: function execute(editor) {
        var page = editor.getCurrentPage();
        var graph = page.getGraph();
        var item = this.getItem(editor);
        var selectedModel = item.getModel();

        if (selectedModel.collapsed) {
            graph.update(item, {
                collapsed: false
            });
            this.toCollapsed = false;
        } else {
            graph.update(item, {
                collapsed: true
            });
            this.toCollapsed = true;
        }

        page.clearSelected();
        page.setSelected(item, true);

        if (this.executeTimes === 1) {
            this.itemId = item.id;
        }
    },
    back: function back(editor) {
        var page = editor.getCurrentPage();
        var graph = page.getGraph();
        var item = this.getItem(editor);
        if (this.toCollapsed) {
            graph.update(item, {
                collapsed: false
            });
        } else {
            graph.update(item, {
                collapsed: true
            });
        }
        page.clearSelected();
        page.setSelected(item, true);
    },
    shortcutCodes: [['metaKey', '/'], ['ctrlKey', '/']]
});
_command2.default.registerCommand('collapse', {
    enable: function enable(editor) {
        var item = this.getItem(editor);
        return item && item.collapseExpand !== false && item.getChildren().length > 0 && !item.getModel().collapsed;
    }
}, 'collapseExpand');
_command2.default.registerCommand('expand', {
    enable: function enable(editor) {
        var item = this.getItem(editor);
        return item && item.collapseExpand !== false && item.getChildren().length > 0 && item.getModel().collapsed;
    }
}, 'collapseExpand');