'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _util = require('../util');

var _util2 = _interopRequireDefault(_util);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var Command = {};
// 执行命令前存储快照
function snapShotexecute(editor) {
    var page = editor.getCurrentPage();
    this.snapShot = page.save();
    this.selectedItems = page.getSelected().map(function (item) {
        return item.id;
    });
    if (this.method) {
        if (_util2.default.isString(this.method)) {
            page[this.method]();
        } else {
            this.method(editor);
        }
    }
}
// 通过快照回滚
function snapShotBack(editor) {
    var page = editor.getCurrentPage();
    page.read(this.snapShot);
    page.setSelected(this.selectedItems, true);
}
// 变更模型判断是否可用
function changeModeEnable(editor) {
    var page = editor.getCurrentPage();
    return page.getMode() !== this.toMode;
}
// 切换模式类执行方法
function changeModeexecute(editor) {
    var page = editor.getCurrentPage();
    this.fromMode = page.getMode();
    page.changeMode(this.toMode);
}
// 模式回滚
function changeModeBack(editor) {
    var page = editor.getCurrentPage();
    page.changeMode(this.fromMode);
}

Command.list = [];
Command.registerCommand = function (name, cfg, extend) {
    if (Command[name]) {
        _util2.default.mix(Command[name], cfg);
    } else {
        var command = _extends({
            enable: function enable() {
                return true;
            },
            init: function init() {},
            execute: snapShotexecute,
            back: snapShotBack,
            shortcutCodes: undefined,
            executeTimes: 1,
            name: name,
            queue: true
        }, cfg);

        if (extend && Command[extend]) {
            command = _util2.default.mix({}, Command[extend], cfg);
        }

        Command[name] = command;
        Command.list.push(command);
    }
};

Command.execute = function (name, editor, obj) {
    var command = _util2.default.mix(true, {}, Command[name], obj);
    var cfg = editor.get('_command');

    if (command.enable(editor)) {
        command.init();
        if (command.queue) {
            cfg.queue.splice(cfg.current, cfg.queue.length - cfg.current, command);
            cfg.current += 1;
        }

        editor.emit('beforecommandexecute', {
            command: command
        });
        command.execute(editor);
        editor.emit('aftercommandexecute', {
            command: command
        });
    }
    return command;
};

Command.enable = function (name, editor) {
    return Command[name].enable(editor);
};

Command.registerCommand('common', {
    back: snapShotBack
});
// 添加命令
Command.registerCommand('add', {
    enable: function enable() {
        return this.type && this.addModel;
    },
    execute: function execute(editor) {
        var page = editor.getCurrentPage();
        var graph = page.getGraph();
        var addItem = graph.add(this.type, this.addModel);

        if (this.executeTimes === 1) {
            this.addId = addItem.id;
            this.page = page;
        }
    },
    back: function back() {
        var page = this.page;
        var graph = page.getGraph();
        graph.remove(this.addId);
    }
});
// 更新命令
Command.registerCommand('update', {
    enable: function enable() {
        return this.itemId && this.updateModel;
    },
    execute: function execute(editor) {
        var page = editor.getCurrentPage();
        var graph = page.getGraph();
        var item = graph.find(this.itemId);

        if (this.executeTimes === 1) {
            this.originModel = _util2.default.getContrast(item.getModel(), this.updateModel);
            this.page = page;
        }

        graph.update(item, this.updateModel);
    },
    back: function back() {
        var page = this.page;
        var graph = page.getGraph();
        var item = graph.find(this.itemId);
        graph.update(item, this.originModel);
    }
});
// 重做
Command.registerCommand('redo', {
    queue: false,
    enable: function enable(editor) {
        var cfg = editor.get('_command');
        var queue = cfg.queue;
        return cfg.current < queue.length;
    },
    execute: function execute(editor) {
        var cfg = editor.get('_command');
        var queue = cfg.queue;
        var current = cfg.current;
        queue[current].execute(editor);
        cfg.current += 1;
    },
    shortcutCodes: [['metaKey', 'shiftKey', 'z'], ['ctrlKey', 'shiftKey', 'z']]
});
// 撤销
Command.registerCommand('undo', {
    queue: false,
    enable: function enable(editor) {
        var cfg = editor.get('_command');
        return cfg.current > 0;
    },
    execute: function execute(editor) {
        var cfg = editor.get('_command');
        var queue = cfg.queue;
        var current = cfg.current;
        var command = queue[current - 1];
        command.executeTimes++;
        command.back(editor);
        cfg.current -= 1;
    },
    shortcutCodes: [['metaKey', 'z'], ['ctrlKey', 'z']]
});

Command.registerCommand('delete', {
    getDeleteItems: function getDeleteItems(editor) {
        var page = editor.getCurrentPage();
        var graph = page.getGraph();
        var items = this.itemIds ? this.itemIds.map(function (itemId) {
            return graph.find(itemId);
        }) : page.getSelected();
        items = items.filter(function (item) {
            return item.deleteable !== false;
        });
        return items;
    },
    enable: function enable(editor) {
        var items = this.getDeleteItems(editor);
        return items.length > 0;
    },
    method: function method(editor) {
        var page = editor.getCurrentPage();
        var items = this.getDeleteItems(editor);
        var graph = page.getGraph();
        page.emit('beforedelete', {
            items: items
        });
        _util2.default.each(items, function (item) {
            graph.remove(item);
        });
        page.emit('afterdelete', {
            items: items
        });
        this.itemIds = items.map(function (item) {
            return item.getModel().id;
        });
    },
    back: snapShotBack,
    shortcutCodes: ['Delete', 'Backspace']
});
Command.registerCommand('selectAll', {
    method: function method(editor) {
        var page = editor.getCurrentPage();
        var graph = page.getGraph();
        var items = graph.getItems();
        items.forEach(function (item) {
            page.setSelected(item, true);
        });
    },
    back: snapShotBack,
    shortcutCodes: [['metaKey', 'a']]
});
Command.registerCommand('clear', {
    enable: function enable(editor) {
        var page = editor.getCurrentPage();
        var items = page.getItems();
        return items.length > 0;
    },
    method: 'clear',
    back: snapShotBack
});
Command.registerCommand('multiSelect', {
    enable: changeModeEnable,
    toMode: 'multiSelect',
    execute: changeModeexecute,
    back: changeModeBack
});
Command.registerCommand('move', {
    enable: changeModeEnable,
    toMode: 'move',
    execute: changeModeexecute,
    back: changeModeBack
});

exports.default = Command;