'use strict';

var _command = require('./command');

var _command2 = _interopRequireDefault(_command);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function zoomexecute(editor) {
    var page = editor.getCurrentPage();
    this.matrixCache = page.getMatrix().slice(0);
    this._zoom(editor);
    page.updateStatus();
}
// 视口缩放类回滚
function zoomBack(editor) {
    var page = editor.getCurrentPage();
    page.updateMatrix(this.matrixCache);
}

_command2.default.registerCommand('zoomTo', {
    _zoom: function _zoom(editor) {
        var page = editor.getCurrentPage();
        page.zoom(Number(this.zoom));
    },
    queue: false,
    execute: zoomexecute,
    back: zoomBack
});
_command2.default.registerCommand('zoomIn', {
    enable: function enable(editor) {
        var page = editor.getCurrentPage();
        var maxZoom = page.getMaxZoom();
        var minZoom = page.getMinZoom();
        var zoom = page.getZoom();
        return zoom < maxZoom || zoom === minZoom;
    },
    _zoom: function _zoom(editor) {
        var page = editor.getCurrentPage();
        var cfg = editor.get('_command');
        var zoom = page.getZoom();
        var maxZoom = page.getMaxZoom();
        var toZoom = zoom + cfg.zoomDelta;

        if (toZoom >= maxZoom) {
            toZoom = maxZoom;
        }
        page.zoom(toZoom);
    },
    queue: false,
    execute: zoomexecute,
    back: zoomBack,
    shortcutCodes: [['metaKey', '='], ['ctrlKey', '=']]
});
_command2.default.registerCommand('zoomOut', {
    enable: function enable(editor) {
        var page = editor.getCurrentPage();
        var maxZoom = page.getMaxZoom();
        var minZoom = page.getMinZoom();
        var zoom = page.getZoom();
        return zoom > minZoom || zoom === maxZoom;
    },
    _zoom: function _zoom(editor) {
        var page = editor.getCurrentPage();
        var zoom = page.getZoom();
        var minZoom = page.getMinZoom();
        var cfg = editor.get('_command');
        var toZoom = zoom - cfg.zoomDelta;

        if (toZoom <= minZoom) {
            toZoom = minZoom;
        }

        page.zoom(toZoom);
    },
    queue: false,
    execute: zoomexecute,
    back: zoomBack,
    shortcutCodes: [['metaKey', '-'], ['ctrlKey', '-']]
});
_command2.default.registerCommand('autoZoom', {
    enable: function enable() {
        return true;
    },
    _zoom: function _zoom(editor) {
        var page = editor.getCurrentPage();
        page.autoZoom();
    },
    queue: false,
    execute: zoomexecute,
    back: zoomBack
});
_command2.default.registerCommand('resetZoom', {
    enable: function enable() {
        return true;
    },
    _zoom: function _zoom(editor) {
        var page = editor.getCurrentPage();
        page.resetZoom();
    },
    queue: false,
    execute: zoomexecute,
    back: zoomBack,
    shortcutCodes: [['metaKey', '0'], ['ctrlKey', '0']]
});