'use strict';

var _page = require('../page');

var _page2 = _interopRequireDefault(_page);

var _global = require('../global');

var _global2 = _interopRequireDefault(_global);

var _util = require('../util');

var _util2 = _interopRequireDefault(_util);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

_page2.default.registerBehaviour('wheelChangeViewport', function (page) {
    var graph = page.getGraph();
    var timeout = void 0;
    graph.behaviourOn('wheel', function (ev) {
        ev.domEvent.preventDefault();
    });
    graph.behaviourOn('wheel', _util2.default.throttle(update, 16));
    function update(ev) {
        if (page.getSignal('preventWheelPan') || page.destroyed) {
            return;
        }

        var domEvent = ev.domEvent;
        var wheelZoom = page.getSignal('wheelZoom');

        if (!timeout) {
            page.setCapture(false);
        }

        if (wheelZoom) {
            var delta = domEvent.deltaY;
            var times = 1.05;

            if (Math.abs(delta) > 10) {
                // 控制灵敏度
                var originScale = graph.getMatrix()[0];

                if (delta > 0) {
                    graph.zoom({
                        x: ev.x,
                        y: ev.y
                    }, originScale * times);
                } else {
                    graph.zoom({
                        x: ev.x,
                        y: ev.y
                    }, originScale * (1 / times));
                }
            }
        } else {
            var matrix = [];
            var startMatrix = graph.getMatrix();
            _util2.default.mat3.translate(matrix, startMatrix, [-domEvent.deltaX * _global2.default.wheelPanRatio, -domEvent.deltaY * _global2.default.wheelPanRatio]);
            page.translateLimt(matrix) && graph.updateMatrix(matrix);
        }

        timeout && clearTimeout(timeout);
        timeout = setTimeout(function () {
            if (page.destroyed) {
                return;
            }

            page.setCapture(true);
            timeout = undefined;
        }, 50);
    }
});