'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _simpleBase = require('../../../simpleBase');

var _simpleBase2 = _interopRequireDefault(_simpleBase);

var _global = require('../global');

var _global2 = _interopRequireDefault(_global);

var _util = require('../util');

var _util2 = _interopRequireDefault(_util);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; } /**
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                * @fileOverview align controller
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                */


function getLineDisObject(line, point) {
    return {
        line: line,
        point: point,
        dis: _util2.default.pointLineDistance(line[0], line[1], line[2], line[3], point.x, point.y)
    };
}

var Controller = function (_Base) {
    _inherits(Controller, _Base);

    function Controller() {
        _classCallCheck(this, Controller);

        return _possibleConstructorReturn(this, (Controller.__proto__ || Object.getPrototypeOf(Controller)).apply(this, arguments));
    }

    _createClass(Controller, [{
        key: 'getDefaultCfg',
        value: function getDefaultCfg() {
            return {
                /**
                 * align line style shoule set g attrs
                 * @type {object}
                 */
                line: _global2.default.alignLineStyle,

                /**
                 * item align type
                 * @type {String|True|False}
                 */
                item: true,
                // true || false || 'horizontal' || 'vertical' || 'center',

                /**
                 * align type could be true || false || 'cc' || 'tl',
                 * @type {String|Boolean}
                 */
                grid: false,

                /**
                 * tolerance to item force align
                 * @type {String|True|False}
                 */
                tolerance: 5,
                _horizontalLines: {},
                _verticalLines: {},
                _alignLines: []
            };
        }
    }, {
        key: 'init',
        value: function init() {
            this.item && this._cacheBoxLine();
        }
    }, {
        key: '_cacheBoxLine',
        value: function _cacheBoxLine() {
            var graph = this.graph;
            var horizontalLines = this._horizontalLines;
            var verticalLines = this._verticalLines;
            var itemAlignType = this.item;
            graph.on('afteritemdraw', function (ev) {
                var item = ev.item;
                if (!_util2.default.isEdge(item)) {
                    var bbox = item.getBBox(); // set horizontal lines

                    if (itemAlignType === true || itemAlignType === 'horizontal') {
                        horizontalLines[item.id + 'tltr'] = [bbox.minX, bbox.minY, bbox.maxX, bbox.minY, item];
                        horizontalLines[item.id + 'lcrc'] = [bbox.minX, bbox.centerY, bbox.maxX, bbox.centerY, item];
                        horizontalLines[item.id + 'blbr'] = [bbox.minX, bbox.maxY, bbox.maxX, bbox.maxY, item];
                    } else if (itemAlignType === 'center') {
                        horizontalLines[item.id + 'lcrc'] = [bbox.minX, bbox.centerY, bbox.maxX, bbox.centerY, item];
                    } // set vertical lines


                    if (itemAlignType === true || itemAlignType === 'vertical') {
                        verticalLines[item.id + 'tlbl'] = [bbox.minX, bbox.minY, bbox.minX, bbox.maxY, item];
                        verticalLines[item.id + 'tcbc'] = [bbox.centerX, bbox.minY, bbox.centerX, bbox.maxY, item];
                        verticalLines[item.id + 'trbr'] = [bbox.maxX, bbox.minY, bbox.maxX, bbox.maxY, item];
                    } else if (itemAlignType === 'center') {
                        verticalLines[item.id + 'tcbc'] = [bbox.centerX, bbox.minY, bbox.centerX, bbox.maxY, item];
                    }
                }
            });
            graph.on('beforeitemdestroy', function (ev) {
                var item = ev.item;
                delete horizontalLines[item.id + 'tltr'];
                delete horizontalLines[item.id + 'lcrc'];
                delete horizontalLines[item.id + 'blbr'];
                delete verticalLines[item.id + 'tlbl'];
                delete verticalLines[item.id + 'tcbc'];
                delete verticalLines[item.id + 'trbr'];
            });
        }
    }, {
        key: 'align',
        value: function align(point, bbox) {
            var originPoint = _util2.default.mix({}, point);
            var flow = this.flow;
            var gridController = flow.getController('grid');
            this.grid && gridController && gridController.visible && this._gridAlign(point, bbox);
            this.item && this._itemAlign(point, bbox, originPoint);
            return point;
        }
    }, {
        key: '_gridAlign',
        value: function _gridAlign(point, bbox) {
            var flow = this.flow;
            var grid = this.grid;
            var cell = flow.getGridCell();

            if (grid === 'cc') {
                var centerX = Math.round((point.x + bbox.width / 2) / cell) * cell;
                var centerY = Math.round((point.y + bbox.height / 2) / cell) * cell;
                point.x = centerX - bbox.width / 2;
                point.y = centerY - bbox.height / 2;
            } else {
                point.x = Math.round(point.x / cell) * cell;
                point.y = Math.round(point.y / cell) * cell;
            }
        }
    }, {
        key: '_itemAlign',
        value: function _itemAlign(point, bbox, originPoint) {
            var horizontalLines = this._horizontalLines;
            var verticalLines = this._verticalLines;
            var tolerance = this.tolerance;
            var tc = {
                x: originPoint.x + bbox.width / 2,
                y: originPoint.y
            };
            var cc = {
                x: originPoint.x + bbox.width / 2,
                y: originPoint.y + bbox.height / 2
            };
            var bc = {
                x: originPoint.x + bbox.width / 2,
                y: originPoint.y + bbox.height
            };
            var lc = {
                x: originPoint.x,
                y: originPoint.y + bbox.height / 2
            };
            var rc = {
                x: originPoint.x + bbox.width,
                y: originPoint.y + bbox.height / 2
            };
            var horizontalDis = [];
            var verticalDis = [];
            var alignCfg = null;
            this.clearAlignLine();
            _util2.default.each(horizontalLines, function (line) {
                if (line[4].isVisible()) {
                    horizontalDis.push(getLineDisObject(line, tc));
                    horizontalDis.push(getLineDisObject(line, cc));
                    horizontalDis.push(getLineDisObject(line, bc));
                }
            });
            _util2.default.each(verticalLines, function (line) {
                if (line[4].isVisible()) {
                    verticalDis.push(getLineDisObject(line, lc));
                    verticalDis.push(getLineDisObject(line, cc));
                    verticalDis.push(getLineDisObject(line, rc));
                }
            });
            horizontalDis.sort(function (a, b) {
                return a.dis - b.dis;
            });
            verticalDis.sort(function (a, b) {
                return a.dis - b.dis;
            });

            if (horizontalDis.length !== 0 && horizontalDis[0].dis < tolerance) {
                point.y = horizontalDis[0].line[1] - horizontalDis[0].point.y + originPoint.y;
                alignCfg = {
                    type: 'item',
                    horizontals: [horizontalDis[0]]
                };

                for (var i = 1; i < 3; i++) {
                    if (horizontalDis[0].dis === horizontalDis[i].dis) {
                        alignCfg.horizontals.push(horizontalDis[i]);
                    }
                }
            }

            if (verticalDis.length !== 0 && verticalDis[0].dis < tolerance) {
                point.x = verticalDis[0].line[0] - verticalDis[0].point.x + originPoint.x;
                if (!alignCfg) {
                    alignCfg = {
                        type: 'item',
                        verticals: [verticalDis[0]]
                    };
                } else {
                    alignCfg.verticals = [verticalDis[0]];
                }

                for (var _i = 1; _i < 3; _i++) {
                    if (verticalDis[0].dis === verticalDis[_i].dis) {
                        alignCfg.verticals.push(verticalDis[_i]);
                    }
                }
            }

            if (alignCfg) {
                alignCfg.bbox = bbox;

                this._addAlignLine(alignCfg);
            }
        }
    }, {
        key: 'clearAlignLine',
        value: function clearAlignLine() {
            var alignLines = this._alignLines;
            _util2.default.each(alignLines, function (line) {
                line.remove();
            });
            this._alignLines = [];
        }
    }, {
        key: '_addAlignLine',
        value: function _addAlignLine(cfg) {
            var bbox = cfg.bbox;
            var graph = this.graph;
            var rootGroup = graph.getRootGroup();
            var lineStyle = this.line;
            var alignLines = this._alignLines;

            if (cfg.type === 'item') {
                if (cfg.horizontals) {
                    _util2.default.each(cfg.horizontals, function (horizontal) {
                        var refLine = horizontal.line;
                        var refPoint = horizontal.point;
                        var lineCenterX = (refLine[0] + refLine[2]) / 2;
                        var x1 = void 0;
                        var x2 = void 0;

                        if (refPoint.x < lineCenterX) {
                            x1 = refPoint.x - bbox.width / 2;
                            x2 = Math.max(refLine[0], refLine[2]);
                        } else {
                            x1 = refPoint.x + bbox.width / 2;
                            x2 = Math.min(refLine[0], refLine[2]);
                        }

                        var line = rootGroup.addShape('line', {
                            attrs: _util2.default.mix({
                                x1: x1,
                                y1: refLine[1],
                                x2: x2,
                                y2: refLine[1]
                            }, lineStyle),
                            capture: false
                        });
                        alignLines.push(line);
                    });
                }

                if (cfg.verticals) {
                    _util2.default.each(cfg.verticals, function (vertical) {
                        var refLine = vertical.line;
                        var refPoint = vertical.point;
                        var lineCenterY = (refLine[1] + refLine[3]) / 2;
                        var y1 = void 0;
                        var y2 = void 0;

                        if (refPoint.y < lineCenterY) {
                            y1 = refPoint.y - bbox.height / 2;
                            y2 = Math.max(refLine[1], refLine[3]);
                        } else {
                            y1 = refPoint.y + bbox.height / 2;
                            y2 = Math.min(refLine[1], refLine[3]);
                        }

                        var line = rootGroup.addShape('line', {
                            attrs: _util2.default.mix({
                                x1: refLine[0],
                                y1: y1,
                                x2: refLine[0],
                                y2: y2
                            }, lineStyle),
                            capture: false
                        });
                        alignLines.push(line);
                    });
                }
            }
        }
    }]);

    return Controller;
}(_simpleBase2.default);

exports.default = Controller;