'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _simpleBase = require('../../../simpleBase');

var _simpleBase2 = _interopRequireDefault(_simpleBase);

var _global = require('../global');

var _global2 = _interopRequireDefault(_global);

var _util = require('../util');

var _util2 = _interopRequireDefault(_util);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var Controller = function (_Base) {
    _inherits(Controller, _Base);

    function Controller() {
        _classCallCheck(this, Controller);

        return _possibleConstructorReturn(this, (Controller.__proto__ || Object.getPrototypeOf(Controller)).call(this, arguments));
    }

    _createClass(Controller, [{
        key: 'getDefaultCfg',
        value: function getDefaultCfg() {
            return {
                /**
                 * grid cell
                 * @type {number}
                 */
                cell: 16,

                /**
                 * grid line style
                 * @type {object}
                 */
                line: _global2.default.gridStyle,

                /**
                 * grid line style
                 * @type {string}
                 */
                type: 'point',

                /**
                 * visible
                 * @type {boolean}
                 */
                visible: true
            };
        }
    }, {
        key: 'init',
        value: function init() {
            this._draw();
            this._onViewPortChange();
            !this.visible && this.hide();
        }
    }, {
        key: '_onViewPortChange',
        value: function _onViewPortChange() {
            var _this2 = this;

            var graph = this.graph;
            graph.on('afterviewportchange', function () {
                _this2.update();
            });
            graph.on('beforechangesize', function () {
                _this2.update();
            });
        }
    }, {
        key: '_draw',
        value: function _draw() {
            var graph = this.graph;
            var path = this._getPath();
            var group = graph.getRootGroup();
            var attrs = _util2.default.mix({}, this.line);
            var matrix = graph.getMatrix();
            var type = this.type;
            var lineWidth = type === 'line' ? 1 / matrix[0] : 2 / matrix[0];

            if (type === 'point') {
                attrs.lineDash = null;
            }

            attrs.lineWidth = lineWidth;
            attrs.path = path;
            var gridEl = group.addShape('path', {
                attrs: attrs,
                capture: false,
                zIndex: 0
            });
            _util2.default.toBack(gridEl, group);
            this.gridEl = gridEl;
        }
    }, {
        key: 'show',
        value: function show() {
            this.gridEl.show();
            this.visible = true;
        }
    }, {
        key: 'hide',
        value: function hide() {
            this.gridEl.hide();
            this.visible = false;
        }
    }, {
        key: '_getLinePath',
        value: function _getLinePath() {
            var graph = this.graph;
            var width = graph.get('width');
            var height = graph.get('height');
            var tl = graph.getPoint({
                x: 0,
                y: 0
            });
            var br = graph.getPoint({
                x: width,
                y: height
            });
            var cell = this.cell;
            var flooX = Math.ceil(tl.x / cell) * cell;
            var flooY = Math.ceil(tl.y / cell) * cell;
            var path = [];

            for (var i = 0; i <= br.x - tl.x; i += cell) {
                var x = flooX + i;
                path.push(['M', x, tl.y]);
                path.push(['L', x, br.y]);
            }

            for (var j = 0; j <= br.y - tl.y; j += cell) {
                var y = flooY + j;
                path.push(['M', tl.x, y]);
                path.push(['L', br.x, y]);
            }
            return path;
        }
    }, {
        key: '_getPointPath',
        value: function _getPointPath() {
            var graph = this.graph;
            var width = graph.get('width');
            var height = graph.get('height');
            var tl = graph.getPoint({
                x: 0,
                y: 0
            });
            var matrix = graph.getMatrix();
            var detalx = 2 / matrix[0];
            var br = graph.getPoint({
                x: width,
                y: height
            });
            var cell = this.getCell();
            var flooX = Math.ceil(tl.x / cell) * cell;
            var flooY = Math.ceil(tl.y / cell) * cell;
            var path = [];

            for (var i = 0; i <= br.x - tl.x; i += cell) {
                var x = flooX + i;
                for (var j = 0; j <= br.y - tl.y; j += cell) {
                    var y = flooY + j;
                    path.push(['M', x, y]);
                    path.push(['L', x + detalx, y]);
                }
            }
            return path;
        }
    }, {
        key: 'getCell',
        value: function getCell() {
            var cell = this.cell;
            var graph = this.graph;
            var matrix = graph.getMatrix();
            var scale = matrix[0];
            if (cell * scale < 9.6) {
                return 9.6 / scale;
            }

            return cell;
        }
    }, {
        key: '_getPath',
        value: function _getPath() {
            var type = this.type;
            return this['_get' + _util2.default.upperFirst(type) + 'Path']();
        }
    }, {
        key: 'update',
        value: function update(cfg) {
            _util2.default.mix(this, cfg);
            var path = this._getPath();
            var gridEl = this.gridEl;
            var graph = this.graph;
            var matrix = graph.getMatrix();
            var type = this.type;
            var lineWidth = type === 'line' ? 1 / matrix[0] : 2 / matrix[0];
            gridEl.attr('lineWidth', lineWidth);
            gridEl.attr('path', path);
        }
    }, {
        key: 'destroy',
        value: function destroy() {
            var gridEl = this.gridEl;
            gridEl && gridEl.remove();
        }
    }]);

    return Controller;
}(_simpleBase2.default);

exports.default = Controller;