'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _util = require('../util');

var _util2 = _interopRequireDefault(_util);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var Mixin = {};
Mixin.CFG = {
    /**
     * @type {boolean}
     */
    activeable: true,

    /**
     * @type {object}
     */
    _activedCache: {}
};
Mixin.INIT = '_initActived';
Mixin.AUGMENT = {
    _initActived: function _initActived() {
        var _this = this;

        var graph = this.get('_graph');
        graph.on('afteritemdraw', function (_ref) {
            var item = _ref.item;
            item.isActived && _this.setItemActived(item);
        });
        graph.on('beforeitemdestroy', function (ev) {
            _this.clearItemActived(ev.item);
        });
    },

    /**
     * @param {object} item g6 item
     */
    setItemActived: function setItemActived(item) {
        var graph = this.get('_graph');
        var shapeObj = graph.getShapeObj(item);
        var activedCache = this.get('_activedCache');
        var activedStyle = shapeObj.getActivedStyle(item);
        var keyShape = item.getKeyShape();
        activedCache[item.id] = item;
        activedStyle && keyShape.attr(activedStyle);

        if (item.isEdge) {
            item.startArrow && item.startArrow.attr({
                fill: activedStyle.stroke
            });
            item.endArrow && item.endArrow.attr({
                fill: activedStyle.stroke
            });
        }
    },

    /**
     * @param {object} item g6 item
     */
    clearItemActived: function clearItemActived(item) {
        var graph = this.get('_graph');
        var keyShape = item.getKeyShape();
        var shapeObj = graph.getShapeObj(item);
        var initAttrs = shapeObj.getStyle(item);
        var activedCache = this.get('_activedCache');
        var activedStyle = shapeObj.getActivedStyle(item);
        var contrastAttrs = _util2.default.getContrast(initAttrs, activedStyle);
        keyShape.attr(contrastAttrs);

        if (item.isEdge) {
            item.startArrow && item.startArrow.attr({
                fill: contrastAttrs.stroke
            });
            item.endArrow && item.endArrow.attr({
                fill: contrastAttrs.stroke
            });
        }

        delete activedCache[item.id];
    },

    /**
     * @param {object|string|array} param item || itemId || items
     * @param {boolean} bool true || false
     */
    setActived: function setActived(param, bool) {
        var _this2 = this;

        var activeable = this.get('activeable');
        var graph = this.get('_graph');

        if (!activeable) {
            return;
        }

        var items = void 0;

        if (_util2.default.isArray(param)) {
            items = param;
        } else {
            items = [param];
        }

        _util2.default.each(items, function (item) {
            if (_util2.default.isString(item)) {
                item = graph.find(item);
            }

            if (!item || item.destroyed) {
                return;
            }

            if (bool) {
                _this2.emit('beforeitemactived', {
                    item: item
                });
                _this2.setItemActived(item);
                _this2.emit('afteritemactived', {
                    item: item
                });
            } else {
                _this2.emit('beforeitemunactived', {
                    item: item
                });
                _this2.clearItemActived(item);
                _this2.emit('afteritemunactived', {
                    item: item
                });
            }
            item.isActived = bool;
        });
        graph.draw();
    },

    /**
     * get all actived items
     * @return {array} items
     */
    getActived: function getActived() {
        var activedCache = this.get('_activedCache');
        return _util2.default.objectToValues(activedCache);
    },

    /**
     * clear all actived items
     */
    clearActived: function clearActived() {
        var _this3 = this;

        var graph = this.get('_graph');
        var items = this.get('_activedCache');
        _util2.default.each(items, function (item) {
            item.isActived && _this3.setActived(item, false);
        });
        graph.draw();
    }
};
exports.default = Mixin;