'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
/**
 * @fileOverview grid
 */
var Mixin = {};

function bindBaseGraphEvent(graph, flow, eventName) {
    graph.on(eventName, function (ev) {
        flow.emit(eventName, ev);
    });
    graph.on('node:' + eventName, function (ev) {
        flow.emit('node:' + eventName, ev);
    });
    graph.on('edge:' + eventName, function (ev) {
        flow.emit('edge:' + eventName, ev);
    });
    graph.on('group:' + eventName, function (ev) {
        flow.emit('group:' + eventName, ev);
    });
    graph.on('anchor:' + eventName, function (ev) {
        flow.emit('anchor:' + eventName, ev);
    });
}

Mixin.INIT = '_initEvent';
Mixin.AUGMENT = {
    _initEvent: function _initEvent() {
        var _this = this;

        var graph = this.get('_graph');
        bindBaseGraphEvent(graph, this, 'click');
        bindBaseGraphEvent(graph, this, 'dblclick');
        bindBaseGraphEvent(graph, this, 'mouseenter');
        bindBaseGraphEvent(graph, this, 'mouseleave');
        bindBaseGraphEvent(graph, this, 'mousedown');
        bindBaseGraphEvent(graph, this, 'mouseup');
        bindBaseGraphEvent(graph, this, 'contextmenu');
        graph.on('keydown', function (ev) {
            _this.emit('keydown', ev);
        });
        graph.on('keyup', function (ev) {
            _this.emit('keyup', ev);
        });
        graph.on('beforechange', function (ev) {
            _this.emit('beforechange', ev);
        });
        graph.on('afterchange', function (ev) {
            _this.emit('afterchange', ev);
        });
        graph.on('afterviewportchange', function (ev) {
            _this.emit('afterviewportchange', ev);

            if (ev.updateMatrix[0] !== ev.originMatrix[0]) {
                _this.emit('afterzoom', ev);
            }
        });
        graph.on('beforeviewportchange', function (ev) {
            _this.emit('beforeviewportchange', ev);

            if (ev.updateMatrix[0] !== ev.originMatrix[0]) {
                _this.emit('beforezoom', ev);
            }
        });
    }
};
exports.default = Mixin;