'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _util = require('../util');

var _util2 = _interopRequireDefault(_util);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var Mixin = {};
Mixin.INIT = '_initGraph';
Mixin.AUGMENT = {
    // graph
    _initGraph: function _initGraph() {
        var graphCfg = this.get('graph');
        var GraphConstructor = this.get('graphConstructor');
        var graph = new GraphConstructor(_extends({
            page: this
        }, graphCfg));
        graph.draw();
        this.set('_graph', graph);
    },
    setMaxZoom: function setMaxZoom(zoom) {
        var graph = this.get('_graph');
        graph.set('maxZoom', zoom);
    },
    setMinZoom: function setMinZoom(zoom) {
        var graph = this.get('_graph');
        graph.set('minZoom', zoom);
    },
    setFitView: function setFitView(fitView) {
        var graph = this.get('_graph');
        graph.setFitView(fitView);
    },
    getGraphContainer: function getGraphContainer() {
        var graph = this.get('_graph');
        return graph.getGraphContainer();
    },
    getMouseEventWrapper: function getMouseEventWrapper() {
        var graph = this.get('_graph');
        return graph.getMouseEventWrapper();
    },
    getCanvas: function getCanvas() {
        var graph = this.get('_graph');
        return graph.getCanvas();
    },
    changeMode: function changeMode(mode) {
        var graph = this.get('_graph');
        graph.changeMode(mode);
    },
    updateMatrix: function updateMatrix(matrix) {
        var graph = this.get('_graph');
        graph.updateMatrix(matrix);
    },
    getMode: function getMode() {
        var graph = this.get('_graph');
        return graph.get('mode');
    },
    getMatrix: function getMatrix() {
        var graph = this.get('_graph');
        var matrix = graph.getMatrix();
        return matrix;
    },
    getZoom: function getZoom() {
        var graph = this.get('_graph');
        var matrix = graph.getMatrix();
        return matrix[0];
    },
    getMaxZoom: function getMaxZoom() {
        var graph = this.get('_graph');
        return graph.get('maxZoom');
    },
    getMinZoom: function getMinZoom() {
        var graph = this.get('_graph');
        return graph.get('minZoom');
    },
    getGraph: function getGraph() {
        return this.get('_graph');
    },
    getItems: function getItems() {
        var graph = this.get('_graph');
        return graph.getItems();
    },
    getNodes: function getNodes() {
        var graph = this.get('_graph');
        return graph.getNodes();
    },
    translate: function translate(dx, dy) {
        var graph = this.get('_graph');
        return graph.translate(dx, dy);
    },
    getEdges: function getEdges() {
        var graph = this.get('_graph');
        return graph.getEdges();
    },
    getGroups: function getGroups() {
        var graph = this.get('_graph');
        return graph.getGroups();
    },
    render: function render() {
        var graph = this.get('_graph');
        graph.render();
        return this;
    },
    add: function add(type, model) {
        var graph = this.get('_graph');
        graph.add(type, model);
        return this;
    },
    forceFit: function forceFit(type) {
        var graph = this.get('_graph');
        graph.forceFit(type);
    },
    focusPointByDom: function focusPointByDom(domPoint) {
        var graph = this.get('_graph');
        graph.focusPointByDom(domPoint);
        return this;
    },
    focusPoint: function focusPoint(graphPoint) {
        var graph = this.get('_graph');
        graph.focusPoint(graphPoint);
        return this;
    },
    find: function find(id) {
        var graph = this.get('_graph');
        return graph.find(id);
    },
    focus: function focus(id) {
        var graph = this.get('_graph');
        var item = graph.find(id);

        if (item) {
            var point = item.getCenter();
            graph.focusPoint(point);
        }

        return this;
    },
    save: function save() {
        var graph = this.get('_graph');
        return graph.save();
    },
    read: function read(data) {
        var graph = this.get('_graph');
        graph.read(data);
    },
    clear: function clear() {
        var graph = this.get('_graph');
        graph.clear();
    },
    remove: function remove(item) {
        var graph = this.get('_graph');
        graph.remove(item);
        return this;
    },
    update: function update(item, model) {
        var graph = this.get('_graph');
        graph.update(item, model);
        return this;
    },
    zoom: function zoom(point, ratio) {
        var graph = this.get('_graph');
        graph.zoom(point, ratio);
        return this;
    },
    getDomPoint: function getDomPoint(graphPoint) {
        var graph = this.get('_graph');
        return graph.getDomPoint(graphPoint);
    },
    getPoint: function getPoint(domPoint) {
        var graph = this.get('_graph');
        return graph.getPoint(domPoint);
    },
    zoomByDom: function zoomByDom(domPoint, ratio) {
        var graph = this.get('_graph');
        var point = graph.getPoint(domPoint);
        graph.zoom(point, ratio);
        return this;
    },
    autoZoom: function autoZoom() {
        var graph = this.get('_graph');
        graph.autoZoom();
        return this;
    },
    resetZoom: function resetZoom() {
        var graph = this.get('_graph');
        var width = graph.getWidth();
        var height = graph.getHeight();
        graph.zoomByDom({
            x: width / 2,
            y: height / 2
        }, 1);
        return this;
    },
    css: function css(style) {
        var graph = this.get('_graph');
        var mouseEventWarrper = graph.getMouseEventWrapper();
        _util2.default.modifyCSS(mouseEventWarrper, style);
    },
    setCapture: function setCapture(bool) {
        var graph = this.get('_graph');
        var rootGroup = graph.getRootGroup();
        rootGroup.set('capture', bool);
    },
    changeSize: function changeSize(width, height) {
        var graph = this.get('_graph');
        graph.changeSize(width, height);
    },
    getWidth: function getWidth() {
        var graph = this.get('_graph');
        return graph.getWidth();
    },
    getHeight: function getHeight() {
        var graph = this.get('_graph');
        return graph.getHeight();
    },
    delete: function _delete() {
        var selectedItems = this.getSelected();
        var graph = this.get('_graph');
        _util2.default.each(selectedItems, function (item) {
            graph.remove(item);
        });
    }
};
exports.default = Mixin;