'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _util = require('../util');

var _util2 = _interopRequireDefault(_util);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var Mixin = {};
Mixin.CFG = {
    /**
     * @type {boolean}
     */
    selectable: true,

    /**
     * @type {boolean}
     */
    multiSelectable: true,

    /**
     * @type {object}
     */
    _selectedCache: {}
};
Mixin.INIT = '_initSelected';
Mixin.AUGMENT = {
    _initSelected: function _initSelected() {
        var _this = this;

        var graph = this.get('_graph');
        graph.on('afteritemdraw', function (_ref) {
            var item = _ref.item;
            item.isSelected && _this.setItemSelected(item);
        });
        graph.on('beforeitemdestroy', function (ev) {
            _this.clearItemSelected(ev.item);
        });
    },

    /**
     * @param {object} item g6 item
     */
    setItemSelected: function setItemSelected(item) {
        var graph = this.get('_graph');
        var shapeObj = graph.getShapeObj(item);
        var selectedStyle = shapeObj.getSelectedStyle(item);
        var keyShape = item.getKeyShape();
        var selectedCache = this.get('_selectedCache');
        selectedCache[item.id] = item;
        selectedStyle && keyShape.attr(selectedStyle);

        if (item.isEdge) {
            item.startArrow && item.startArrow.attr({
                fill: selectedStyle.stroke
            });
            item.endArrow && item.endArrow.attr({
                fill: selectedStyle.stroke
            });
        }
    },

    /**
     * @param {object} item g6 item
     */
    clearItemSelected: function clearItemSelected(item) {
        var graph = this.get('_graph');
        var keyShape = item.getKeyShape();
        var shapeObj = graph.getShapeObj(item);
        var initAttrs = shapeObj.getStyle(item);
        var selectedStyle = shapeObj.getSelectedStyle(item);
        var selectedCache = this.get('_selectedCache');
        var contrastAttrs = _util2.default.getContrast(initAttrs, selectedStyle);
        keyShape.attr(contrastAttrs);

        if (item.isEdge) {
            item.startArrow && item.startArrow.attr({
                fill: contrastAttrs.stroke
            });
            item.endArrow && item.endArrow.attr({
                fill: contrastAttrs.stroke
            });
        }
        delete selectedCache[item.id];
    },

    /**
     * @param {object|string|array} param item || itemId || items
     * @param {boolean} bool true || false
     */
    setSelected: function setSelected(param, bool) {
        var _this2 = this;

        var selectable = this.get('selectable');
        var graph = this.get('_graph');

        if (!selectable) {
            return;
        }

        var items = void 0;

        if (_util2.default.isArray(param)) {
            items = param;
        } else {
            items = [param];
        }

        _util2.default.each(items, function (item) {
            if (_util2.default.isString(item)) {
                item = graph.find(item);
            }

            if (!item || item.destroyed) {
                return;
            }
            // if (bool && item.isSelected) {
            //   return;
            // }
            // if (!bool && !item.isSelected) {
            //   return;
            // }
            if (bool) {
                _this2.emit('beforeitemselected', {
                    item: item
                });
                _this2.setItemSelected(item);
                _this2.emit('afteritemselected', {
                    item: item
                });
            } else {
                _this2.emit('beforeitemunselected', {
                    item: item
                });
                _this2.clearItemSelected(item);
                _this2.emit('afteritemunselected', {
                    item: item
                });
            }
            item.isSelected = bool;
            _this2.updateStatus();
            graph.draw();
        });
    },

    /**
     * get all selected items
     * @return {array} items
     */
    getSelected: function getSelected() {
        var selectedCache = this.get('_selectedCache');
        return _util2.default.objectToValues(selectedCache);
    },

    /**
     * clear all selected items
     */
    clearSelected: function clearSelected() {
        var _this3 = this;

        var graph = this.get('_graph');
        var items = this.get('_selectedCache');
        _util2.default.each(items, function (item) {
            item.isSelected && _this3.setSelected(item, false);
        });
        graph.draw();
    }
};
exports.default = Mixin;