'use strict';

var _base = require('../../base');

var _base2 = _interopRequireDefault(_base);

var _util = require('../util');

var _util2 = _interopRequireDefault(_util);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

_base2.default.registerBehaviour('panMindNode', function (mind) {
    var tree = mind.getGraph();
    var dragNodeModel = void 0;
    var originNth = void 0;
    var lastHotArea = void 0;
    tree.behaviourOn('beforeshowdelegation', function () {
        mind.clearSelected();
        mind.clearActived();
    });
    tree.behaviourOn('node:dragstart', function (ev) {
        if (ev.button === 2) {
            return;
        }

        var item = ev.item;
        dragNodeModel = item.getModel();
        if (!dragNodeModel.parent || ev.shape.isCollapsedButton || ev.shape.isExpandedButton) {
            resetStatus();
            return;
        }
        originNth = tree.getNth(item);
        tree.remove(item);
    });
    tree.behaviourOn('itempanning', function (ev) {
        if (ev.shape && ev.shape.isPlaceholder) {
            return;
        }
        var hotArea = mind.getHotArea(ev);
        var root = mind.getRoot();

        if (lastHotArea) {
            if (hotArea) {
                if (lastHotArea.id !== hotArea.id) {
                    tree.remove(tree.find(lastHotArea.id));
                }
            } else {
                tree.remove(tree.find(lastHotArea.id));
            }
        }
        lastHotArea = hotArea;
        if (hotArea) {
            var parent = hotArea.parent;

            if (!tree.find(hotArea.id)) {
                // const box = dragItem.getBBox();
                // const shapeObj = dragItem.getShapeObj();
                // const lineWidth = shapeObj.getStyle().lineWidth;
                var addModel = {
                    id: hotArea.id,
                    parent: parent.id,
                    isPlaceholder: true,
                    parentModel: parent,
                    baseline: lastHotArea.parent.id === root.id ? 'center' : undefined,
                    shape: 'mind-placeholder',
                    // lineWidth,
                    // width: box.width,
                    // height: box.height,
                    nth: hotArea.nth
                };

                if (hotArea.side) {
                    addModel.side = hotArea.side;
                }

                tree.add('node', addModel);
            }
        }
    });
    tree.behaviourOn('drop', function () {
        if (dragNodeModel) {
            if (lastHotArea) {
                var model = _util2.default.clone(dragNodeModel);
                tree.remove(lastHotArea.id);
                mind.executeCommand('moveMindNode', {
                    model: model,
                    newParentId: lastHotArea.parent.id,
                    newNth: lastHotArea.nth,
                    newSide: lastHotArea.side,
                    originParentId: dragNodeModel.parent,
                    originNth: originNth,
                    originSide: dragNodeModel.side
                });
            } else {
                resetDragNode();
            }
        }
        resetStatus();
    });
    tree.behaviourOn('canvas:mouseleave', function () {
        if (dragNodeModel) {
            resetDragNode();
            resetStatus();
        }
    });

    function resetDragNode() {
        dragNodeModel.nth = originNth;
        var node = tree.add('node', dragNodeModel);
        mind.setSelected(node, true);

        if (lastHotArea) {
            tree.remove(lastHotArea.id);
        }
    }

    function resetStatus() {
        tree.emit('panitemend');
        dragNodeModel = undefined;
        lastHotArea = undefined;
        originNth = undefined;
    }
}, ['startPanItem', 'processPanItem', 'endPanItem']);