'use strict';

var _command = require('../../../command');

var _command2 = _interopRequireDefault(_command);

var _util = require('../util');

var _util2 = _interopRequireDefault(_util);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function enable(editor) {
    var page = editor.getCurrentPage();
    var items = page.getSelected();
    return page.isMind && items.length > 0;
}

function addRootChild(page, selectedItem, addItemId) {
    var graph = page.getGraph();
    var selectedModel = selectedItem.getModel();
    var firstLefts = page.getFirstChildrenBySide('left');
    var firstLeftItem = firstLefts[0] && graph.find(firstLefts[0].id);
    return graph.add('node', {
        id: addItemId,
        parent: selectedItem.id,
        label: '新建节点',
        side: selectedModel.children.length > 2 ? 'left' : 'right',
        nth: firstLeftItem ? graph.getNth(firstLeftItem) : undefined
    });
}
// 添加相邻节点
_command2.default.registerCommand('append', {
    enable: function enable(editor) {
        var page = editor.getCurrentPage();
        var items = page.getSelected();
        return page.isMind && items.length === 1;
    },
    getItem: function getItem(editor) {
        var page = editor.getCurrentPage();
        var graph = page.getGraph();

        if (this.selectedItemId) {
            return graph.find(this.selectedItemId);
        }
        return page.getSelected()[0];
    },
    execute: function execute(editor) {
        var page = editor.getCurrentPage();
        var graph = page.getGraph();
        var root = page.getRoot();
        var selectedItem = this.getItem(editor);
        var selectedModel = selectedItem.getModel();
        var hierarchy = selectedModel.hierarchy;
        var selectedParent = selectedItem.getParent();
        var addNode = void 0;

        if (selectedItem.isRoot) {
            addNode = addRootChild(page, selectedItem, this.addItemId);
        } else {
            var nth = graph.getNth(selectedItem);
            addNode = graph.add('node', {
                id: this.addItemId,
                parent: selectedParent.id,
                side: hierarchy === 2 && root.children.length === 3 ? 'left' : selectedModel.side,
                label: '新建节点',
                nth: selectedModel.side === 'left' && hierarchy === 2 ? nth : nth + 1
            });
        }

        page.clearSelected();
        page.clearActived();
        page.setSelected(addNode, true);

        if (this.executeTimes === 1) {
            this.selectedItemId = selectedItem.id;
            this.addItemId = addNode.id;
            page.beginEditLabel(addNode);
        }
    },
    back: function back(editor) {
        var page = editor.getCurrentPage();
        var graph = page.getGraph();
        graph.remove(this.addItemId);
        page.clearSelected();
        page.clearActived();
        page.setSelected(this.selectedItemId, true);
    },
    shortcutCodes: ['Enter']
});
// 添加子节点
_command2.default.registerCommand('appendChild', {
    enable: enable,
    getItem: function getItem(editor) {
        var page = editor.getCurrentPage();
        var graph = page.getGraph();

        if (this.selectedItemId) {
            return graph.find(this.selectedItemId);
        }
        return page.getSelected()[0];
    },
    execute: function execute(editor) {
        var page = editor.getCurrentPage();
        var graph = page.getGraph();
        var selectedItem = this.getItem(editor);
        var addNode = void 0;

        if (selectedItem.isRoot) {
            addNode = addRootChild(page, selectedItem, this.addItemId);
        } else {
            addNode = graph.add('node', {
                id: this.addItemId,
                parent: selectedItem.id,
                label: '新建节点'
            });
        }

        page.clearSelected();
        page.clearActived();
        page.setSelected(addNode, true);

        if (this.executeTimes === 1) {
            this.selectedItemId = selectedItem.id;
            this.addItemId = addNode.id;
            page.beginEditLabel(addNode);
        }
    },
    back: function back(editor) {
        var page = editor.getCurrentPage();
        var graph = page.getGraph();
        graph.remove(this.addItemId);
        page.clearSelected();
        page.clearActived();
        page.setSelected(this.selectedItemId, true);
    },
    shortcutCodes: ['Tab']
});
// 移动节点
_command2.default.registerCommand('moveMindNode', {
    enable: function enable(editor) {
        var page = editor.getCurrentPage();
        var panItems = page.get('panItems');
        return page.isMind && panItems && panItems.length > 0;
    },
    execute: function execute(editor) {
        var page = editor.getCurrentPage();
        var graph = page.getGraph();
        var newParentId = this.newParentId;
        var newNth = this.newNth;
        var newSide = this.newSide;
        var model = _util2.default.clone(this.model);
        delete model.shape;
        delete model.side;
        graph.remove(model.id);
        _util2.default.mix(model, {
            parent: newParentId,
            nth: newNth,
            side: newSide
        });
        var node = graph.add('node', model);
        page.clearSelected();
        page.setSelected(node, true);
    },
    back: function back(editor) {
        var page = editor.getCurrentPage();
        var graph = page.getGraph();
        var originParentId = this.originParentId;
        var originNth = this.originNth;
        var originSide = this.originSide;
        var model = _util2.default.clone(this.model);
        delete model.shape;
        delete model.side;
        graph.remove(model.id);
        _util2.default.mix(model, {
            parent: originParentId,
            nth: originNth,
            side: originSide
        });
        var node = graph.add('node', model);
        page.clearSelected();
        page.setSelected(node, true);
    }
});