'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _get = function get(object, property, receiver) { if (object === null) object = Function.prototype; var desc = Object.getOwnPropertyDescriptor(object, property); if (desc === undefined) { var parent = Object.getPrototypeOf(object); if (parent === null) { return undefined; } else { return get(parent, property, receiver); } } else if ("value" in desc) { return desc.value; } else { var getter = desc.get; if (getter === undefined) { return undefined; } return getter.call(receiver); } };

var _g = require('@antv/g6');

var _g2 = _interopRequireDefault(_g);

var _base = require('../base');

var _base2 = _interopRequireDefault(_base);

var _util = require('./util');

var _util2 = _interopRequireDefault(_util);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var Mixins = [];

var Mind = function (_Page) {
    _inherits(Mind, _Page);

    function Mind(inputCfg) {
        _classCallCheck(this, Mind);

        var cfg = {
            /**
             * 是否显示交互热区
             * showHotArea
             * @type {boolean}
             */
            showHotArea: false,

            /**
             * 默认数据
             * default data
             * @type {object}
             */
            defaultData: {
                roots: [{
                    label: '思维导图',
                    children: [{
                        label: '新建节点'
                    }, {
                        label: '新建节点'
                    }, {
                        label: '新建节点'
                    }]
                }]
            },

            /**
             * 快捷键列表
             * shortcut list
             * @type {object}
             */
            shortcut: {
                append: true,
                appendChild: true,
                collapseExpand: true
            },

            /**
             * 是否支持标签文本编辑
             * label editable
             * @type {boolean}
             */
            labelEditable: true,

            /**
             * 图配置项
             * graph cfg
             * @type {object|undefined}
             */
            graph: {
                modes: {
                    default: ['clickNodeSelected', 'keydownMoveSelection', 'clickCanvasSelected', 'hoverMindExpandButton', 'keydownEditLabel', 'panBlank', 'wheelChangeViewport', 'panMindNode', 'clickCollapsedButton', 'clickExpandedButton', 'hoverButton', 'hoverNodeActived', 'dblclickItemEditLabel'],
                    readOnly: ['clickNodeSelected', 'wheelChangeViewport', 'keydownMoveSelection', 'hoverNodeActived', 'panCanvas', 'clickExpandedButton', 'hoverButton', 'clickCanvasSelected'],
                    none: ['clickCanvasSelected']
                },
                layout: new _g2.default.Layouts.Mindmap({
                    direction: 'H',
                    getSubTreeSep: function getSubTreeSep(node) {
                        if (node.children && node.children.length > 0) {
                            if (node.hierarchy <= 2) {
                                return 8;
                            }
                            return 2;
                        }
                        return 0;
                    },
                    getHGap: function getHGap(node) {
                        if (node.hierarchy === 1) {
                            return 8;
                        }

                        if (node.hierarchy === 2) {
                            return 24;
                        }

                        return 18;
                    },
                    getVGap: function getVGap(node) {
                        if (node.hierarchy === 1) {
                            return 8;
                        }

                        if (node.hierarchy === 2) {
                            return 12;
                        }

                        return 2;
                    },
                    getSide: function getSide(node) {
                        if (node.data.side) {
                            return node.data.side;
                        }

                        return 'right';
                    }
                }),
                mode: 'default',
                defaultIntersectBox: 'rect',
                nodeDefaultShape: 'mind-base',
                edgeDefaultShape: 'mind-edge',
                minZoom: .2,
                maxZoom: 2
            },

            /**
             * 是否展示对齐线
             * is show align line
             * @type {object}
             */
            align: {
                item: false
            },

            /**
             * 根节点图形
             * root shape
             * @type {string}
             */
            rootShape: 'mind-root',

            /**
             * 第一层子节点图形
             * first sub child shape
             * @type {string}
             */
            firstSubShape: 'mind-first-sub',

            /**
             * 第二层子节点图形
             * second sub child shape
             * @type {string}
             */
            secondSubShape: 'mind-second-sub',

            /**
             * 子节点图形
             * sub child shape
             * @type {string}
             */
            subShape: 'mind-base',

            /**
             * 默认节点图形
             * default node shape
             * @type {boolean}
             */
            nodeDefaultShape: 'mind-base',

            /**
             * 图类构造函数
             * graph constructor
             * @type {string}
             */
            graphConstructor: _g2.default.Tree,

            /**
             * 默认边旁
             * default first class side
             * @type {string}
             */
            defaultSide: 'right'
        };
        var mixinCfg = {};
        _util2.default.each(Mixins, function (Mixin) {
            _util2.default.mix(mixinCfg, Mixin.CFG);
        });
        _util2.default.mix(true, cfg, mixinCfg, inputCfg);

        var _this = _possibleConstructorReturn(this, (Mind.__proto__ || Object.getPrototypeOf(Mind)).call(this, cfg));

        _this.isMind = true;
        return _this;
    }

    _createClass(Mind, [{
        key: '_init',
        value: function _init() {
            var _this2 = this;

            _get(Mind.prototype.__proto__ || Object.getPrototypeOf(Mind.prototype), '_init', this).call(this);
            _util2.default.each(Mixins, function (Mixin) {
                Mixin.INIT && _this2[Mixin.INIT]();
            });
            var graph = this.getGraph();
            var defaultData = this.get('defaultData');
            var mode = graph.get('mode');
            var rootGroup = graph.getRootGroup();
            var hotAreaGroup = rootGroup.addGroup();
            this.set('hotAreaGroup', hotAreaGroup);
            graph.edge().shape(function (model) {
                var target = graph.find(model.target);
                if (target.getModel().isPlaceholder) {
                    return 'mind-placeholder-edge';
                }
            });
            defaultData && this.read(defaultData);

            if (mode === 'default') {
                if (defaultData) {
                    var root = this.getRoot();
                    var rootItem = graph.find(root.id);
                    this.setSelected(rootItem, true);
                }
            } else if (mode === 'readOnly') {
                var shortcut = this.get('shortcut');
                shortcut.append = false;
                shortcut.appendChild = false;
                shortcut.selectAll = false;
                shortcut.delete = false;
            }

            if (defaultData) {
                var _root = this.getRoot();
                this.focus(_root.id);
            }
        }
    }, {
        key: 'bindEvent',
        value: function bindEvent() {
            var _this3 = this;

            _get(Mind.prototype.__proto__ || Object.getPrototypeOf(Mind.prototype), 'bindEvent', this).call(this);
            var graph = this.get('_graph');
            graph.on('keydown', function (ev) {
                ev.domEvent.preventDefault();
            });
            graph.on('beforechange', function (ev) {
                if (ev.action === 'add') {
                    _this3._beforeAdd(ev);
                } else if (ev.action === 'changeData') {
                    _this3._beforeChangeData(ev);
                }
                _this3.cancelEditLabel();
            });
            graph.on('aftersource', function () {
                _this3._setHierarchy();
            });
            graph.on('afterchange', function () {
                _this3._setHotArea();
            });
            graph.on('afteritemdraw', function (ev) {
                var item = ev.item;
                var model = item.getModel();

                if (item.isEdge) {
                    var group = item.getGraphicGroup();
                    _util2.default.toBack(group, group.getParent());
                    group.setSilent('capture', false);
                }

                if (!model.parent) {
                    item.isRoot = true;
                    item.deleteable = false;
                    item.collapseExpand = false;
                    item.appendable = false;
                    item.dragable = false;
                }
            });
            this.on('beforedelete', function (ev) {
                var item = ev.items[0];

                var brothers = _this3._getBrothers(item);

                var nth = _this3._getNth(item);

                if (brothers[nth - 1]) {
                    _this3.setSelected(brothers[nth - 1].id, true);
                } else if (brothers[nth + 1]) {
                    _this3.setSelected(brothers[nth + 1].id, true);
                } else {
                    _this3.setSelected(item.getParent(), true);
                }
            });
            this.on('afteritemselected', function (ev) {
                _this3._tryAdjustViewport(ev.item);
            });
        }
    }, {
        key: 'getRoot',
        value: function getRoot() {
            var graph = this.getGraph();
            return graph.getSource().roots[0];
        }
        /**
         * 获取拖拽节点时的委托图形
         * get delegation when drag node
         * @type {function}
         * @param  {array} items source data
         * @param  {G.Group} group graphic group
         * @return {G.Shape} shape
         */
        // getDelegation(items, group) {
        //   const item = items[0];
        //   const keyShape = item.getKeyShape();
        //   const canvas = keyShape.get('canvas');
        //   const ctx = canvas.get('context');
        //   const itemBox = item.getBBox();
        //   const totalMatrix = keyShape.get('totalMatrix');
        //   const itemCanvasBox = Util.getBBox(keyShape, totalMatrix ? totalMatrix : canvas);
        //   const virtualCanvas = Util.createDOM('<canvas>');
        //   const virtualCtx = virtualCanvas.getContext('2d');
        //   const itemCanvasWidth = itemCanvasBox.maxX - itemCanvasBox.minX;
        //   const itemCanvasHeight = itemCanvasBox.maxY - itemCanvasBox.minY;
        //   const padding = [ 2.5, 2.5, 2.5, 4 ];
        //   virtualCanvas.setAttribute('width', itemCanvasWidth);
        //   virtualCanvas.setAttribute('height', itemCanvasHeight);
        //   const imgData = ctx.getImageData(
        //     itemCanvasBox.minX + padding[3],
        //     itemCanvasBox.minY + padding[0],
        //     itemCanvasWidth - padding[1] - padding[3],
        //     itemCanvasHeight - padding[0] - padding[2]
        //   );
        //   virtualCtx.putImageData(imgData, 0, 0);
        //   return group.addShape('image', {
        //     attrs: {
        //       x: itemBox.minX,
        //       y: itemBox.minY,
        //       width: itemBox.width,
        //       height: itemBox.height,
        //       swidth: itemCanvasWidth,
        //       sheight: itemCanvasHeight,
        //       img: virtualCanvas,
        //       opacity: 0.5
        //     }
        //   });
        // }

    }, {
        key: '_setHierarchy',
        value: function _setHierarchy(root) {
            var graph = this.getGraph();
            var firstSubShape = this.get('firstSubShape');
            var secondSubShape = this.get('secondSubShape');
            var defaultSide = this.get('defaultSide');

            if (!root) {
                root = this.getRoot();
                root.hierarchy = 1;
            } else {
                var parent = graph.find(root.parent);
                if (parent) {
                    var parentModel = parent.getModel();
                    root.hierarchy = parentModel.hierarchy + 1;
                    if (root.shape !== 'mind-placeholder') {
                        if (root.hierarchy === 2) {
                            root.shape = firstSubShape;
                            if (!root.side) {
                                root.side = defaultSide;
                            }
                        }

                        if (root.hierarchy === 3) {
                            root.shape = secondSubShape;
                        }
                    }
                }
            }

            _util2.default.traverseTree(root, function (child, parent) {
                child.hierarchy = parent.hierarchy + 1;
                if (!child.side) {
                    child.side = defaultSide;
                }
                if (parent.side) {
                    child.side = parent.side;
                }
                if (child.hierarchy === 2) {
                    child.shape = firstSubShape;
                } else if (child.hierarchy === 3) {
                    child.shape = secondSubShape;
                }
            }, function (parent) {
                return parent.children;
            });
        }
    }, {
        key: 'getFirstChildrenBySide',
        value: function getFirstChildrenBySide(side) {
            var root = this.getRoot();
            var rst = [];
            root.children.forEach(function (child) {
                if (child.side === side) {
                    rst.push(child);
                }
            });
            return rst;
        }
    }, {
        key: '_getNth',
        value: function _getNth(item) {
            var graph = this.getGraph();
            return graph.getNth(item);
        }
    }, {
        key: '_getBrothers',
        value: function _getBrothers(item) {
            var parent = item.getParent();
            return parent.getModel().children;
        }
    }, {
        key: '_getMoveChildModel',
        value: function _getMoveChildModel(children) {
            if (children && children.length > 0) {
                var l = children.length;
                return children[parseInt(l / 2)];
            }
        }
    }, {
        key: '_getVerticalMoveItem',
        value: function _getVerticalMoveItem(item, condition, getLong) {
            var graph = this.getGraph();
            var nodes = graph.getNodes();
            var bbox = item.getBBox();
            var checkXs = [bbox.minX, bbox.maxX];
            var inserts = [];
            var toItem = void 0;

            for (var i = 0; i < checkXs.length; i++) {
                var checkX = checkXs[i];
                for (var j = 0; j < nodes.length; j++) {
                    var node = nodes[j];
                    var nodeBox = node.getBBox();

                    if (condition(nodeBox, bbox, checkX)) {
                        inserts.push({
                            long: getLong(nodeBox, bbox),
                            node: node
                        });
                    }
                }
            }

            if (inserts.length > 0) {
                inserts.sort(function (a, b) {
                    return a.long - b.long;
                });
                toItem = inserts[0].node;
            }
            return toItem;
        }
    }, {
        key: '_arrowTopItem',
        value: function _arrowTopItem(item) {
            var brothers = this._getBrothers(item);
            var curNth = this._getNth(item);

            if (brothers[curNth - 1]) {
                return brothers[curNth - 1];
            }

            return this._getVerticalMoveItem(item, function (nodeBox, bbox, checkX) {
                return nodeBox.centerY < bbox.centerY && checkX <= nodeBox.maxX && checkX >= nodeBox.minX;
            }, function (nodeBox, bbox) {
                return bbox.centerY - nodeBox.centerY;
            });
        }
    }, {
        key: '_arrowBottomItem',
        value: function _arrowBottomItem(item) {
            var brothers = this._getBrothers(item);
            var curNth = this._getNth(item);
            if (brothers[curNth + 1]) {
                return brothers[curNth + 1];
            }

            return this._getVerticalMoveItem(item, function (nodeBox, bbox, checkX) {
                return nodeBox.centerY > bbox.centerY && checkX <= nodeBox.maxX && checkX >= nodeBox.minX;
            }, function (nodeBox, bbox) {
                return nodeBox.centerY - bbox.centerY;
            });
        }
    }, {
        key: '_arrowLeftItem',
        value: function _arrowLeftItem(item) {
            var side = _util2.default.getMindNodeSide(item);

            if (item.isRoot) {
                var children = this.getFirstChildrenBySide('left');
                return this._getMoveChildModel(children);
            }

            if (side === 'left') {
                var _children = item.getModel().children;
                return this._getMoveChildModel(_children);
            }

            var parent = item.getParent();
            return parent;
        }
    }, {
        key: '_arrowRightItem',
        value: function _arrowRightItem(item) {
            var side = _util2.default.getMindNodeSide(item);
            if (item.isRoot) {
                var children = this.getFirstChildrenBySide('right');
                return this._getMoveChildModel(children);
            }

            if (side === 'right') {
                var _children2 = item.getModel().children;
                return this._getMoveChildModel(_children2);
            }

            var parent = item.getParent();
            return parent;
        }
    }, {
        key: '_moveItemSelection',
        value: function _moveItemSelection(ev) {
            var graph = this.getGraph();
            var item = this.getSelected()[0];

            if (!item) {
                return;
            }

            var domEvent = ev.domEvent;
            var key = _util2.default.getKeyboradKey(domEvent);
            var toItem = void 0;

            if (key === 'ArrowUp' && !item.isRoot) {
                toItem = this._arrowTopItem(item);
            } else if (key === 'ArrowDown' && !item.isRoot) {
                toItem = this._arrowBottomItem(item);
            } else if (key === 'ArrowLeft') {
                toItem = this._arrowLeftItem(item);
            } else if (key === 'ArrowRight') {
                toItem = this._arrowRightItem(item);
            }

            if (toItem) {
                toItem = graph.find(toItem.id);
                if (toItem.isVisible()) {
                    this.clearSelected();
                    this.setSelected(toItem, true);
                }
            }
        }
    }, {
        key: 'showLabelEditor',
        value: function showLabelEditor(ev) {
            var domEvent = ev.domEvent;
            var item = this.getSelected()[0];
            var key = _util2.default.getKeyboradKey(domEvent);
            if (item && key.length === 1 && !domEvent.metaKey && !domEvent.ctrlKey) {
                var labelTextArea = this.get('labelTextArea');
                if (item) {
                    this.beginEditLabel(item);
                    labelTextArea.innerHTML = "";
                    _util2.default.setEndOfContenteditable(labelTextArea);
                }
            }
        }
    }, {
        key: '_tryAdjustViewport',
        value: function _tryAdjustViewport(selectedItem) {
            var graph = this.get('_graph');
            var box = selectedItem.getBBox();
            var tl = {
                x: box.minX,
                y: box.minY
            };
            var br = {
                x: box.maxX,
                y: box.maxY
            };
            var domTl = graph.getDomPoint(tl);
            var domBr = graph.getDomPoint(br);
            var width = graph.getWidth();
            var height = graph.getHeight();
            var padding = 16;

            if (domTl.x < 0) {
                graph.translate(-domTl.x + padding, 0);
            }

            if (domTl.y < 0) {
                graph.translate(0, -domTl.y + padding);
            }

            if (domBr.x > width) {
                graph.translate(width - domBr.x - padding, 0);
            }

            if (domBr.y > height) {
                graph.translate(0, height - domBr.y - padding);
            }
        }
    }, {
        key: '_beforeChangeData',
        value: function _beforeChangeData(ev) {
            var root = ev.data.roots[0];
            var rootShape = this.get('rootShape');
            root.shape = rootShape;
            this._setHierarchy(root);
        }
    }, {
        key: '_beforeAdd',
        value: function _beforeAdd(ev) {
            var graph = this.get('_graph');
            var model = ev.model;
            var parent = graph.find(model.parent);
            var parentModel = parent.getModel();
            if (parentModel.collapsed) {
                graph.update(parent, {
                    collapsed: false
                });
            }

            this._setHierarchy(model);
        }
        // 绘制交互热区（调试用）

    }, {
        key: '_drawHotAreaShape',
        value: function _drawHotAreaShape() {
            var graph = this.get('_graph');
            var hotAreaGroup = this.get('hotAreaGroup');
            var hotAreas = this.get('hotAreas');
            hotAreaGroup.clear();
            hotAreas.forEach(function (hotArea) {
                hotAreaGroup.addShape('rect', {
                    attrs: {
                        x: hotArea.minX,
                        y: hotArea.minY,
                        width: hotArea.maxX - hotArea.minX,
                        height: hotArea.maxY - hotArea.minY,
                        fill: hotArea.color,
                        fillOpacity: 0.4
                    },
                    capture: false
                });
            });
            graph.draw();
        }
        // 设置交互热区

    }, {
        key: '_setHotArea',
        value: function _setHotArea() {
            var hotAreas = [];
            var graph = this.get('_graph');
            var root = this.getRoot();
            var postFix = 'placeholder';
            var showHotArea = this.get('showHotArea');
            // 设置根节点热区
            var rootNode = graph.find(root.id);
            var rootBox = rootNode.getBBox();
            var rootDx = 90;
            var rootDy = 60;
            hotAreas.push({
                minX: rootBox.minX - rootDx,
                minY: rootBox.minY - rootDy,
                maxX: (rootBox.minX + rootBox.maxX) / 2,
                maxY: rootBox.maxY + rootDy,
                parent: root,
                current: root,
                id: root.id + 'left' + postFix,
                nth: root.children.length,
                side: 'left',
                color: 'orange'
            });
            hotAreas.push({
                minX: (rootBox.minX + rootBox.maxX) / 2,
                minY: rootBox.minY - rootDy,
                maxX: rootBox.maxX + rootDx,
                maxY: rootBox.maxY + rootDy,
                parent: root,
                current: root,
                id: root.id + 'right' + postFix,
                nth: root.children.length,
                side: 'right',
                color: 'pink'
            });

            function getNext(initNextIndex, child, parent) {
                var children = parent.children;
                // 所在节点
                var nextIndex = initNextIndex;

                if (!parent.parent) {
                    while (children[nextIndex] && children[nextIndex].side !== child.side) {
                        nextIndex++;
                    }
                }

                while (children[nextIndex] && children[nextIndex].isPlaceholder) {
                    nextIndex++;
                }

                if (children[nextIndex] && children[nextIndex].side === child.side) {
                    return children[nextIndex];
                }
            }

            function getLast(initNextIndex, child, parent) {
                var children = parent.children;
                // 所在节点
                var lastIndex = initNextIndex;

                if (!parent.parent) {
                    while (children[lastIndex] && children[lastIndex].side !== child.side) {
                        lastIndex--;
                    }
                }

                while (children[lastIndex] && children[lastIndex].isPlaceholder) {
                    lastIndex--;
                }

                if (children[lastIndex] && children[lastIndex].side === child.side) {
                    return children[lastIndex];
                }
            }
            // 设置子节点热区
            _util2.default.traverseTree(root, function (child, parent, index) {
                var childItem = graph.find(child.id);
                if (child.isPlaceholder || !childItem || !childItem.isVisible()) {
                    return;
                }

                var next = getNext(index + 1, child, parent);
                var last = getLast(index - 1, child, parent);
                var childBox = graph.find(child.id).getBBox();
                var children = parent.children;
                // 所在节点
                var firstSubDx = 90;
                var dy = 16;
                var isFirstRight = child.hierarchy === 2 && child.side === 'right';
                var isFirstLeft = child.hierarchy === 2 && child.side === 'left';

                if (!last) {
                    hotAreas.push({
                        minX: isFirstRight ? childBox.minX - firstSubDx : childBox.minX,
                        minY: function () {
                            var minY = last ? childBox.minY : childBox.minY - dy;

                            if (children[index - 1] && children[index - 1].isPlaceholder && children[index - 1].side === child.side) {
                                var placeholderBox = graph.find(children[index - 1].id).getBBox();
                                minY = placeholderBox.minY;
                            }

                            return minY;
                        }(),
                        maxX: isFirstLeft ? childBox.maxX + firstSubDx : childBox.maxX,
                        maxY: (childBox.minY + childBox.maxY) / 2,
                        parent: parent,
                        id: (last ? last.id : undefined) + child.id + parent.id + postFix,
                        side: child.side,
                        color: 'yellow',
                        nth: index
                    });
                }

                if (next) {
                    var nextBox = graph.find(next.id).getBBox();
                    hotAreas.push({
                        minX: function () {
                            if (child.side === 'left') {
                                return Math.max(childBox.minX, nextBox.minX);
                            }
                            return isFirstRight ? childBox.minX - firstSubDx : childBox.minX;
                        }(),
                        minY: (childBox.minY + childBox.maxY) / 2,
                        maxX: function () {
                            if (child.side === 'right') {
                                return Math.min(childBox.maxX, nextBox.maxX);
                            }
                            return isFirstLeft ? childBox.maxX + firstSubDx : childBox.maxX;
                        }(),
                        maxY: (nextBox.minY + nextBox.maxY) / 2,
                        parent: parent,
                        id: child.id + (next ? next.id : undefined) + parent.id + postFix,
                        side: child.side,
                        color: 'blue',
                        nth: index + 1
                    });
                } else {
                    hotAreas.push({
                        minX: isFirstRight ? childBox.minX - firstSubDx : childBox.minX,
                        minY: (childBox.minY + childBox.maxY) / 2,
                        maxX: isFirstLeft ? childBox.maxX + firstSubDx : childBox.maxX,
                        maxY: function () {
                            var maxY = childBox.maxY + dy;
                            if (children[index + 1] && children[index + 1].isPlaceholder && children[index + 1].side === child.side) {
                                var placeholderBox = graph.find(children[index + 1].id).getBBox();
                                maxY = placeholderBox.maxY;
                            }
                            return maxY;
                        }(),
                        parent: parent,
                        id: child.id + undefined + parent.id + postFix,
                        color: 'red',
                        nth: index + 1,
                        addOrder: 'push',
                        side: child.side
                    });
                }

                if (!child.children || child.children.length === 0 || child.children.length === 1 && child.children[0].isPlaceholder) {
                    var dx = 100;
                    var _dy = 0;
                    var box = void 0;

                    if (child.x > parent.x) {
                        box = {
                            minX: childBox.maxX,
                            minY: childBox.minY - _dy,
                            maxX: childBox.maxX + dx,
                            maxY: childBox.maxY + _dy
                        };
                    } else {
                        box = {
                            minX: childBox.minX - dx,
                            minY: childBox.minY - _dy,
                            maxX: childBox.minX,
                            maxY: childBox.maxY + _dy
                        };
                    }

                    hotAreas.push(_extends({}, box, {
                        parent: child,
                        id: undefined + undefined + child.id + postFix,
                        nth: 0,
                        color: 'green',
                        side: child.side,
                        addOrder: 'push'
                    }));
                }
            }, function (parent) {
                return parent.children;
            });
            this.set('hotAreas', hotAreas);
            showHotArea && this._drawHotAreaShape();
        }
    }, {
        key: 'hideHotArea',
        value: function hideHotArea() {
            var graph = this.get('_graph');
            var hotAreaGroup = this.get('hotAreaGroup');
            hotAreaGroup.clear();
            graph.draw();
            this.set('showHotArea', false);
        }
        // 显示交互热区

    }, {
        key: 'showHotArea',
        value: function showHotArea() {
            this._drawHotAreaShape();
            this.set('showHotArea', true);
        }
        // 热区击中

    }, {
        key: 'getHotArea',
        value: function getHotArea(point) {
            var hotAreas = this.get('hotAreas');
            var rst = void 0;
            hotAreas.forEach(function (hotArea) {
                if (point.x > hotArea.minX && point.x < hotArea.maxX && point.y > hotArea.minY && point.y < hotArea.maxY) {
                    rst = hotArea;
                    return false;
                }
            });
            return rst;
        }
    }, {
        key: 'saveExpandImage',
        value: function saveExpandImage(options) {
            var _this4 = this;

            var graph = this.getGraph();
            var box = graph.getBBox();
            var padding = graph.getFitViewPadding();
            var collapsedIds = void 0;
            var seletedIds = void 0;
            var activedIds = void 0;
            return graph.saveImage(_extends({
                width: box.width + padding[1] + padding[3],
                height: box.height + padding[0] + padding[2],
                beforeTransform: function beforeTransform() {
                    var nodes = graph.getNodes();
                    collapsedIds = nodes.filter(function (node) {
                        return node.getModel().collapsed;
                    }).map(function (node) {
                        return node.getModel().id;
                    });
                    collapsedIds.forEach(function (collapsedId) {
                        graph.update(collapsedId, {
                            collapsed: false
                        });
                    });
                    graph.layout();
                    seletedIds = _this4.getSelected().map(function (item) {
                        return item.id;
                    });
                    activedIds = _this4.getSelected().map(function (item) {
                        return item.id;
                    });

                    _this4.clearSelected();

                    _this4.clearActived();
                },
                afterTransform: function afterTransform() {
                    collapsedIds.forEach(function (collapsedId) {
                        graph.update(collapsedId, {
                            collapsed: true
                        });
                    });
                    _this4.setSelected(seletedIds, true);
                    _this4.setActived(activedIds, true);
                }
            }, options));
        }
    }, {
        key: 'save',
        value: function save() {
            var graph = this.get('_graph');
            var data = graph.save();
            data.roots.forEach(function (root) {
                _util2.default.traverseTree(root, function (child) {
                    delete child.x;
                    delete child.y;
                    delete child.width;
                    delete child.height;
                    delete child.parent;
                    delete child.nth;
                    delete child.hierarchy;
                    delete child.index;
                    delete child.shape;
                }, function (parent) {
                    return parent.children;
                }, true);
            });
            return data;
        }
    }]);

    return Mind;
}(_base2.default);

_util2.default.each(Mixins, function (Mixin) {
    _util2.default.mix(Mind.prototype, Mixin.AUGMENT);
});
_base2.default.setRegister(Mind, 'mind', 'page');
exports.default = Mind;