'use strict';

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _mind = require('../../mind');

var _mind2 = _interopRequireDefault(_mind);

var _util = require('../../util');

var _util2 = _interopRequireDefault(_util);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var defaultLabelStyle = {
    fill: '#000',
    textAlign: 'left',
    textBaseline: 'top'
};
var expandedButtonStyle = {
    stroke: '#959EA6',
    strokeOpacity: 0,
    fill: '#959EA6',
    cursor: 'pointer'
};

_mind2.default.registerNode('mind-base', {
    dy: 4,
    afterDraw: function afterDraw(item) {
        var model = item.getModel();

        if (model.children && model.children.length > 0) {
            if (model.collapsed) {
                this.drawExpandedButton(item);
            }
        }
    },
    debugDrawLayoutPoint: function debugDrawLayoutPoint(item) {
        var model = item.getModel();
        var group = item.getGraphicGroup();
        group.addShape('circle', {
            attrs: {
                x: model.x,
                y: model.y,
                r: 5,
                fill: 'red'
            }
        });
    },
    drawExpandedButton: function drawExpandedButton(item) {
        var w = 16;
        var h = 7;
        var s = w / 4;
        var keyShape = item.getKeyShape();
        var keyShapeBox = keyShape.getBBox();
        var group = item.getGraphicGroup();
        var buttonGroup = group.addGroup();
        var buttonPath = buttonGroup.addShape('path', {
            attrs: _extends({
                path: _util2.default.getRectPath(0, 0, w, h, 3)
            }, expandedButtonStyle)
        });
        var buttonBox = buttonPath.getBBox();
        var side = _util2.default.getMindNodeSide(item);
        var positon = this.getButtonPositon(keyShapeBox, buttonBox, side);
        var dotStyle = {
            fill: 'white',
            r: 1
        };
        buttonGroup.addShape('circle', {
            attrs: _extends({}, dotStyle, {
                x: s,
                y: h / 2
            }),
            capture: false
        });
        buttonGroup.addShape('circle', {
            attrs: _extends({}, dotStyle, {
                x: s * 2,
                y: h / 2
            }),
            capture: false
        });
        buttonGroup.addShape('circle', {
            attrs: _extends({}, dotStyle, {
                x: s * 3,
                y: h / 2
            }),
            capture: false
        });
        buttonPath.attr('lineAppendWidth', 20);
        buttonGroup.translate(positon.x, positon.y);
        buttonPath.isExpandedButton = true;
        buttonPath.isButton = true;
    },
    getButtonPositon: function getButtonPositon(keyShapeBox, buttonBox, side) {
        if (side === 'right') {
            return {
                x: keyShapeBox.maxX + 2,
                y: keyShapeBox.maxY - (buttonBox.maxY - buttonBox.minY) / 2
            };
        }

        return {
            x: keyShapeBox.minX - (buttonBox.maxX - buttonBox.minX) - 2,
            y: keyShapeBox.maxY - (buttonBox.maxY - buttonBox.minY) / 2
        };
    },
    getLabel: function getLabel(item) {
        var model = item.getModel();
        return model.label;
    },
    getPadding: function getPadding() {
        return [4, 8, 4, 8];
    },
    getSize: function getSize(item) {
        var model = item.getModel();
        var group = item.getGraphicGroup();
        var size = model.size;

        if (model.size) {
            if (_util2.default.isArray(size)) {
                return size;
            }
            if (_util2.default.isNumber(size)) {
                return [size, size];
            }
        }

        var label = group.findByClass('label')[0];
        var padding = this.getPadding(item);
        var labelBox = label.getBBox();
        return [labelBox.width + padding[1] + padding[3], labelBox.height + padding[0] + padding[2]];
    },
    getPath: function getPath(item) {
        var size = this.getSize(item);
        var style = this.getStyle(item);
        return _util2.default.getRectPath(-size[0] / 2, -size[1] / 2 + this.dy, size[0], size[1], style.radius);
    },
    drawLabel: function drawLabel(item) {
        var group = item.getGraphicGroup();
        var label = this.getLabel(item);
        var labelStyle = this.getLabelStyle(item);

        if (!label) {
            label = ' ';
        }

        var attrs = _util2.default.mix(true, {}, defaultLabelStyle, labelStyle, {
            x: 0,
            y: 0
        });

        if (!_util2.default.isObject(label)) {
            attrs.text = label;
        } else {
            _util2.default.mix(attrs, label);
        }

        var shape = group.addShape('text', {
            class: 'label',
            attrs: attrs
        });
        this.adjustLabelText(shape);
        this.adjustLabelPosition(item, shape);
        return shape;
    },
    adjustLabelText: function adjustLabelText(labelShape) {
        var labelText = labelShape.attr('text');
        var labelBox = labelShape.getBBox();

        if (labelBox.maxX - labelBox.minX > 400) {
            var font = labelShape.attr('font');
            labelText = _util2.default.getLabelTextByTextLineWidth(labelText, font);
            labelShape.attr('text', labelText);
        }
    },
    adjustLabelPosition: function adjustLabelPosition(item, labelShape) {
        var size = this.getSize(item);
        var padding = this.getPadding();
        var width = size[0];
        var labelBox = labelShape.getBBox();
        labelShape.attr({
            x: -width / 2 + padding[3],
            y: -labelBox.height / 2 + this.dy // - padding[2]/2

        });
    },
    getLabelStyle: function getLabelStyle()
    /* item */
    {
        return {
            fill: 'rgba(38,38,38,0.85)',
            lineHeight: 18,
            fontSize: 12
        };
    },
    getStyle: function getStyle()
    /* item */
    {
        return {
            fill: '#ccc',
            fillOpacity: 0,
            radius: 4,
            lineWidth: 2
        };
    },
    getActivedStyle: function getActivedStyle()
    /* item */
    {
        return {
            stroke: '#44C0FF',
            lineWidth: 2
        };
    },
    getSelectedStyle: function getSelectedStyle()
    /* item */
    {
        return {
            stroke: '#1AA7EE',
            lineWidth: 2
        };
    },
    anchor: [[0, 1], [1, 1]]
});