'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _tinsoEditorEngine = require('tinso-editor-engine');

var _tinsoEditorEngine2 = _interopRequireDefault(_tinsoEditorEngine);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var StringUtils = _tinsoEditorEngine2.default.StringUtils;
/**
 * @fileOverview 图片渲染
 */

var escape = StringUtils.escape,
    sanitizeUrl = StringUtils.sanitizeUrl;
exports.default = {
  // 获取图片 dom 节点
  _getImageHtml: function _getImageHtml(src) {
    var pageWidth = this.page.getWidth();
    var pageHeight = this.page.getHeight();
    return '<img src="'.concat(escape(sanitizeUrl(src)), '" style="width: ').concat(escape(pageWidth), 'px; height: ').concat(escape(pageHeight), 'px;"/>');
  },
  // 存 canvas base64 图片
  _saveImageBase64: function _saveImageBase64() {
    var pageCanvas = this.page.getCanvas();
    var pageCanvasDOM = pageCanvas.get('el');
    var pixelRatio = pageCanvas.get('pixelRatio');
    var pageWidth = this.page.getWidth();
    var pageHeight = this.page.getHeight();
    var pageCanvasWidth = pageWidth * pixelRatio;
    var pageCanvasHeight = pageHeight * pixelRatio;
    var canvas = document.createElement('canvas');
    var ctx = canvas.getContext('2d');

    pageCanvas._drawSync();

    canvas.width = pageCanvasWidth;
    canvas.height = pageCanvasHeight;
    ctx.drawImage(pageCanvasDOM, 0, 0, pageCanvasWidth, pageCanvasHeight, 0, 0, pageCanvasWidth, pageCanvasHeight);
    return canvas.toDataURL('image/png', 2.0);
  },
  // 渲染图片
  _renderImage: function _renderImage() {
    var src = this._saveImageBase64();
    this.imageContainer.html(this._getImageHtml(src));
    var mask = this._renderImageMask();
    this.imageContainer.append(mask);
  },
  _renderImageMask: function _renderImageMask() {
    var page = this.page;

    var lactions = page.getTextDomLactions();
    var zoom = page.getZoom();
    var search = '';
    var mask = '<div class="zacheditor-image-mask">';
    lactions.forEach(function (laction) {
      var x = laction.x,
          y = laction.y,
          text = laction.text,
          fontSize = laction.fontSize;

      var style = 'font-size: '.concat(fontSize * zoom, 'px;left: ').concat(x, 'px;top: ').concat(y, 'px;');
      mask += '<p class="zacheditor-image-mask-point" style="'.concat(style, '">').concat(text, '</p>');
      search += ' '.concat(text);
    });
    this.searchContent = search;
    return mask += '</div>';
  }
};