'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _get = function get(object, property, receiver) { if (object === null) object = Function.prototype; var desc = Object.getOwnPropertyDescriptor(object, property); if (desc === undefined) { var parent = Object.getPrototypeOf(object); if (parent === null) { return undefined; } else { return get(parent, property, receiver); } } else if ("value" in desc) { return desc.value; } else { var getter = desc.get; if (getter === undefined) { return undefined; } return getter.call(receiver); } };

var _tinsoEditorEngine = require('tinso-editor-engine');

var _tinsoEditorEngine2 = _interopRequireDefault(_tinsoEditorEngine);

var _graphEditor = require('./graph-editor');

var _graphEditor2 = _interopRequireDefault(_graphEditor);

var _toolMixin = require('./tool-mixin');

var _toolMixin2 = _interopRequireDefault(_toolMixin);

var _readMixin = require('./read-mixin');

var _readMixin2 = _interopRequireDefault(_readMixin);

var _editMixin = require('./edit-mixin');

var _editMixin2 = _interopRequireDefault(_editMixin);

var _imageMixin = require('./image-mixin');

var _imageMixin2 = _interopRequireDefault(_imageMixin);

var _maximizeMixin = require('./maximize-mixin');

var _maximizeMixin2 = _interopRequireDefault(_maximizeMixin);

var _base = require('../base');

var _base2 = _interopRequireDefault(_base);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; } /**
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                * @fileOverview 图Section基类 - 用于拓展各式各样的Section
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                */


var $ = _tinsoEditorEngine2.default.$;

var Mixins = [_toolMixin2.default, _readMixin2.default, _editMixin2.default, _imageMixin2.default, _maximizeMixin2.default];

var GraphSection = function (_SectionBase) {
  _inherits(GraphSection, _SectionBase);

  function GraphSection(_ref) {
    _classCallCheck(this, GraphSection);

    var _this = _possibleConstructorReturn(this, (GraphSection.__proto__ || Object.getPrototypeOf(GraphSection)).call(this));

    var engine = _ref.engine,
        contentView = _ref.contentView,
        pageType = _ref.pageType;

    _this.engine = engine;
    _this.graphEditor = new _graphEditor2.default();
    _this.rendererOptions = engine ? engine.options : contentView.options;
    _this.state = {};
    _this.pageType = pageType;
    _this.zoomStep = 0.1;
    // 缩放步长
    _this.currentZoom = 1;
    _this.minHeight = 240;
    _this.defaultGraphCfg = {
      minZoom: 0.2,
      maxZoom: 2,
      height: _this.minHeight,
      mode: !engine ? 'readOnly' : 'default'
    };
    _this.searchContent = '';
    Mixins.forEach(function (Mixin) {
      Object.assign(_this, Mixin);
    });

    if (!engine) {
      var _ret;

      return _ret = _this, _possibleConstructorReturn(_this, _ret);
    }
    _this.locale = _this.engine.locale[_this.pageType];
    _this.mobile = 'mobile' === _this.engine.options.type;
    return _this;
  }

  _createClass(GraphSection, [{
    key: 'topToolbar',
    value: function topToolbar() {}
  }, {
    key: 'getChangeDataCommandNames',
    value: function getChangeDataCommandNames() {}
  }, {
    key: 'embedToolbar',
    value: function embedToolbar() {
      var config = this.rendererOptions.graphsection || {};
      var embed = this.mobile ? [{
        type: 'copy'
      }, {
        type: 'delete'
      }] : [{
        type: 'dnd'
      }, {
        type: 'maximize'
      }, {
        type: 'separator'
      }, {
        type: 'copy'
      }, {
        type: 'delete'
      }];
      if (Array.isArray(config.embed)) {
        return config.embed;
      } else if (_typeof(config.embed) === 'object') {
        var embedArray = [];
        embed.forEach(function (item) {
          if (config.embed[item.type] !== false) {
            embedArray.push(item);
          }
        });
        return embedArray;
      } else {
        return embed;
      }
    }
    // 变更缩放

  }, {
    key: '_changeZoom',
    value: function _changeZoom(zoom) {
      this._changePageZoom(zoom);
      this._updateToolZoom();
    }
    // 变更图缩放

  }, {
    key: '_changePageZoom',
    value: function _changePageZoom(zoom) {
      var page = this.page;
      if (_graphEditor2.default.Util.isNumber(zoom)) {
        page.zoom(zoom);
      }

      if (_graphEditor2.default.Util.isString(zoom)) {
        var maxZoom = page.getMaxZoom();
        page.setMaxZoom(1);
        page.setFitView(zoom);
        page.setMaxZoom(maxZoom);
      }
      // 改变画布缩放记录视口
      if (!this.state.maximize) {
        this.setValue({
          matrix: page.getMatrix()
        });
      }
    }
    // 渲染画布尺寸变更控件

  }, {
    key: '_renderResizeController',
    value: function _renderResizeController() {
      var _this2 = this;

      if (this.resizeController) {
        return;
      }

      var resizeController = $('<div class="section-resize-button-ud" draggable="true" />');
      this.container.append(resizeController);
      var changeHeight = void 0;
      var startClientPoint = void 0;
      var width = void 0;
      var height = void 0;
      resizeController.hide();
      this.bindEvent(resizeController, 'dragstart', function (ev) {
        ev.preventDefault();
        ev.stopPropagation();
        ev.cancelBubble = true;
        startClientPoint = {
          x: ev.clientX,
          y: ev.clientY
        };
        width = _this2.page.getWidth();
        height = _this2.page.getHeight();
      });
      this.bindEvent($(document), 'mousemove', function (ev) {
        if (!startClientPoint) {
          return;
        }

        var dy = ev.clientY - startClientPoint.y;
        changeHeight = height + dy;

        if (changeHeight >= _this2.minHeight) {
          _this2.page.changeSize(width, changeHeight);
        }
      });
      this.bindEvent($(document), 'mouseup', function () {
        startClientPoint = undefined;
        if (changeHeight) {
          _this2.setValue({
            height: changeHeight
          });
        }
        changeHeight = undefined;
      });
      this.resizeController = resizeController;
    }

    /**
     * Section获得焦点时调用
     */

  }, {
    key: 'activate',
    value: function activate() {
      if (this.state.activated === true || this.destroyed) {
        return;
      }

      this.tool.show();
      this.pageContainer[0].style.visibility = 'visible';
      this.imageContainer.hide();

      if (!this.engine) {
        this.page.changeMode('readOnly');
      } else {
        this.container.css('border', '1px solid #D9D9D9');
        this.resizeController.show();
        this.engine.toolbar.set(this.topToolbar());
        this.page.changeMode('default');
      }
      this.state.focus = true;
    }
    /**
     * Section失去焦点时调用
     */

  }, {
    key: 'unactivate',
    value: function unactivate() {
      if (this.state.activated === false || this.destroyed) {
        return;
      }

      if (!this.engine) {
        this.page.changeMode('none');
        this._renderImage();
      } else {
        this.container.css('border', '1px solid rgba(0, 0, 0, 0)');
        this.resizeController.hide();
        this.engine.toolbar.restore();
        this.page.changeMode('none');
        this._renderImage();
      }

      this.tool.hide();
      this.pageContainer[0].style.visibility = 'hidden';
      this.imageContainer.show();
      this.page.clearSelected();
    }
    // 保存数据
    // setValue(value) {
    //   if (this.state.maximize === true && Util.isPlainObject(value)) {
    //     for (const key in value) {
    //       if (key !== 'graphData') delete value[key];
    //     }
    //   }
    //   section.setValue(this.sectionRoot, Object.assign(this.value, value));
    // }

  }, {
    key: '_initContainer',
    value: function _initContainer(container) {
      var imageContainer = $('<div class="image-container"></div>');
      var pageContainer = $('<div class="page-container"></div>');
      container.addClass('graph-section');
      this.imageContainer = imageContainer;
      this.pageContainer = pageContainer;
      this.container.append(pageContainer);
      this.container.append(imageContainer);
    }
  }, {
    key: 'normalizeValue',
    value: function normalizeValue() {
      var value = this.value;

      if (value.width) {
        value.width = +value.width;
      }
      if (value.height) {
        value.height = +value.height;
      }
    }
    /**
     * 渲染Section
     * @param  {$DOM} container - Section容器
     * @param  {boolean} value - Section值
     */

  }, {
    key: 'render',
    value: function render(container, value) {
      this.value = value || {};
      this._initContainer(container);
      this.normalizeValue();
      if (this.engine) {
        // 编辑模式
        this._renderEditView();
      } else {
        // 阅读模式
        this._renderReadView();
      }
    }
    // 读取值

  }, {
    key: '_readValue',
    value: function _readValue() {
      var page = this.page;
      var _value = this.value,
          height = _value.height,
          graphData = _value.graphData,
          matrix = _value.matrix;

      var readonly = this.state.readonly;

      if (height) {
        var width = page.getWidth();
        page.changeSize(width, height);
      }

      if (graphData) {
        page.read(graphData);
      }

      if (!readonly) {
        if (matrix) {
          page.updateMatrix(matrix);
        }
      } else {
        this._changePageZoom('autoZoom');
      }
    }
    /**
     * 销毁Section
     */

  }, {
    key: 'destroy',
    value: function destroy() {
      // 记录是否Section销毁
      _get(GraphSection.prototype.__proto__ || Object.getPrototypeOf(GraphSection.prototype), 'destroy', this).call(this);
      this.page.destroy();
      this.engine.toolbar.restore();
    }
  }]);

  return GraphSection;
}(_base2.default);

GraphSection.type = 'block';
GraphSection.uid = true;
exports.default = GraphSection;