'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _eventemitter = require('eventemitter2');

var _isArray = require('lodash/isArray');

var _isArray2 = _interopRequireDefault(_isArray);

var _isNumber = require('lodash/isNumber');

var _isNumber2 = _interopRequireDefault(_isNumber);

var _tinsoEditorEngine = require('tinso-editor-engine');

var _tinsoEditorEngine2 = _interopRequireDefault(_tinsoEditorEngine);

var _utils = require('./utils');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var HTMLParser = _tinsoEditorEngine2.default.HTMLParser,
    StringUtils = _tinsoEditorEngine2.default.StringUtils,
    ChangeUtils = _tinsoEditorEngine2.default.ChangeUtils,
    ClipboardUtils = _tinsoEditorEngine2.default.ClipboardUtils,
    TextParser = _tinsoEditorEngine2.default.TextParser,
    $ = _tinsoEditorEngine2.default.$;

var Command = function (_EventEmitter) {
  _inherits(Command, _EventEmitter);

  function Command(section) {
    _classCallCheck(this, Command);

    var _this = _possibleConstructorReturn(this, (Command.__proto__ || Object.getPrototypeOf(Command)).call(this, section));

    _this.insertColLeft = function () {
      _this.insertCol('left');
    };

    _this.insertColRight = function () {
      _this.insertCol('right');
    };

    _this.insertRowUp = function () {
      _this.insertRow('up');
    };

    _this.insertRowDown = function () {
      _this.insertRow('down');
    };

    _this.insertColAt = function (colIndex, count, isLeft, silence, widths) {
      var _this$section = _this.section,
          container = _this$section.container,
          selection = _this$section.selection,
          tableRoot = _this$section.tableRoot,
          template = _this$section.template;

      var tableModel = selection.tableModel;
      var table = tableModel.table;
      // 第一行插在前面，其他行插在后面
      isLeft = colIndex === 0 || isLeft;
      var colBase = isLeft ? colIndex : colIndex - 1;
      var insertMethod = isLeft ? 'after' : 'before';
      var colsHeader = container.find(template.COLS_HEADER_ITEM_CLASS);
      var baseColHeader = colsHeader[colBase];
      var head = container.find(template.COLS_HEADER_CLASS);
      var totalWidth = 0;

      if (!widths) {
        widths = baseColHeader.offsetWidth;
      }

      if ((0, _isArray2.default)(widths)) {
        widths.forEach(function (w) {
          totalWidth += w;
        });
      } else if ((0, _isNumber2.default)(widths)) {
        totalWidth = count * widths;
      }

      head.css('width', head[0].offsetWidth + totalWidth + 'px');
      var colgroup = tableRoot.find('colgroup');
      var trs = container.find('tr');
      var cols = tableRoot.find('col');
      var cloneNode = cols[colBase].cloneNode();
      var insertCol = colIndex;
      var counter = count;

      while (counter > 0) {
        // 插入头 和 col
        // const useWidthIndex = isLeft ? count - counter : counter - 1;
        var cloneColHeader = $(baseColHeader.outerHTML);
        // cloneColHeader.css('width', widths[useWidthIndex] + 'px');
        $(baseColHeader)[insertMethod](cloneColHeader);
        var insertCloneCol = cloneNode.cloneNode();
        $(insertCloneCol).attr('width', (0, _isArray2.default)(widths) ? widths[count - counter] : widths);
        var baseCol = cols[insertCol];
        colgroup[0].insertBefore(insertCloneCol, baseCol);
        counter--;
      }
      // 在指定行内的指定位置插入多个 td；并设置 rowSpan
      function insertMultiCell(tr, index, count, tdBase) {
        for (var i = 0; i < count; i++) {
          var td = tr.insertCell(index);
          td.innerHTML = template.EmptyCell;
          td.rowSpan = tdBase.rowSpan;
          (0, _utils.copyCss)(tdBase, td);
        }
      }
      // 插入 td
      trs.each(function (tr, r) {
        var trModel = table[r];
        var tdModel = trModel[colBase];
        var insertIndex = selection.getTdIndex(r, insertCol);

        if (tdModel.isMulti) {
          if (!isLeft && tdModel.colSpan > 1) {
            tdModel.element.colSpan = tdModel.colSpan + count;
          } else {
            insertMultiCell(tr, insertIndex, count, tdModel.element);
          }
          return;
        }

        if (tdModel.isEmpty) {
          if (tdModel.parent.col === colBase) return;
          var parentTd = table[tdModel.parent.row][tdModel.parent.col];

          if (tdModel.parent.col < insertCol && tdModel.parent.col + parentTd.colSpan - 1 >= insertCol) {
            parentTd.element.colSpan = parentTd.colSpan + count;
          } else {
            if (tdModel.parent.row === r) {
              insertMultiCell(tr, insertIndex, count, parentTd.element);
            }
          }
          return;
        }
        insertMultiCell(tr, insertIndex, count, tdModel.element);
      });

      _this.emit('actioned', 'insertCol', silence);
      // 必须等插入完在选择，否则 tableModel 没更新
      if (selection.area) {
        var newArea = Object.assign(selection.area, {
          col: colIndex,
          col2: colIndex + count - 1
        });
        selection.selectArea(newArea);
      }
    };

    _this.insertCol = function (position) {
      var count = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 1;
      var silence = arguments.length > 2 ? arguments[2] : undefined;
      var selection = this.section.selection;
      var area = selection.area,
          tableModel = selection.tableModel;
      var isLeft = position === 'left';
      var isEnd = position === 'end' || !position;

      var colBars = this.section.container.find(this.section.template.COLS_HEADER_ITEM_CLASS);
      var colBase = tableModel.cols - 1;

      if (!isEnd) {
        count = selection.isSingleArea() ? 1 : selection.colCount();

        if (area) {
          var _selection$normalizeA = selection.normalizeArea(),
              col_min = _selection$normalizeA.col_min,
              col_max = _selection$normalizeA.col_max;

          colBase = isLeft ? col_min : col_max;
        }
      }

      var insertCol = isLeft ? colBase : colBase + 1;
      var width = colBars[colBase].offsetWidth;

      this.insertColAt(insertCol, count, isLeft, silence, width);
      if (isEnd) {
        var viewPort = this.section.selection.viewport[0];
        viewPort.scrollLeft = viewPort.scrollWidth - viewPort.offsetWidth;
      }
      return;
    };

    _this.removeCol = function (silence) {
      var _this$section2 = _this.section,
          selection = _this$section2.selection,
          tableRoot = _this$section2.tableRoot,
          controllBar = _this$section2.controllBar,
          template = _this$section2.template;

      var area = selection.area,
          tableModel = selection.tableModel;
      if (!area) return;
      var table = tableModel.table;
      var col = area.col,
          col2 = area.col2;

      var count = selection.colCount();
      var col_min = Math.min(col, col2);
      var col_max = Math.max(col, col2);
      var colgroup = tableRoot.find('colgroup');
      var trs = tableRoot.find('tr');
      var cols = colgroup.find('col');
      var isLastCol = col_max === cols.length - 1;
      var isTotalTable = isLastCol && col_min === 0;

      if (isTotalTable) {
        _this.removeTable();
        return;
      }

      for (var c = col_max; c >= col_min; c--) {
        controllBar.removeCol(c);
        cols[c].remove();
      }

      table.forEach(function (trModel, r) {
        for (var _c = col_max; _c >= col_min; _c--) {
          var tdModel = trModel[_c];
          if (tdModel.isEmpty) {
            // 删除列如果在单元格内，修正单元格的 colSpan
            var parentTd = table[tdModel.parent.row][tdModel.parent.col];
            if (tdModel.parent.col < col_min) {
              var colRemoved = Math.min(count, tdModel.parent.col + parentTd.colSpan - col_min);
              parentTd.element.colSpan = parentTd.colSpan - colRemoved;
            }
            continue;
          } else {
            if (tdModel.isMulti) {
              // 合并单元格的头部被切掉，要生成尾部单元格补充到行内
              var cutHeader = _c + tdModel.colSpan - 1 > col_max;
              var cutCount = col_max + 1 - _c;

              if (cutHeader) {
                var insertIndex = 0;

                for (var i = 0; i <= col_max; i++) {
                  if (!trModel[i].isEmpty) {
                    insertIndex++;
                  }
                }

                var td = trs[r].insertCell(insertIndex);
                td.innerHTML = template.EmptyCell;
                td.colSpan = tdModel.colSpan - cutCount;
                td.rowSpan = tdModel.rowSpan;
                (0, _utils.copyCss)(tdModel.element, td);
              }
            }
            tdModel.element.remove();
          }
        }
      });
      _this.emit('actioned', 'removeCol', silence);
    };

    _this.insertRowAt = function (rowIndex) {
      var count = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 1;
      var isUp = arguments.length > 2 ? arguments[2] : undefined;
      var silence = arguments.length > 3 ? arguments[3] : undefined;
      var _section = this.section,
          container = _section.container,
          selection = _section.selection,
          tableRoot = _section.tableRoot,
          template = _section.template;

      var tableModel = selection.tableModel;
      var table = tableModel.table;
      isUp = rowIndex === 0 || isUp;
      var insertMethod = isUp ? 'after' : 'before';
      var baseRow = isUp ? rowIndex : rowIndex - 1;
      var rowBars = container.find(template.ROWS_HEADER_ITEM_CLASS);
      var baseRowBar = rowBars[baseRow];
      var insertRow = isUp ? baseRow : baseRow + 1;
      var trModel = table[baseRow];
      var insertTdProps = [];

      trModel.forEach(function (tdModel, c) {
        if (tdModel.isMulti) {
          if (!isUp && tdModel.rowSpan > 1) {
            tdModel.element.rowSpan = tdModel.rowSpan + count;
          } else {
            insertTdProps.push({
              tdBase: tdModel.element
            });
          }
          return;
        }

        if (tdModel.isEmpty) {
          var parentTd = table[tdModel.parent.row][tdModel.parent.col];
          if (tdModel.parent.row < insertRow && tdModel.parent.row + parentTd.rowSpan - 1 >= insertRow) {
            parentTd.element.rowSpan = parentTd.rowSpan + count;
          } else {
            if (tdModel.parent.row < baseRow && tdModel.parent.col === c) {
              insertTdProps.push({
                tdBase: parentTd.element
              });
            }
          }
          return;
        }
        insertTdProps.push({
          tdBase: tdModel.element
        });
      });
      var _count = count;

      var _loop = function _loop() {
        // 复制相同的行
        var tr = tableRoot[0].insertRow(insertRow);
        insertTdProps.forEach(function (props) {
          var td = tr.insertCell();
          td.innerHTML = template.EmptyCell;
          td.colSpan = props.tdBase.colSpan;
          (0, _utils.copyCss)(props.tdBase, td);
        });
        $(baseRowBar)[insertMethod]($(baseRowBar.outerHTML));
        _count--;
      };

      while (_count > 0) {
        _loop();
      }

      this.emit('actioned', 'insertRow', silence);
      // 必须等插入完在选择，否则 tableModel 没更新
      if (selection.area) {
        var newArea = Object.assign(selection.area, {
          row: rowIndex,
          row2: rowIndex + count - 1
        });
        selection.selectArea(newArea);
      }
      this.section.onTableSizeChange();
    };

    _this.insertRow = function (position) {
      var count = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 1;
      var silence = arguments.length > 2 ? arguments[2] : undefined;
      var selection = this.section.selection;
      var area = selection.area,
          tableModel = selection.tableModel;

      var isUp = position === 'up';
      var isEnd = position === 'end' || !position;
      var baseRow = tableModel.rows - 1;

      if (!isEnd) {
        count = selection.isSingleArea() ? 1 : selection.rowCount();

        if (area) {
          var row = area.row,
              row2 = area.row2;

          baseRow = isUp ? Math.min(row, row2) : Math.max(row, row2);
        }
      }
      var insertRow = isUp ? baseRow : baseRow + 1;
      this.insertRowAt(insertRow, count, isUp, silence);
    };

    _this.removeRow = function (silence) {
      var _this$section3 = _this.section,
          selection = _this$section3.selection,
          tableRoot = _this$section3.tableRoot,
          controllBar = _this$section3.controllBar,
          template = _this$section3.template;
      var area = selection.area,
          tableModel = selection.tableModel;

      if (!area) return;
      var table = tableModel.table;
      var row = area.row,
          row2 = area.row2;

      var count = selection.rowCount();
      var row_min = Math.min(row, row2);
      var row_max = Math.max(row, row2);
      var trs = tableRoot.find('tr');
      var isLastRow = row_max === tableModel.rows - 1;
      var isTotalTable = isLastRow && row_min === 0;

      if (isTotalTable) {
        _this.removeTable();
        return;
      }
      // 修正 rowSpan 和 补充单元格
      var _loop = function _loop(r) {
        var trModel = table[r];
        trModel.forEach(function (tdModel, c) {
          if (tdModel.isMulti && tdModel.rowSpan > 1) {
            // 合并单元格头部被切掉，需要补充 td
            if (r + tdModel.rowSpan - 1 > row_max) {
              var insertIndex = selection.getTdIndex(row_max + 1, c);
              var td = trs[row_max + 1].insertCell(insertIndex);
              var cutCount = row_max - r + 1;
              td.innerHTML = template.EmptyCell;
              td.colSpan = tdModel.colSpan;
              td.rowSpan = tdModel.rowSpan - cutCount;
            }
          }

          if (tdModel.isEmpty) {
            var parentTd = table[tdModel.parent.row][tdModel.parent.col]; // 合并单元格尾部或中部被切掉，修正 rowSpan
            if (tdModel.parent.row < row_min) {
              var _cutCount = Math.min(count, tdModel.parent.row + parentTd.rowSpan - row_min);
              parentTd.element.rowSpan = parentTd.rowSpan - _cutCount;
            }
          }
        });
      };

      for (var r = row_min; r <= row_max; r++) {
        _loop(r);
      }

      for (var _r = row_max; _r >= row_min; _r--) {
        tableRoot[0].deleteRow(_r);
        controllBar.removeRow(_r);
      }
      _this.emit('actioned', 'removeRow', silence);
      _this.section.onTableSizeChange();
    };

    _this.mergeCell = function (silence) {
      var selection = _this.section.selection;
      var area = selection.area;
      if (!area) return;

      _this.splitCell();
      var row = area.row,
          col = area.col,
          row2 = area.row2,
          col2 = area.col2;

      var row_min = Math.min(row, row2);
      var col_min = Math.min(col, col2);
      var row_count = selection.rowCount();
      var col_count = selection.colCount();
      var content = [];
      var mergeTd = void 0;
      selection.each(function (tdModel, r, c) {
        if (c === col_min && r === row_min) {
          mergeTd = tdModel.element;
          mergeTd.rowSpan = row_count;
          mergeTd.colSpan = col_count;
          return;
        }

        if (tdModel.element) {
          // 空单元格里面也有 html，只有在有实际内容时才会在合并的时候将内容合并
          if (tdModel.element.innerText.trim() !== '') {
            content.unshift(tdModel.element.innerHTML);
          }

          tdModel.element.remove();
        }
      });
      mergeTd.innerHTML = mergeTd.innerHTML + content.join('');
      _this.emit('actioned', 'mergeCell', silence);
    };

    _this.splitCell = function (silence) {
      var _this$section4 = _this.section,
          selection = _this$section4.selection,
          tableRoot = _this$section4.tableRoot,
          template = _this$section4.template;

      var area = selection.area,
          tableModel = selection.tableModel;
      if (!area) return;
      var table = tableModel.table;
      var trs = tableRoot.find('tr');

      var _selection$normalizeA2 = selection.normalizeArea(),
          row_min = _selection$normalizeA2.row_min,
          row_max = _selection$normalizeA2.row_max,
          col_min = _selection$normalizeA2.col_min,
          col_max = _selection$normalizeA2.col_max;
      // 注意这里用倒序，见 selection.each 方法的最后一个参数传的时 true
      // 因为是倒序，所有空位一定先转换为 td, 这样在补齐切断的单元格时，需要考虑插入时的偏移量

      selection.each(function (tdModel, r, c) {
        if (tdModel.isMulti) {
          tdModel.element.colSpan = 1;
          tdModel.element.rowSpan = 1; // 切左上角

          if (r + tdModel.rowSpan - 1 > row_max) {
            var insertRow = row_max + 1;
            var insertCol = c;
            var insertIndex = selection.getTdIndex(insertRow, insertCol);
            var td = trs[insertRow].insertCell(insertIndex);
            td.rowSpan = r + tdModel.rowSpan - 1 - row_max;
            td.colSpan = tdModel.colSpan;
            (0, _utils.copyCss)(tdModel.element, td);
            td.innerHTML = template.EmptyCell;
          }
          // 切头
          if (c + tdModel.colSpan - 1 > col_max) {
            var _insertRow = r;

            var _insertCol = col_max + 1;

            var _insertIndex = selection.getTdIndex(_insertRow, _insertCol); // 之前的空位都会变成td，这里要补齐偏移量
            var emptyCount = col_max - c;
            var _td = trs[_insertRow].insertCell(_insertIndex + emptyCount);
            _td.rowSpan = Math.min(tdModel.rowSpan, row_max - r + 1);
            _td.colSpan = c + tdModel.colSpan - 1 - col_max;
            (0, _utils.copyCss)(tdModel.element, _td);
            _td.innerHTML = template.EmptyCell;
          }
        }

        if (tdModel.isEmpty) {
          var pRow = tdModel.parent.row;
          var pCol = tdModel.parent.col;
          var parentTd = table[pRow][pCol];
          var pRow2 = pRow + parentTd.rowSpan - 1;
          var pCol2 = pCol + parentTd.colSpan - 1; // 如果选区切断了合并单元格，那么需要补齐被切断的部分
          // 会有以下切断方式：
          // 切左边，切左下角，切上边，切右上角
          // 其中要考虑选区是否贯穿单元格的情况
          // 切左边

          if (pRow < row_min) {
            var cross = col_max >= pCol2;
            parentTd.element.rowSpan = row_min - pRow;
            parentTd.element.colSpan = cross ? parentTd.colSpan : col_max - pCol + 1; // 为了防止重复补充单元格，只在遍历到选区的右上角的位置进行补充单元格，
            // 未贯穿
            if (!cross && c === col_max && r === row_min) {
              var cutCol = col_max + 1;
              var cutRow = pRow;
              var _insertIndex3 = selection.getTdIndex(cutRow, cutCol);
              var _td3 = trs[cutRow].insertCell(_insertIndex3);
              _td3.rowSpan = parentTd.rowSpan;
              _td3.colSpan = pCol2 - col_max;
              (0, _utils.copyCss)(parentTd.element, _td3);
              _td3.innerHTML = template.EmptyCell;
            }
            // 切左边非左下角，下方需要补合并单元格的左下缺口
            if (pRow2 > row_max && c === Math.min(pCol2, col_max) && r === row_min) {
              var _insertRow2 = row_max + 1;

              var _insertCol2 = pCol;

              var _insertIndex4 = selection.getTdIndex(_insertRow2, _insertCol2);
              var _td4 = trs[_insertRow2].insertCell(_insertIndex4);
              _td4.rowSpan = pRow2 - row_max;
              _td4.colSpan = cross ? parentTd.colSpan : col_max - pCol + 1;
              (0, _utils.copyCss)(parentTd.element, _td4);
              _td4.innerHTML = template.EmptyCell;
            }
          }
          // 切上边
          if (pCol < col_min) {
            var _cross = row_max >= pRow2;
            parentTd.element.colSpan = col_min - pCol;
            parentTd.element.rowSpan = _cross ? parentTd.rowSpan : row_max - pRow + 1; // 为了防止重复补充单元格，只在遍历到选区的左下角的位置进行补充单元格，
            // 非贯穿
            if (!_cross && r === row_max && c === col_min) {
              var _cutRow = row_max + 1;
              var _cutCol = pCol;
              var _insertIndex5 = selection.getTdIndex(_cutRow, _cutCol);
              var _td5 = trs[_cutRow].insertCell(_insertIndex5);
              _td5.colSpan = parentTd.colSpan;
              _td5.rowSpan = pRow2 - row_max;
              (0, _utils.copyCss)(parentTd.element, _td5);
              _td5.innerHTML = template.EmptyCell;
            } // 非右上角，需要在选区右边再补一个合并单元格的右上缺口

            if (pCol2 > col_max && r === Math.min(pRow2, row_max) && c === col_max) {
              var _insertRow3 = pRow;
              var _insertCol3 = col_max + 1;
              var _insertIndex6 = selection.getTdIndex(_insertRow3, _insertCol3);
              var _td6 = trs[_insertRow3].insertCell(_insertIndex6);
              _td6.colSpan = pCol2 - col_max;
              _td6.rowSpan = _cross ? parentTd.rowSpan : row_max - pRow + 1;
              (0, _utils.copyCss)(parentTd.element, _td6);
              _td6.innerHTML = template.EmptyCell;
            }
          }
          var _insertIndex2 = selection.getTdIndex(r, c);
          var _td2 = trs[r].insertCell(_insertIndex2);
          _td2.innerHTML = template.EmptyCell;
        }
      }, true);
      _this.emit('actioned', 'splitCell', silence);
    };

    _this.changed = function () {
      _this.tableCleared = false;
    };

    _this.clear = function () {
      var _this$section5 = _this.section,
          selection = _this$section5.selection,
          subEngine = _this$section5.subEngine,
          template = _this$section5.template;

      var area = selection.area;
      if (!area || subEngine) return;

      if (area.total_table) {
        if (_this.tableCleared) {
          _this.removeTable();
          _this.tableCleared = false;
          return;
        }
        _this.tableCleared = true;
      }

      if (area.total_row) {
        if (_this.rowCleared) {
          _this.removeRow();
          _this.rowCleared = false;
          return;
        }
        _this.rowCleared = true;
      }

      if (area.total_col) {
        if (_this.colCleared) {
          _this.removeCol();
          _this.colCleared = false;
          return;
        }
        _this.colCleared = true;
      }

      selection.each(function (tdModel) {
        if (tdModel.element) {
          tdModel.element.innerHTML = template.EmptyCell;
        }
      });
      _this.emit('actioned', 'clear');
    };

    _this.clearFormat = function () {
      var selection = _this.section.selection;
      var area = selection.area;
      if (!area) return;
      selection.each(function (tdModel) {
        if (tdModel.element) {
          tdModel.element.removeAttribute('style');
        }
      });
      _this.emit('actioned', 'clearFormat');
    };

    _this.hasCopyData = function () {
      return (0, _utils.getCopyData)();
    };

    _this.mockPaste = function (silence) {
      var data = (0, _utils.getCopyData)();
      if (!data) return;
      _this.paste(data, silence);
    };

    _this.shortcutPaste = function (e) {
      e.preventDefault();
      e.stopPropagation();
      var data = ClipboardUtils.getClipboardData(e);
      _this.paste(data);
    };

    _this.paste = function (data, silence) {
      var selection = _this.section.selection;
      var area = selection.area,
          tableModel = selection.tableModel;
      if (!area) return;

      var _selection$normalizeA3 = selection.normalizeArea(),
          row_min = _selection$normalizeA3.row_min,
          col_min = _selection$normalizeA3.col_min,
          row_max = _selection$normalizeA3.row_max,
          col_max = _selection$normalizeA3.col_max;

      var isSingleTd = row_min === row_max && col_min === col_max;
      var isSingleArea = selection.isSingleArea();
      var html = data.html,
          text = data.text;
      var pasteHTML = new HTMLParser(html, _this.section.engine.schema, _this.section.engine.conversion).toValue();
      var element = (0, _utils.trimBlankSpan)($(pasteHTML));

      if (element.name === 'table') {
        (0, _utils.normalizeTable)(element[0]);
        var pasteTableModel = (0, _utils.getTableModel)(element[0]);
        var rowCount = pasteTableModel.rows;
        var colCount = pasteTableModel.cols;
        var _pasteTableModel$tabl = pasteTableModel.table[0][0],
            rowSpan = _pasteTableModel$tabl.rowSpan,
            colSpan = _pasteTableModel$tabl.colSpan;

        var isPasteSingle = rowSpan === rowCount && colSpan === colCount;

        if (isPasteSingle && isSingleArea) {
          (0, _utils.copyTo)(pasteTableModel.table[0][0].element, tableModel.table[row_min][col_min].element);
          _this.emit('actioned', 'paste', silence);
          return;
        } // 只在选中一个非合并单元格的时候才会延伸平铺，遇到表格边界会自动增加行列
        // 若选中的是一个区域或合并单元格，则只要将区域中的单元格填充上数据即可

        if (isSingleTd) {
          if (colCount + col_min > tableModel.cols) {
            var insertColCount = colCount + col_min - tableModel.cols;
            _this.insertCol('end', insertColCount, true);
          }

          if (rowCount + row_min > tableModel.rows) {
            var insertRowCount = rowCount + row_min - tableModel.rows;
            _this.insertRow('end', insertRowCount, true);
          }
          // 选中和将要粘贴表格等大的区域
          selection.selectArea({
            row: row_min,
            col: col_min,
            row2: row_min + rowCount - 1,
            col2: col_min + colCount - 1
          });
        }

        var newArea = selection.area;
        var row_min_new = Math.min(newArea.row, newArea.row2);
        var row_max_new = Math.max(newArea.row, newArea.row2);
        var col_min_new = Math.min(newArea.col, newArea.col2);
        var col_max_new = Math.max(newArea.col, newArea.col2);
        // 先拆分单元格，拷贝的表格中可能有合并单元格，需要重新复制合并单元格情况
        _this.splitCell(true);
        selection.each(function (tdModel, r, c) {
          var paste_r = (r - row_min_new) % rowCount;
          var paste_c = (c - col_min_new) % colCount;
          var paste_td = pasteTableModel.table[paste_r][paste_c];

          if (!paste_td) {
            return;
          }

          if (paste_td.isMulti) {
            tdModel.element.rowSpan = Math.min(paste_td.rowSpan, row_max_new - r + 1);
            tdModel.element.colSpan = Math.min(paste_td.colSpan, col_max_new - c + 1);
            (0, _utils.copyTo)(paste_td.element, tdModel.element);
            return;
          }

          if (paste_td.isEmpty) {
            tdModel.element.remove();
            return;
          }

          if (paste_td.element) {
            (0, _utils.copyTo)(paste_td.element, tdModel.element);
          }
        });
      } else {
        _this.mergeCell(true);
        var $td = $(tableModel.table[row_min][col_min].element);
        _this.section.createEditor($td, html || new TextParser(text).toHTML());
      }
      _this.emit('actioned', 'paste', silence);
    };

    _this.copy = function () {
      var areaHtml = _this.section.selection.getSelectionHtml();
      if (!areaHtml) return;
      ClipboardUtils.copyNode(areaHtml);
      (0, _utils.copyHTML)(areaHtml);
    };

    _this.mockCopy = function () {
      var areaHtml = _this.section.selection.getSelectionHtml();
      if (!areaHtml) return;
      (0, _utils.copyHTML)(areaHtml);
    };

    _this.shortcutCopy = function (e) {
      var areaHtml = _this.section.selection.getSelectionHtml();
      if (!areaHtml) return;
      e.clipboardData.clearData();
      e.clipboardData.setData('text/plain', $(areaHtml)[0].innerText);
      e.clipboardData.setData('text/html', areaHtml);
      (0, _utils.copyHTML)(areaHtml);
      e.preventDefault();
    };

    _this.cut = function () {
      _this.copy();
      _this.clear();
    };

    _this.shortcutCut = function (e) {
      _this.shortcutCopy(e);
      _this.clear();
    };

    _this.removeTable = function () {
      _this.emit('tableRemoved');
      _this.section.engine.change.removeSection(_this.section.sectionRoot);
    };

    _this.alignLeft = function () {
      _this.align('left');
    };

    _this.alignCenter = function () {
      _this.align('center');
    };

    _this.alignRight = function () {
      _this.align('right');
    };

    _this.align = function (direction) {
      var selection = _this.section.selection;
      if (!selection.area) return;
      selection.each(function (tdModel) {
        if (tdModel.element) {
          $(tdModel.element).css('text-align', direction);
        }
      });
      _this.emit('actioned', 'align');
    };

    _this.valign = function (direction) {
      var selection = _this.section.selection;
      if (!selection.area) return;
      selection.each(function (tdModel) {
        if (tdModel.element) {
          $(tdModel.element).css('vertical-align', direction);
        }
      });
      _this.emit('actioned', 'align');
    };

    _this.background = function (rgb) {
      var selection = _this.section.selection;
      if (!selection.area) return;
      selection.each(function (tdModel) {
        if (tdModel.element) {
          $(tdModel.element).css('background-color', rgb || '#f0f0f0');
        }
      });
      _this.emit('actioned', 'background');
    };

    _this.fontcolor = function (rgb) {
      var selection = _this.section.selection;
      if (!selection.area) return;
      selection.each(function (tdModel) {
        if (tdModel.element) {
          $(tdModel.element).css('color', rgb || '#666');
        }
      });
      _this.emit('actioned', 'fontcolor');
    };

    _this.queryState = function (status) {
      var _this$section6 = _this.section,
          selection = _this$section6.selection,
          engine = _this$section6.engine,
          subEngine = _this$section6.subEngine;

      if (subEngine) {
        return subEngine.command.queryState(status);
      }

      if (!selection.area) return false;
      var selectHtml = selection.getSelectionHtml();
      var hasContent = $(selectHtml)[0].innerText.trim() !== '';
      var findContent = false;
      var result = void 0;
      selection.each(function (tdModel) {
        if (tdModel.element && hasContent && !findContent && tdModel.element.innerText.trim() !== '') {
          $(tdModel.element).attr('contenteditable', true);
          var range = engine.change.getRange();
          range.selectNodeContents(tdModel.element);
          engine.change.select(range);
          engine.change.marks = ChangeUtils.getActiveMarks(range);
          result = engine.command.queryState(status);
          findContent = true;
          $(tdModel.element).removeAttr('contenteditable');
        }
      });
      return result;
    };

    _this.fill = function (table) {
      var selection = _this.section.selection;
      var pasteTableModel = (0, _utils.getTableModel)(table[0]);

      var _selection$normalizeA4 = selection.normalizeArea(),
          row_min = _selection$normalizeA4.row_min,
          col_min = _selection$normalizeA4.col_min;

      selection.each(function (tdModel, r, c) {
        var paste_r = r - row_min;
        var paste_c = c - col_min;
        var paste_td = pasteTableModel.table[paste_r][paste_c];

        if (!paste_td || !tdModel.element) {
          return;
        }
        if (tdModel.element) {
          tdModel.element.innerHTML = paste_td.element.innerHTML;
        }
      });
    };

    _this.execute = function () {
      var _engine$command = void 0;
      var _section2 = this.section,
          selection = _section2.selection,
          sectionRoot = _section2.sectionRoot,
          engine = _section2.engine,
          subEngine = _section2.subEngine,
          state = _section2.state;

      if (state.readonly) return;
      if (subEngine) {
        var _subEngine$command = void 0;
        (_subEngine$command = subEngine.command).execute.apply(_subEngine$command, arguments);

        return;
      }

      var area = selection.area;
      if (!area) return;
      // 暂停历史，否则执行 command.execute 时会执行 save 很耗时，放到最后执行
      engine.history.stop();
      var selectHtml = selection.getSelectionHtml();
      // 单元格内可能有Section
      selectHtml = new HTMLParser(selectHtml, engine.schema, engine.conversion).toValue();
      selectHtml = StringUtils.transformCustomTags(selectHtml);
      var tableShadow = $(selectHtml);
      sectionRoot.append(tableShadow);
      tableShadow.attr('contenteditable', true);
      tableShadow.addClass('tmp-table');
      var range = engine.change.getRange();
      range.selectNodeContents(tableShadow[0]);
      engine.change.select(range);
      (_engine$command = engine.command).execute.apply(_engine$command, arguments);
      tableShadow.removeAttr('contenteditable');
      this.fill(tableShadow);
      tableShadow.remove();
      engine.history.start();
      this.emit('actioned', 'fontstyle');
    };

    _this.section = section;
    return _this;
  }

  return Command;
}(_eventemitter.EventEmitter2);

exports.default = Command;