'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _eventemitter = require('eventemitter2');

var _keymaster = require('keymaster');

var _keymaster2 = _interopRequireDefault(_keymaster);

var _utils = require('./utils');

var _tinsoEditorEngine = require('tinso-editor-engine');

var _tinsoEditorEngine2 = _interopRequireDefault(_tinsoEditorEngine);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var $ = _tinsoEditorEngine2.default.$;

var Selection = function (_EventEmitter) {
  _inherits(Selection, _EventEmitter);

  function Selection(_section) {
    _classCallCheck(this, Selection);

    var _this = _possibleConstructorReturn(this, (Selection.__proto__ || Object.getPrototypeOf(Selection)).call(this, _section));

    _this.selectFirstCell = function () {
      var tds = _this.section.tableRoot.find('td');
      _this.selectCell(tds[0]);
    };

    _this.rowCount = function () {
      if (!_this.area) return 0;
      var _this$area = _this.area,
          row = _this$area.row,
          row2 = _this$area.row2;

      return Math.abs(row - row2) + 1;
    };

    _this.colCount = function () {
      if (!_this.area) return 0;
      var _this$area2 = _this.area,
          col = _this$area2.col,
          col2 = _this$area2.col2;

      return Math.abs(col - col2) + 1;
    };

    _this.selectCell = function (td, mode) {
      if (!td || !td.tagName || td.tagName.toLowerCase() !== 'td') {
        return;
      }

      var table = _this.tableModel.table;
      var tr = td.parentNode;
      var rowSpan = td.rowSpan,
          colSpan = td.colSpan;
      var row = tr.rowIndex;
      var col = void 0;
      table[row].forEach(function (tdModel, i) {
        if (tdModel.element === td) {
          col = i;
        }
      });
      var row2 = row + rowSpan - 1;
      var col2 = col + colSpan - 1;
      var hoverArea = {
        row: row,
        col: col,
        row2: row2,
        col2: col2
      };

      if (!mode || !_this.area) {
        _this.area = hoverArea;
        delete _this.from_col;
        delete _this.from_row;
        delete _this.shift_base_area;
        _this.renderActiveBox();
        return;
      }

      var _this$normalizeArea = _this.normalizeArea(),
          row_min = _this$normalizeArea.row_min,
          row_max = _this$normalizeArea.row_max,
          col_min = _this$normalizeArea.col_min,
          col_max = _this$normalizeArea.col_max;

      if (mode === 'contextmenu') {
        if (row_min <= row && row <= row_max && col_min <= col && col <= col_max) {
          return;
        }
        _this.area = hoverArea;
        delete _this.shift_base_area;
        _this.renderActiveBox();
        return;
      }

      if (mode === 'shift') {
        _this.shift_base_area = _this.shift_base_area || _this.area;
        _this.area = _this.getAdjustArea(_this.shift_base_area, hoverArea);
        _this.renderActiveBox();
        return;
      }

      switch (mode) {
        case 'drag_nw':
          _this.base_area = _this.base_area || _this._getSouthEastTdArea();
          break;

        case 'drag_se':
          _this.base_area = _this.base_area || _this._getNorthWestTdArea();
          break;

        case 'drag_n':
          _this.row_base = _this.row_base !== undefined ? _this.row_base : row_max;
          break;

        case 'drag_s':
          _this.row_base = _this.row_base !== undefined ? _this.row_base : row_min;
          break;

        case 'drag_w':
          _this.col_base = _this.col_base !== undefined ? _this.col_base : col_max;
          break;

        case 'drag_e':
          _this.col_base = _this.col_base !== undefined ? _this.col_base : col_min;
          break;

        default:
          break;
      }

      if (_this.base_area) {
        _this.area = _this.getAdjustArea(_this.base_area, hoverArea);
      }

      if (_this.col_base !== undefined) {
        _this.area.col = _this.col_base;
        _this.area.col2 = col;
      }

      if (_this.row_base !== undefined) {
        _this.area.row = _this.row_base;
        _this.area.row2 = row;
      }
      _this.renderActiveBox();
    };

    _this.getAdjustArea = function (oldArea, newArea) {
      var table = _this.tableModel.table;
      var row = oldArea.row,
          col = oldArea.col,
          row2 = oldArea.row2,
          col2 = oldArea.col2;
      var row3 = newArea.row;
      var row4 = newArea.row2;
      var col3 = newArea.col;
      var col4 = newArea.col2;
      var row_min = Math.min(row, row2, row3, row4);
      var row_max = Math.max(row, row2, row3, row4);
      var col_min = Math.min(col, col2, col3, col4);
      var col_max = Math.max(col, col2, col3, col4);
      var comboArea = {
        row: row_min,
        col: col_min,
        row2: row_max,
        col2: col_max
      };

      for (var r = row_min; r <= row_max; r++) {
        for (var c = col_min; c <= col_max; c++) {
          if (table[r][c].isEmpty) {
            var parent = table[r][c].parent;
            if (parent.row < row_min || parent.col < col_min) {
              var parentTd = table[parent.row][parent.col];
              var outerArea = {
                row: parent.row,
                col: parent.col,
                row2: parent.row + parentTd.rowSpan - 1,
                col2: parent.col + parentTd.colSpan - 1
              };
              return _this.getAdjustArea(comboArea, outerArea);
            }
          }

          if (table[r][c].isMulti) {
            var _parentTd = table[r][c];

            if (r + _parentTd.rowSpan - 1 > row_max || c + _parentTd.colSpan - 1 > col_max) {
              var _outerArea = {
                row: r,
                col: c,
                row2: r + _parentTd.rowSpan - 1,
                col2: c + _parentTd.colSpan - 1
              };
              return _this.getAdjustArea(comboArea, _outerArea);
            }
          }
        }
      }
      return comboArea;
    };

    _this.isSingleArea = function () {
      var area = _this.area,
          tableModel = _this.tableModel;
      var table = tableModel.table;

      if (!area) return false;

      var _this$normalizeArea2 = _this.normalizeArea(),
          row_min = _this$normalizeArea2.row_min,
          row_max = _this$normalizeArea2.row_max,
          col_min = _this$normalizeArea2.col_min,
          col_max = _this$normalizeArea2.col_max;

      var firstTd = table[row_min][col_min];
      if (row_min + firstTd.rowSpan - 1 === row_max && col_min + firstTd.colSpan - 1 === col_max) {
        return true;
      }
      return false;
    };

    _this.hasMergeCell = function () {
      var area = _this.area;
      if (!area) return false;
      var result = false;

      _this.each(function (td) {
        if (td.isEmpty || td.isMulti) {
          result = true;
        }
      });
      return result;
    };

    _this._getSouthEastTdArea = function () {
      var tableModel = _this.tableModel,
          area = _this.area;
      var table = tableModel.table;

      if (!area) return null;

      var _this$normalizeArea3 = _this.normalizeArea(),
          row_max = _this$normalizeArea3.row_max,
          col_max = _this$normalizeArea3.col_max;

      var td = table[row_max][col_max];
      var row = row_max;
      var col = col_max;

      if (td.isEmpty) {
        td = table[td.parent.row][td.parent.col];
        row = td.parent.row;
        col = td.parent.col;
      }

      return {
        row: row,
        col: col,
        row2: row_max,
        col2: col_max
      };
    };

    _this._getNorthWestTdArea = function () {
      var tableModel = _this.tableModel,
          area = _this.area;
      var table = tableModel.table;

      if (!area) return null;

      var _this$normalizeArea4 = _this.normalizeArea(),
          row_min = _this$normalizeArea4.row_min,
          col_min = _this$normalizeArea4.col_min;

      var td = table[row_min][col_min];
      return {
        row: row_min,
        col: col_min,
        row2: row_min + td.rowSpan - 1,
        col2: col_min + td.colSpan - 1
      };
    };

    _this.selectCol = function (index) {
      var tableModel = _this.tableModel;


      _this.area = {
        row: 0,
        row2: tableModel.rows - 1,
        col: _keymaster2.default.shift && _this.area.total_col ? _this.area.col : index,
        col2: index,
        total_col: true
      };
      _this.renderActiveBox();
    };

    _this.selectCols = function (col, col2) {
      var tableModel = _this.tableModel;

      _this.area = {
        row: 0,
        row2: tableModel.rows - 1,
        col: col,
        col2: col2,
        total_col: true
      };
      _this.renderActiveBox();
    };

    _this.selectRow = function (index) {
      var tableModel = _this.tableModel;

      _this.area = {
        row: _keymaster2.default.shift && _this.area.total_row ? _this.area.row : index,
        row2: index,
        col: 0,
        col2: tableModel.cols - 1,
        total_row: true
      };
      _this.renderActiveBox();
    };

    _this.selectRows = function (row, row2) {
      var tableModel = _this.tableModel;

      _this.area = {
        row: row,
        row2: row2,
        col: 0,
        col2: tableModel.cols - 1,
        total_row: true
      };

      _this.renderActiveBox();
    };

    _this.selectArea = function (area) {
      _this.area = area;
      _this.renderActiveBox();
    };

    _this.selectTable = function (e) {
      if (!_this.section.active) return;
      if (e && _this.section.subEngine) return;

      var area = _this.area,
          tableModel = _this.tableModel;

      if (area && area.total_table) {
        _this.clear();
        return;
      }

      _this.selectArea({
        row: 0,
        row2: tableModel.rows - 1,
        col: 0,
        col2: tableModel.cols - 1,
        total_table: true
      });
    };

    _this.selectLeft = function (e, isTab) {
      if (_this.section.subEngine) return;
      var area = _this.area,
          tableModel = _this.tableModel;

      if (!area) return;

      var _this$normalizeArea5 = _this.normalizeArea(),
          row_min = _this$normalizeArea5.row_min,
          col_min = _this$normalizeArea5.col_min;

      var new_row = _this.from_row || row_min;
      var new_col = Math.max(col_min - 1, 0);
      _this.from_row = _this.from_row || new_row;
      if (col_min === 0 && isTab && new_row > 0) {
        new_col = tableModel.cols - 1;
        new_row -= 1;
        _this.from_row = new_row;
      }
      delete _this.from_col;
      _this.selectTd(new_row, new_col);
    };

    _this.selectRight = function (e, isTab) {
      if (_this.section.subEngine) return;
      var area = _this.area,
          tableModel = _this.tableModel;

      if (!area) return;

      var _this$normalizeArea6 = _this.normalizeArea(),
          row_min = _this$normalizeArea6.row_min,
          col_max = _this$normalizeArea6.col_max;

      var new_row = _this.from_row || row_min;
      var new_col = Math.min(col_max + 1, tableModel.cols - 1);
      _this.from_row = _this.from_row || new_row;

      if (col_max + 1 === tableModel.cols && isTab) {
        new_col = 0;
        new_row += 1;
        if (new_row === tableModel.rows) {
          _this.section.command.insertRow('end');
        }
        _this.from_row = new_row;
      }
      delete _this.from_col;
      _this.selectTd(new_row, new_col);
    };

    _this.selectUp = function () {
      if (_this.section.subEngine) return;
      var area = _this.area;

      if (!area) return;

      var _this$normalizeArea7 = _this.normalizeArea(),
          row_min = _this$normalizeArea7.row_min,
          col_min = _this$normalizeArea7.col_min;

      var new_row = Math.max(row_min - 1, 0);
      var new_col = _this.from_col || col_min;
      _this.from_col = _this.from_col || col_min;
      delete _this.from_row;
      _this.selectTd(new_row, new_col);
    };

    _this.selectDown = function () {
      if (_this.section.subEngine) return;
      var area = _this.area,
          tableModel = _this.tableModel;

      if (!area) return;

      var _this$normalizeArea8 = _this.normalizeArea(),
          row_max = _this$normalizeArea8.row_max,
          col_min = _this$normalizeArea8.col_min;

      var new_row = Math.min(row_max + 1, tableModel.rows - 1);
      var new_col = _this.from_col || col_min;
      _this.from_col = _this.from_col || col_min;
      delete _this.from_row;
      _this.selectTd(new_row, new_col);
    };

    _this.selectTab = function (e) {
      if (_this.section.subEngine) {
        _this.section.removeEditor();
      }

      e.preventDefault();
      if (_keymaster2.default.shift) {
        _this.selectLeft(e, true);
      } else {
        _this.selectRight(e, true);
      }
    };

    _this.selectEnter = function (e) {
      if (_this.section.subEngine) return;
      e.preventDefault();
      _this.selectDown();
    };

    _this.selectTd = function (row, col) {
      var tableModel = _this.tableModel;


      var table = tableModel.table;
      var tdModel = table[row][col];

      if (tdModel.isEmpty) {
        var parentTd = table[tdModel.parent.row][tdModel.parent.col];

        _this.selectArea({
          row: tdModel.parent.row,
          col: tdModel.parent.col,
          row2: tdModel.parent.row + parentTd.rowSpan - 1,
          col2: tdModel.parent.col + parentTd.colSpan - 1
        });
        return;
      }

      _this.selectArea({
        row: row,
        col: col,
        row2: row + tdModel.rowSpan - 1,
        col2: col + tdModel.colSpan - 1
      });

      var left = parseInt(_this.activeBox.css('left'));

      if (left < _this.viewport[0].scrollLeft) {
        _this.viewport[0].scrollLeft = left;
      } else if (left + 80 > _this.viewport[0].scrollLeft + _this.viewport[0].offsetWidth) {
        _this.viewport[0].scrollLeft = left - _this.viewport[0].offsetWidth + 80;
      }
    };

    _this.renderActiveBox = function () {
      _this.hideSubEditor();
      _this.reRenderActiveBox();
      // 这里需要每次都触发，否则头部不会选中，性能的优化放到后续操作中
      _this.emit('select');
    };

    _this.reRenderActiveBox = function (actionPreview) {
      var area = _this.area,
          tableModel = _this.tableModel;

      var template = _this.section.template;
      if (!area) return;
      _this.single = _this.isSingleArea();
      var table = tableModel.table;

      var _this$normalizeArea9 = _this.normalizeArea(),
          row_min = _this$normalizeArea9.row_min,
          row_max = _this$normalizeArea9.row_max,
          col_min = _this$normalizeArea9.col_min,
          col_max = _this$normalizeArea9.col_max;

      var left = void 0;
      var top = void 0;
      var width = void 0;
      var height = void 0;

      if (_this.single) {
        var td = table[row_min][col_min].element;
        left = td.offsetLeft;
        top = td.parentNode.offsetTop;
        width = td.offsetWidth;
        height = td.offsetHeight;
      } else {
        //const colBars = this.colsHeader.find(template.COLS_HEADER_ITEM_CLASS)
        //const rowBars = this.rowsHeader.find(template.ROWS_HEADER_ITEM_CLASS)
        //left = colBars[col_min].offsetLeft
        //top = rowBars[row_min].offsetTop
        //width = colBars[col_max].offsetLeft - left + colBars[col_max].offsetWidth
        //height = rowBars[row_max].offsetTop - top + rowBars[row_max].offsetHeight

        var tdLeft = table[0][col_min].element.offsetLeft;
        var tdTop = table[row_min][0].element.offsetTop;

        left = tdLeft;
        top = tdTop;

        var tdWidth = table[0][col_max].element.offsetLeft - left + table[0][col_max].element.offsetWidth;
        var tdHeight = table[row_max][0].element.offsetTop - top + table[row_max][0].element.offsetHeight;

        width = tdWidth;
        height = tdHeight;
      }

      _this.showActiveBox(left, top, height, width, actionPreview);
      _this.areaWidth = width;
      _this.areaHeight = height;
      _this.areaLeft = left;
      _this.areaTop = top;
      _this.reRenderSubEditor();
      _this.td = _this.single ? table[row_min][col_min].element : null;
    };

    _this.showActiveBox = function (left, top, height, width, actionPreview) {
      var activeBox = _this.activeBox,
          hideTextarea = _this.hideTextarea;

      var l = activeBox.find('.l');
      var r = activeBox.find('.r');
      var t = activeBox.find('.t');
      var b = activeBox.find('.b');
      var mask = activeBox.find('.mask');
      var maskCol = activeBox.find('.mask-col');
      var maskRow = activeBox.find('.mask-row');
      var se = activeBox.find('.se');
      var n = activeBox.find('.n');
      var s = activeBox.find('.s');
      var w = activeBox.find('.w');
      var e = activeBox.find('.e');
      activeBox.removeClass('col');
      activeBox.removeClass('row');
      activeBox.removeClass('total');
      activeBox.removeClass('preview-mask-col');
      activeBox.removeClass('preview-mask-row');

      if (_this.area.total_col) {
        activeBox.addClass('col');
      }

      if (_this.area.total_row) {
        activeBox.addClass('row');
      }

      if (_this.area.total_table) {
        activeBox.addClass('total');
      }

      if (actionPreview === 'removeCol') {
        activeBox.addClass('preview-mask-col');
      }

      if (actionPreview === 'removeRow') {
        activeBox.addClass('preview-mask-row');
      }
      var table = _this.tableModel.table;
      l.css('height', ''.concat(height, 'px'));
      r.css('height', ''.concat(height + 1, 'px'));
      r.css('left', ''.concat(width, 'px'));
      t.css('width', ''.concat(width, 'px'));
      b.css('top', ''.concat(height, 'px'));
      b.css('width', ''.concat(width, 'px'));
      se.css('left', ''.concat(width - 3, 'px'));
      se.css('top', ''.concat(height - 3, 'px'));
      n.css('top', '-4px');
      n.css('left', ''.concat(width / 2 - 3, 'px'));
      s.css('top', ''.concat(height - 3, 'px'));
      s.css('left', ''.concat(width / 2 - 3, 'px'));
      w.css('left', '-4px');
      w.css('top', ''.concat(height / 2 - 3, 'px'));
      e.css('left', ''.concat(width - 3, 'px'));
      e.css('top', ''.concat(height / 2 - 3, 'px'));
      mask.css('width', ''.concat(width, 'px'));
      mask.css('height', ''.concat(height, 'px'));
      maskCol.css('width', ''.concat(width + 1, 'px'));
      //maskCol.css('height', "".concat(this.rowsHeader[0].offsetHeight - 33, "px"))
      maskCol.css('height', ''.concat(table[0][0].element.offsetHeight - 33, 'px'));
      maskCol.css('top', '-'.concat(top, 'px'));
      maskRow.css('height', ''.concat(height + 1, 'px'));
      //maskRow.css('width', "".concat(this.colsHeader[0].offsetWidth, "px"))
      maskRow.css('width', ''.concat(table[0][0].element.offsetWidth, 'px'));
      maskRow.css('left', '-'.concat(left, 'px'));
      activeBox.css('left', ''.concat(left, 'px'));
      activeBox.css('top', ''.concat(top, 'px'));
      activeBox.css('display', 'block');
      hideTextarea.css('left', ''.concat(_this.clickX + 10, 'px'));
      hideTextarea.css('top', ''.concat(_this.clickY - 10, 'px'));
    };

    _this.showSubEditor = function () {
      _this.reRenderSubEditor();
      _this.container.css('display', 'table');
      _this.activeBox.addClass('editing');
    };

    _this.reRenderSubEditor = function () {
      _this.editAreaContent = _this.container.find(_this.section.template.SUB_EDITOR_CONTENT_CLASS);
      _this.container.css('width', ''.concat(_this.areaWidth - 1, 'px'));
      _this.editAreaContent.css('width', ''.concat(_this.areaWidth - 2, 'px'));
      _this.editAreaContent.css('max-width', ''.concat(_this.areaWidth - 2, 'px'));
      _this.editAreaContent.css('height', ''.concat(_this.areaHeight, 'px'));
      (0, _utils.copyCss)(_this.td, _this.editAreaContent);
    };

    _this.hideSubEditor = function () {
      var template = _this.section.template;
      _this.container.html('');
      _this.container[0].remove();
      _this.container = $(template.SubEditor);
      _this.activeBox.append(_this.container);
      _this.activeBox.removeClass('editing');
    };

    _this.hideActiveBox = function () {
      _this.activeBox.css('display', 'none');
    };

    _this.startDragTriggerToSelectCell = function (e) {
      var template = _this.section.template;
      var dragTrigger = $(e.target).closest(template.ACTIVE_TD_TRIGGER_CLASS);
      if (!dragTrigger[0]) return;
      e.preventDefault();
      var direction = dragTrigger.attr('direction');
      _this.dragSelecting = true;
      _this.tableWrapper.addClass('drag-selecting');
      _this.dragDirection = direction;
      _this.addDragEvent();
    };

    _this.onTdMouseDown = function (e) {
      _keymaster2.default.shift = e.shiftKey;
      var td = $(e.target).closest('td');
      if (!td[0]) return;
      var mode = void 0;

      if (_keymaster2.default.shift) {
        mode = 'shift';
      }

      if (e.button === 2) {
        mode = 'contextmenu';
      }
      // active 状态的 mousedown 阻止冒泡，可以避免外部编辑器的一些事件带来的性能开销
      if (_this.section.active) {
        e.stopPropagation();
      }
      // 记录下鼠标点击的位置，隐藏的输入框改为 fixed 定位，就定位在这个位置
      // 改为 fixed 是为了增加行列时不滚动页面
      _this.clickX = e.clientX;
      _this.clickY = e.clientY;
      // 已经选中单元格，第二次点击激活编辑器，可以进行编辑
      var activeClick = _this.td === td[0];
      var isEditing = _this.section.isEditing(td);
      if (activeClick && !isEditing && !mode) {
        _this.emit('active', td);
        return;
      }
      // 当前单元格处于编辑态，再次点击不做任何事

      if (_this.section.isEditing(td)) {
        return;
      }
      // 点击非编辑态的单元格，需要阻止默认行为，否则会选中文本，而表格里的选区是自己控制的，非文本流
      e.preventDefault();
      _this.selectCell(td[0], mode);
      // 右键不触发拖选
      if (mode === 'contextmenu') return;
      _this.delayStartDrag(td);
    };

    _this.delayStartDrag = function (td) {
      if (_this.dragTimer) {
        clearTimeout(_this.dragTimer);
      }
      _this.section.container.on('mouseup', _this.cancelStartDrag);
      _this.dragTimer = setTimeout(function () {
        // mousedown 之后可以拖选
        _this.tableWrapper.addClass('drag-selecting');
        _this.dragSelecting = true;
        _this.dragDirection = 'drag_se';
        _this.dragoverTd = td[0];
        _this.addDragEvent();
      }, 100);
    };

    _this.cancelStartDrag = function () {
      if (_this.dragTimer) {
        clearTimeout(_this.dragTimer);
        _this.section.container.off('mouseup', _this.cancelStartDrag);
      }
    };

    _this.addDragEvent = function () {
      _this.section.tableRoot.addClass('drag-select');
      _this.section.tableRoot.addClass(_this.dragDirection);
      _this.areaMask.addClass(_this.dragDirection);
      _this.section.container.on('mouseup', _this.dragEnd).on('mouseleave', _this.delayDragEnd).on('mousemove', _this.dragToSelectCell);
    };

    _this.removeDragEvent = function () {
      _this.section.tableRoot.removeClass('drag-select');
      _this.section.tableRoot.removeClass(_this.dragDirection);
      _this.areaMask.removeClass(_this.dragDirection);
      _this.section.container.off('mouseup', _this.dragEnd).off('mouseleave', _this.delayDragEnd).off('mousemove', _this.dragToSelectCell);
    };

    _this.dragToSelectCell = function (e) {
      if (!_this.dragSelecting) return;
      if (_this.timer) clearTimeout(_this.timer);
      var dragoverTd = $(e.target).closest('td');
      if (!dragoverTd[0]) return;
      if (!dragoverTd.isRoot()) {
        e.preventDefault();
      }
      if (dragoverTd[0] !== _this.dragoverTd) {
        _this.selectCell(dragoverTd[0], _this.dragDirection);
        _this.dragoverTd = dragoverTd[0];
      }
    };

    _this.dragEnd = function () {
      _this.tableWrapper.removeClass('drag-selecting');
      if (_this.dragSelecting) {
        _this.dragSelecting = false;
        delete _this.base_area;
        delete _this.row_base;
        delete _this.col_base;
        _this.removeDragEvent();
      }
    };

    _this.delayDragEnd = function () {
      _this.timer = setTimeout(_this.dragEnd, 500);
    };

    _this.preview = function (action) {
      switch (action) {
        case 'removeCol':
        case 'removeRow':
          _this.reRenderActiveBox(action);
          break;

        default:
          _this.reRenderActiveBox();
          break;
      }
    };

    _this.clear = function () {
      _this.area = null;
      _this.td = null;
      _this.hideActiveBox();
      _this.emit('cancelSelect');
    };

    _this.normalizeArea = function () {
      if (!_this.area) return;
      var _this$area3 = _this.area,
          row = _this$area3.row,
          col = _this$area3.col,
          row2 = _this$area3.row2,
          col2 = _this$area3.col2;

      var row_min = Math.min(row, row2);
      var row_max = Math.max(row, row2);
      var col_min = Math.min(col, col2);
      var col_max = Math.max(col, col2);
      return {
        row_min: row_min,
        row_max: row_max,
        col_min: col_min,
        col_max: col_max
      };
    };

    _this.getEffectRows = function () {
      if (!_this.area) return;

      var _this$normalizeArea10 = _this.normalizeArea(),
          row_min = _this$normalizeArea10.row_min,
          row_max = _this$normalizeArea10.row_max;

      var table = _this.tableModel.table;
      var row_min_min = row_min;
      var row_max_max = row_max;
      var trMin = table[row_min];
      var trMax = table[row_max];
      trMin.forEach(function (td) {
        if (td.isEmpty) {
          var parentRow = td.parent.row;
          var parentCol = td.parent.col;
          row_min_min = Math.min(row_min_min, parentRow);
          row_max_max = Math.max(row_max_max, parentRow + table[parentRow][parentCol].rowSpan - 1);
        }
      });
      trMax.forEach(function (td) {
        if (td.isEmpty) {
          var parentRow = td.parent.row;
          var parentCol = td.parent.col;
          row_max_max = Math.max(row_max_max, parentRow + table[parentRow][parentCol].rowSpan - 1);
        }

        if (td.isMulti) {
          row_max_max = Math.max(row_max_max, row_max + td.rowSpan - 1);
        }
      });
      return {
        row_min: row_min_min,
        row_max: row_max_max
      };
    };

    _this.each = function (fn, reverse) {
      var area = _this.area,
          tableModel = _this.tableModel;

      var table = tableModel.table;
      if (!area) return;

      var _this$normalizeArea11 = _this.normalizeArea(),
          row_min = _this$normalizeArea11.row_min,
          row_max = _this$normalizeArea11.row_max,
          col_min = _this$normalizeArea11.col_min,
          col_max = _this$normalizeArea11.col_max;

      if (reverse) {
        for (var r = row_max; r >= row_min; r--) {
          for (var c = col_max; c >= col_min; c--) {
            var tdModel = table[r][c];
            fn(tdModel, r, c);
          }
        }
      } else {
        for (var _r = row_min; _r <= row_max; _r++) {
          for (var _c = col_min; _c <= col_max; _c++) {
            var _tdModel = table[_r][_c];
            fn(_tdModel, _r, _c);
          }
        }
      }
    };

    _this.getTdIndex = function (row, col) {
      var tableModel = _this.tableModel;

      var table = tableModel.table;
      var trModel = table[row];
      var index = 0;

      for (var i = 0; i < col; i++) {
        if (trModel[i].element) {
          index++;
        }
      }
      return index;
    };

    _this.getTdElement = function (row, col) {
      var table = _this.tableModel.table;
      var tdModel = table[row][col];

      if (tdModel.element) {
        return tdModel.element;
      }

      if (tdModel.isEmpty) {
        return table[tdModel.parent.row][tdModel.parent.col].element;
      }
    };

    _this.getTdRowCol = function (td) {
      var table = _this.tableModel.table;
      var tr = td.parentNode;
      var row = tr.rowIndex;
      var trModel = table[row];
      var col = 0;
      trModel.forEach(function (tdModel, c) {
        if (tdModel.element === td) {
          col = c;
        }
      });
      return {
        row: row,
        col: col
      };
    };

    _this.getSelectionHtml = function () {
      var area = _this.area,
          tableModel = _this.tableModel,
          section = _this.section;

      var table = tableModel.table;
      var tableRoot = section.tableRoot;
      if (!area) return null;

      var _this$normalizeArea12 = _this.normalizeArea(),
          row_min = _this$normalizeArea12.row_min,
          row_max = _this$normalizeArea12.row_max,
          col_min = _this$normalizeArea12.col_min,
          col_max = _this$normalizeArea12.col_max;

      var colsEl = tableRoot.find('col');
      var cols = [];
      var tableWidth = 0;

      for (var c = col_min; c <= col_max; c++) {
        cols.push('<col width="'.concat(colsEl[c].width, '" />'));
        tableWidth += parseInt(colsEl[c].width);
      }

      var colgroup = '<colgroup>'.concat(cols.join(''), '</colgroup>');
      var trHtml = [];

      for (var r = row_min; r <= row_max; r++) {
        var tdHtml = [];

        for (var _c2 = col_min; _c2 <= col_max; _c2++) {
          var tdModel = table[r][_c2];
          var rowRemain = undefined;
          var colRemain = undefined;
          var tdClone = undefined;

          if (tdModel.element) {
            tdClone = tdModel.element.cloneNode(true);
          }

          if (tdModel.isMulti) {
            // 合并单元格尾部被选区切断的情况，需要重新计算合并单元格的跨度
            rowRemain = Math.min(r + tdModel.rowSpan - 1, row_max) - r + 1;
            colRemain = Math.min(_c2 + tdModel.colSpan - 1, col_max) - _c2 + 1;
          }

          if (tdModel.isEmpty) {
            var parentTd = table[tdModel.parent.row][tdModel.parent.col];
            // 选区中含有合并单元格的一部分时，需要补充这一部分的dom结构，这种情况只会出现在行列选择时
            // 列选择时，切断合并单元格后，第一个和父单元格同行，并在选取左测第一个列的位置，补充此单元格
            if (tdModel.parent.row === r && tdModel.parent.col < col_min && _c2 === col_min) {
              var colCut = col_min - tdModel.parent.col;
              colRemain = Math.min(parentTd.colSpan - colCut, col_max - col_min + 1);
              rowRemain = parentTd.rowSpan;
              tdClone = parentTd.element.cloneNode(true);
            }
            // 行选择时，切断合并单元格后，第一个和父单元格同列，并在选取上测第一个行的位置，补充此单元格
            if (tdModel.parent.col === _c2 && tdModel.parent.row < row_min && r === row_min) {
              var rowCut = row_min - tdModel.parent.row;
              rowRemain = Math.min(parentTd.rowSpan - rowCut, row_max - row_min + 1);
              colRemain = parentTd.colSpan;
              tdClone = parentTd.element.cloneNode(true);
            }
          }

          if (tdClone) {
            tdClone.setAttribute('rowspan', rowRemain);
            tdClone.setAttribute('colspan', colRemain);
            tdHtml.push(tdClone.outerHTML);
          }
        }
        trHtml.push('<tr>'.concat(tdHtml.join(''), '</tr>'));
      }

      return '<table style="width: '.concat(tableWidth, 'px">').concat(colgroup).concat(trHtml.join(''), '</table>');
    };

    _this.refreshModel = function () {
      _this.tableModel = (0, _utils.getTableModel)(_this.section.tableRoot[0]);
    };

    _this.cancelSelect = function () {
      if (_this.section.subEngine) {
        _this.emit('cancelSelect');
        return;
      }
      _this.clear();
    };

    _this.onClickMask = function (e) {
      if (e.button === 2) return;
      var offsetX = e.offsetX,
          offsetY = e.offsetY;

      var td = _this.getTdByMaskXY(offsetX, offsetY);
      _this.selectCell(td);
    };

    _this.onMouseMoveMask = function (e) {
      if (!_this.dragSelecting) return;
      var offsetX = e.offsetX,
          offsetY = e.offsetY;

      var td = _this.getTdByMaskXY(offsetX, offsetY);
      _this.selectCell(td, _this.dragDirection);
    };

    _this.getTdByMaskXY = function (x, y) {
      var tableModel = _this.tableModel;

      var template = _this.section.template;

      var _this$normalizeArea13 = _this.normalizeArea(),
          row_min = _this$normalizeArea13.row_min,
          col_min = _this$normalizeArea13.col_min;
      //const colBars = this.colsHeader.find(template.COLS_HEADER_ITEM_CLASS)
      //const rowBars = this.rowsHeader.find(template.ROWS_HEADER_ITEM_CLASS)

      var row = row_min;
      var col = col_min;
      //let w = colBars[col].offsetWidth
      //let h = rowBars[row].offsetHeight
      var w = tableModel.table[0][col].element.offsetWidth;
      var h = tableModel.table[row][0].element.offsetHeight;
      while (w < x) {
        col++;
        //w += colBars[col].offsetWidth
        w += tableModel.table[0][col].offsetWidth;
      }

      while (h < y) {
        row++;
        //h += rowBars[row].offsetHeight
        h += tableModel.table[row][0].element.offsetHeight;
      }
      var td = tableModel.table[row][col];
      if (td.isEmpty) {
        td = tableModel.table[td.parent.row][td.parent.col];
      }
      return td.element;
    };

    _this.render = function (action) {
      _this.refreshModel();
      var tableModel = _this.tableModel,
          area = _this.area;

      if (!area) {
        return;
      }

      var _this$normalizeArea14 = _this.normalizeArea(),
          row_min = _this$normalizeArea14.row_min,
          col_min = _this$normalizeArea14.col_min;

      switch (action) {
        case 'removeRow':
          _this.selectRow(Math.min(row_min, tableModel.rows - 1));
          break;
        case 'removeCol':
          _this.selectCol(Math.min(col_min, tableModel.cols - 1));
          break;
        default:
          _this.renderActiveBox();
      }
    };
    _this.section = _section;
    return _this;
  }

  _createClass(Selection, [{
    key: 'isRowSelected',
    value: function isRowSelected() {
      return this.area && (this.area.total_row || this.area.total_table);
    }
  }, {
    key: 'isColSelected',
    value: function isColSelected() {
      return this.area && (this.area.total_col || this.area.total_table);
    }
  }, {
    key: 'isTableSelected',
    value: function isTableSelected() {
      return this.area && this.area.total_table;
    }
  }, {
    key: 'bindEvents',
    value: function bindEvents() {
      this.section.container.on('mousedown', this.onTdMouseDown).on('dragstart', function (e) {
        return e.stopPropagation();
      }).on('contextmenu', function (e) {
        return e.preventDefault();
      }).on('select', function (e) {
        return e.preventDefault();
      });
      this.activeBox.on('mousedown', this.startDragTriggerToSelectCell);
      this.areaMask.on('mousedown', this.onClickMask).on('mousemove', this.onMouseMoveMask);
    }
  }, {
    key: 'init',
    value: function init() {
      var _section2 = this.section,
          container = _section2.container,
          template = _section2.template;

      this.activeBox = container.find(template.ACTIVE_TD_CLASS);
      //this.colsHeader = container.find(template.COLS_HEADER_CLASS)
      //this.rowsHeader = container.find(template.ROWS_HEADER_CLASS)
      this.viewport = container.find(template.VIEWPORT);
      this.container = this.activeBox.find(template.SUB_EDITOR_CLASS);
      this.tableWrapper = container.find(template.TABLE_WRAPPER_CLASS);
      this.editAreaContent = this.container.find(this.section.template.SUB_EDITOR_CONTENT_CLASS);
      this.areaMask = this.activeBox.find(template.AREA_MASK);
      this.hideTextarea = this.activeBox.find(template.TABLE_TEXTAREA_CLASS);
      this.refreshModel();
      this.bindEvents();
    }
  }]);

  return Selection;
}(_eventemitter.EventEmitter2);

exports.default = Selection;