'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _reactDom = require('react-dom');

var _reactDom2 = _interopRequireDefault(_reactDom);

var _tinsoEditorEngine = require('tinso-editor-engine');

var _tinsoEditorEngine2 = _interopRequireDefault(_tinsoEditorEngine);

var _textDiagramEditor = require('./text-diagram-editor');

var _textDiagramEditor2 = _interopRequireDefault(_textDiagramEditor);

var _textDiagramRender = require('./text-diagram-render');

var _textDiagramRender2 = _interopRequireDefault(_textDiagramRender);

var _imageGenerator = require('../../utils/image-generator');

var _base = require('../base');

var _base2 = _interopRequireDefault(_base);

var _constants = require('./constants');

var _constants2 = _interopRequireDefault(_constants);

require('./index.css');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var EDITOR_LAYOUT = _constants2.default.EDITOR_LAYOUT,
    TEXT_DIAGRAM_TYPES = _constants2.default.TEXT_DIAGRAM_TYPES;
var $ = _tinsoEditorEngine2.default.$;

var template = function template() {
  return '<div class="zacheditor-text-diagram zacheditor-text-diagram-root"></div>';
};

/**
 * 文本图Section
 */

var TextDiagram = function (_SectionBase) {
  _inherits(TextDiagram, _SectionBase);

  function TextDiagram(engine, contentView) {
    _classCallCheck(this, TextDiagram);

    var _this = _possibleConstructorReturn(this, (TextDiagram.__proto__ || Object.getPrototypeOf(TextDiagram)).call(this));

    _this.focusToCodeEditor = function () {
      if (_this.editor) {
        _this.editor.focus();
      }
    };

    _this.onEditorChange = function (code) {
      if (code !== _this.value.code) {
        _this.value.code = code;
        // 编辑区域有变更时，清除 src 属性
        _this.value.src = null;
      }
    };

    _this.changeDiagramType = function (type) {
      _this.diagramType = type;
      _this.prevValue = {
        src: null,
        code: null
      };
    };

    _this.onEditorBlur = function () {
      // 激活工具栏
      _this.engine.toolbar.disable(false);
      // 保存代码内容
      _this.saveDiagram();
    };

    _this.onEditorFocus = function () {
      //debug('editor focus');
      // 置灰工具栏
      _this.engine.toolbar.disable(true);
    };

    _this.needConvertToImage = function () {
      // 有用户输入时，没有 src 时不需要重新生成图片
      if (_this.value.code) {
        return !_this.value.src;
      }
      // 没有用户输入时，判断 prevValue 是否需要生成 src，兼容断网等情况下导致的 src 生成失败
      return _this.prevValue.code && !_this.prevValue.src;
    };

    _this.saveDiagram = function () {
      // tests 中执行 engine.destroy 时会触发 Error: Uncaught TypeError: Cannot read property 'code' of undefined
      // 疑似执行过快导致相关回调执行时 value 已经销毁了，故先判断 value 是否存在
      if (_this.value && _this.value.code !== null) {
        _this.setValue({
          code: _this.value.code,
          url: _this.value.url,
          type: _this.diagramType
        });
      }
    };

    _this.genImage = function (code) {
      return (0, _imageGenerator.preview)(_this.diagramType, code).then(function (res) {
        // 图片成功生成，设置 src 属性
        var src = 'data:image/svg+xml,' + encodeURIComponent(res.svg).replace(/'/g, '%27').replace(/"/g, '%22');
        if (res.success) {
          _this.value.src = src;
        }
        _this.saveDiagram();
        _this.prevValue = _this.value;
        _this.engine.history.update(true);
      });
    };

    _this.onBeforeSave = function () {
      // 有编辑过并且没有生成 src 时生成图片
      if (_this.value.code && !_this.value.src) {
        return _this.genImage(_this.value.code);
      }
      return null;
    };

    _this.getDiagramType = function (value) {
      return 'diagram' === _this.name ? value.type || TEXT_DIAGRAM_TYPES.PUML : _this.name;
    };

    _this.engine = engine;
    _this.contentView = contentView; // 编辑状态中的 section 数据
    _this.value = {
      // 文本图对应的图片地址
      src: null,
      // 文本图源代码
      code: null
    };
    // section 正式生效的数据
    _this.prevValue = {
      src: null,
      code: null
    };
    // 根节点，React 的宿主节点
    _this.root = null; // 图的类别
    _this.diagramType = null; // 编辑区域 Editor 实例
    _this.editor = null;
    return _this;
  }

  _createClass(TextDiagram, [{
    key: 'embedToolbar',
    value: function embedToolbar() {
      var config = (this.engine ? this.engine.options.diagram : this.contentView.options.diagram) || {};
      var embed = [{
        type: 'dnd'
      }, {
        type: 'maximize'
      }, {
        type: 'separator'
      }, {
        type: 'copy'
      }, {
        type: 'delete'
      }];
      if (Array.isArray(config.embed)) {
        return config.embed;
      } else if (_typeof(config.embed) === 'object') {
        var embedArray = [];
        embed.forEach(function (item) {
          if (config.embed[item.type] !== false) {
            embedArray.push(item);
          }
        });
        return embedArray;
      } else {
        return embed;
      }
    }
  }, {
    key: 'destroy',
    value: function destroy() {
      this.engine.asyncEvent.off('save:before', this.onBeforeSave);
    }
    // 最大化

  }, {
    key: 'maximize',
    value: function maximize() {
      if (this.state.readonly) return;
      this.editor.switchLayout(EDITOR_LAYOUT.TWO_COLUMN);
      this.editor.doPreview();
      this.engine.toolbar.hide();
    }
    // 还原

  }, {
    key: 'restore',
    value: function restore() {
      if (this.state.readonly) return;
      this.editor.switchLayout(EDITOR_LAYOUT.DEFAULT);
      this.engine.toolbar.show();
    }
  }, {
    key: 'activate',
    value: function activate() {
      if (this.state.readonly) return;
      if (this.resizeController) {
        this.resizeController.show();
      }
    }
  }, {
    key: 'unactivate',
    value: function unactivate() {
      this.saveDiagram();
      this.engine.history.save();
      if (this.needConvertToImage()) {
        this.genImage(this.value.code);
      }
      if (this.state.readonly) return;
      if (this.resizeController) {
        this.resizeController.hide();
      }
    }
  }, {
    key: 'didInsert',
    value: function didInsert() {
      var _this2 = this;

      window.setTimeout(function () {
        _this2.focusToCodeEditor();
      }, 10);
    }

    /**
     * 渲染编辑器
     * @param {object} value section 的取值
     */

  }, {
    key: 'renderEditor',
    value: function renderEditor(value) {
      var _this3 = this;

      _reactDom2.default.render(_react2.default.createElement(_textDiagramEditor2.default, {
        canChangeType: 'diagram' === this.name,
        type: this.diagramType,
        code: value.code || '',
        onFocus: this.onEditorFocus,
        onBlur: this.onEditorBlur,
        onChange: this.onEditorChange,
        onDiagramTypeChange: this.changeDiagramType,
        onBeforeRenderImage: this.onBeforeRenderImage,
        isMaximize: function isMaximize() {
          return _this3.state.maximize;
        },
        onLoad: function onLoad(ref) {
          _this3.editor = ref.editor;
          _this3.container.on('click', function (e) {
            var hasSelect = $(e.target).closest('.ant-select');
            if (!hasSelect || hasSelect.length === 0) _this3.focusToCodeEditor();
          });
        },
        locale: this.locale
      }), this.root[0], function () {
        var diagram = _this3.root.find('.zacheditor-text-diagram-editor');
        var height = _this3.value.height;

        _this3.addResizeController(diagram);
        if (height) {
          diagram.css('height', height + 'px');
        }
      });
    }
    /**
     * 渲染阅读界面
     * @param {object} value section 的取值
     */

  }, {
    key: 'renderViewer',
    value: function renderViewer(value) {
      _reactDom2.default.render(_react2.default.createElement(
        'div',
        { className: 'zacheditor-text-diagram-viewer' },
        _react2.default.createElement(_textDiagramRender2.default, {
          type: this.diagramType,
          code: value.code,
          onBeforeRenderImage: this.onBeforeRenderImage,
          src: value.src
        })
      ), this.root[0]);
    }
  }, {
    key: 'render',
    value: function render(container, value) {
      if (value) {
        this.prevValue = value;
      }

      container.append(template());
      this.root = this.container.find('.zacheditor-text-diagram-root');
      // section name 即为图的类别
      this.diagramType = this.getDiagramType(value);
      if (this.engine) {
        this.locale = this.engine.locale.textDiagram;
        // 执行 save:before 时保存内容，保证输入内容能被及时存储
        this.engine.on('save:before', this.saveDiagram);
        // 异步执行 save:before 执行生成图片操作
        this.engine.asyncEvent.on('save:before', this.onBeforeSave);
        this.renderEditor(this.prevValue);
      } else {
        this.renderViewer(this.prevValue);
      }
    }
  }]);

  return TextDiagram;
}(_base2.default);

TextDiagram.type = 'block';
TextDiagram.uid = true;
exports.default = TextDiagram;