'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _classnames = require('classnames');

var _classnames2 = _interopRequireDefault(_classnames);

var _codemirror = require('codemirror');

var _codemirror2 = _interopRequireDefault(_codemirror);

var _debounce = require('lodash/debounce');

var _debounce2 = _interopRequireDefault(_debounce);

var _antd = require('antd');

require('antd/lib/dropdown/style');

require('antd/lib/icon/style');

require('antd/lib/menu/style');

require('antd/lib/select/style');

require('antd/lib/tooltip/style');

var _template = require('./template');

var _template2 = _interopRequireDefault(_template);

var _constants = require('./constants');

var _constants2 = _interopRequireDefault(_constants);

var _icons = require('./icons');

var _icons2 = _interopRequireDefault(_icons);

var _textDiagramRender = require('./text-diagram-render');

var _textDiagramRender2 = _interopRequireDefault(_textDiagramRender);

var _tinsoEditorEngine = require('tinso-editor-engine');

var _tinsoEditorEngine2 = _interopRequireDefault(_tinsoEditorEngine);

var _icons3 = require('@ant-design/icons');

var _icons4 = _interopRequireDefault(_icons3);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var macos = _tinsoEditorEngine2.default.userAgent.macos;
var EDITOR_MODE = _constants2.default.EDITOR_MODE,
    EDITOR_LAYOUT = _constants2.default.EDITOR_LAYOUT,
    TEXT_DIAGRAM_GUIDES = _constants2.default.TEXT_DIAGRAM_GUIDES,
    TEXT_DIAGRAM_SYNTAX = _constants2.default.TEXT_DIAGRAM_SYNTAX,
    TEXT_DIAGRAM_TYPES = _constants2.default.TEXT_DIAGRAM_TYPES,
    TEXT_DIAGRAMS = _constants2.default.TEXT_DIAGRAMS;

var TextDiagramEditor = function (_React$PureComponent) {
  _inherits(TextDiagramEditor, _React$PureComponent);

  function TextDiagramEditor(props) {
    _classCallCheck(this, TextDiagramEditor);

    var _this = _possibleConstructorReturn(this, (TextDiagramEditor.__proto__ || Object.getPrototypeOf(TextDiagramEditor)).call(this, props));

    _this.switchType = function (type) {
      if (type !== _this.state.type) {
        var syntax = _this.getCodeMirrorMode(type);
        _this.setState({
          type: type,
          mode: EDITOR_MODE.CODE,
          codeForPreview: null
        });
        _this.instance.codeMirror.setOption('mode', syntax);
        _this.props.onDiagramTypeChange(type);
      }
    };

    _this.switchLayout = function (layout) {
      _this.setState({
        layout: layout,
        mode: EDITOR_MODE.CODE
      });

      var rootCenter = _tinsoEditorEngine2.default.$(_this.rootRef.current).closest('div[data-section-element=center]');
      var sectionResize = rootCenter.find('div.section-resize-button-ud');

      if (_this.instance) {
        _this.instance.focus();
      }
      // 在预览模式下切换为双栏模式时，重置 body 区域高度
      if (layout === EDITOR_LAYOUT.TWO_COLUMN) {
        var body = _this.bodyRef.current;
        body.style.height = '100%';
        sectionResize.hide();
      } else {
        sectionResize.show();
      }
    };

    _this.switchTemplate = function (id) {
      // console.log(`switch tpl: ${id}`);
      var template = _template2.default.getTemplates(_this.state.type);
      if (template) _this.instance.codeMirror.setValue(template[id].text);
    };

    _this.switchView = function (mode) {
      var value = _this.instance.getValue();
      _this.setState({
        mode: mode,
        codeForPreview: mode === EDITOR_MODE.PREVIEW ? value : null
      }, function () {
        // 切换为编辑模式时，修订 body 区域尺寸
        if (mode === EDITOR_MODE.CODE) {
          _this.fixEditorBodySize();
        }
      });
    };

    _this.doPreview = function () {
      var code = _this.instance.getValue();
      _this.setState({
        codeForPreview: code
      });
    };

    _this.getHelpUrl = function () {
      return TEXT_DIAGRAM_GUIDES[_this.state.type];
    };

    _this.getRootDom = function () {
      return _this.rootRef.current;
    };

    _this.onPreviewDone = function () {
      _this.fixEditorBodySize();
    };

    _this.handlePreviewButtonClick = function () {
      var _this$state = _this.state,
          mode = _this$state.mode,
          layout = _this$state.layout;
      // 双栏模式直接执行预览

      if (layout === EDITOR_LAYOUT.TWO_COLUMN) {
        _this.doPreview();
        return;
      }

      var rootCenter = _tinsoEditorEngine2.default.$(_this.rootRef.current).closest('div[data-section-element=center]');
      var sectionResize = rootCenter.find('div.section-resize-button-ud');

      if (mode === EDITOR_MODE.PREVIEW) {
        sectionResize.show();
        _this.switchView(EDITOR_MODE.CODE);
        return;
      }

      if (mode === EDITOR_MODE.CODE) {
        sectionResize.hide();
        _this.switchView(EDITOR_MODE.PREVIEW);
      }
    };

    _this.needShowPreviewPanel = function () {
      if (_this.state.layout === EDITOR_LAYOUT.TWO_COLUMN) return true;
      return _this.state.mode === EDITOR_MODE.PREVIEW;
    };

    _this.renderPreview = function () {
      var _this$state2 = _this.state,
          mode = _this$state2.mode,
          layout = _this$state2.layout,
          codeForPreview = _this$state2.codeForPreview;
      // 单栏布局、代码模式，不渲染

      if (layout === EDITOR_LAYOUT.DEFAULT && mode === EDITOR_MODE.CODE) return null;
      // 无待预览代码，不渲染
      if (!codeForPreview) return null;
      return _react2.default.createElement(_textDiagramRender2.default, {
        type: _this.state.type,
        code: codeForPreview,
        onBeforeRenderImage: _this.props.onBeforeRenderImage,
        onLoad: _this.onPreviewDone
      });
    };

    _this.renderDiagramSelect = function () {
      return _react2.default.createElement(
        _antd.Select,
        {
          className: 'zacheditor-diagram-select',
          showSearch: true,
          defaultValue: _this.props.type,
          size: 'small',
          style: {
            width: 128
          },
          getPopupContainer: _this.getRootDom,
          onSelect: function onSelect(type) {
            _this.switchType(type);
          },
          filterOption: _this.handlerFilter
        },
        TEXT_DIAGRAMS.map(function (item) {
          return _react2.default.createElement(
            _antd.Select.Option,
            { name: item.name.toLowerCase(), value: item.type, key: item.name },
            item.name
          );
        })
      );
    };
    _this.state = {
      mode: EDITOR_MODE.CODE,
      type: props.type || TEXT_DIAGRAM_TYPES.PUML,
      layout: EDITOR_LAYOUT.DEFAULT,
      // 待预览代码
      codeForPreview: null
    };
    _this.rootRef = _react2.default.createRef(); // 代码编辑器宿主引用
    _this.editorRef = _react2.default.createRef(); // 编辑框主体区域引用
    _this.bodyRef = _react2.default.createRef(); // 支持的模板
    _this.templates = _template2.default.getTemplates(props.type); // 编辑器实例
    _this.instance = null;
    return _this;
  }

  _createClass(TextDiagramEditor, [{
    key: 'componentDidMount',
    value: function componentDidMount() {
      this.initEditor();
    }
  }, {
    key: 'initEditor',
    value: function initEditor() {
      var _this2 = this;

      var textAreaDom = this.editorRef.current;
      var options = this.getOptions(this.props.type);
      var isMaximize = this.props.isMaximize;

      var doPreview = (0, _debounce2.default)(function () {
        _this2.doPreview();
      }, 300);
      // 构造代码编辑器
      var cm = _codemirror2.default.fromTextArea(textAreaDom, options);
      cm.on('change', function () {
        var value = cm.getValue();
        if (typeof _this2.props.onChange === 'function') {
          _this2.props.onChange(value);
        }
        if (isMaximize()) {
          doPreview();
        }
      });
      // 点击编辑区域会触发 codemiror 的 focus()
      // 同时也会触发 section 绑定的 onclick 导致再次执行 focus()
      // 但 codemirror 做到了 focus 态时再执行 focus() 时不会再次触发 on focus 事件，

      cm.on('focus', function () {
        if (typeof _this2.props.onFocus === 'function') {
          _this2.props.onFocus();
        }
      });
      cm.on('blur', function () {
        if (typeof _this2.props.onBlur === 'function') {
          var value = cm.getValue();
          _this2.props.onBlur(value);
        }
      });
      this.instance = {
        codeMirror: cm,
        switchLayout: this.switchLayout,
        focus: function focus() {
          cm.focus();
        },
        getValue: function getValue() {
          return cm.getValue();
        },
        doPreview: function doPreview() {
          _this2.doPreview();
        }
      };

      if (typeof this.props.onLoad === 'function') {
        this.props.onLoad({
          editor: this.instance
        });
      }
    }
  }, {
    key: 'getCodeMirrorMode',
    value: function getCodeMirrorMode() {
      return TEXT_DIAGRAM_SYNTAX[this.props.type] || 'simplemode';
    }
  }, {
    key: 'getDiagramName',
    value: function getDiagramName(type) {
      var diagram = TEXT_DIAGRAMS.find(function (item) {
        return item.type === type;
      });
      return diagram ? diagram.name : '';
    }
  }, {
    key: 'getOptions',
    value: function getOptions(type) {
      return {
        tabSize: 2,
        mode: this.getCodeMirrorMode(type),
        lineNumbers: true,
        lineWrapping: false,
        indentWithTabs: true,
        styleActiveLine: false,
        matchBrackets: true,
        autofocus: false,
        dragDrop: false,
        readOnly: false,
        scrollbarStyle: 'null',
        viewportMargin: 1 / 0
      };
    }
    // hack: 将 popover dorpdown 限定在容器范围内，阻止 meun popover 事件被拦截
    // 修订编辑区域的展示尺寸，保证给预览区域足够的空间展示图片

  }, {
    key: 'fixEditorBodySize',
    value: function fixEditorBodySize() {
      var _state = this.state,
          mode = _state.mode,
          layout = _state.layout;
      // 仅单栏模式需要调整

      if (layout !== EDITOR_LAYOUT.DEFAULT) return;
      var body = this.bodyRef.current;
      var node = null;
      var height = null;

      if (mode === EDITOR_MODE.PREVIEW) {
        node = body.querySelector('.zacheditor-text-diagram-preview');
        height = node.scrollHeight;
        body.style.height = height + 'px';
      } else {
        body.style.height = 'auto';
      }
    }
  }, {
    key: 'renderTemplates',
    value: function renderTemplates() {
      var _this3 = this;

      var locale = this.props.locale;
      var template = _template2.default.getTemplates(this.state.type);
      return template ? _react2.default.createElement(
        _antd.Dropdown,
        {
          trigger: ['click'] // 将 Dropdown 控制在 section 内，以便控制焦点
          , getPopupContainer: this.getRootDom,
          disabled: this.state.mode !== EDITOR_MODE.CODE,
          placement: 'bottomLeft',
          overlay: _react2.default.createElement(
            _antd.Menu,
            {
              onClick: function onClick(_ref) {
                var key = _ref.key,
                    domEvent = _ref.domEvent;
                // 需要阻止事件被编辑器捕获

                domEvent.stopPropagation();
                _this3.switchTemplate(key);
              }
            },
            Object.values(this.templates).map(function (template) {
              return _react2.default.createElement(
                _antd.Menu.Item,
                { key: template.id },
                template.name
              );
            })
          )
        },
        _react2.default.createElement(
          'a',
          { className: 'diagram-template-selector' },
          locale.template,
          ' ',
          _react2.default.createElement(_icons3.DownOutlined, null)
        )
      ) : null;
    }
  }, {
    key: 'render',
    value: function render() {
      var _this4 = this;

      var locale = this.props.locale;
      var _state2 = this.state,
          mode = _state2.mode,
          layout = _state2.layout;

      return _react2.default.createElement(
        'div',
        {
          className: (0, _classnames2.default)(['zacheditor-text-diagram-stage', 'zacheditor-text-diagram-layout-'.concat(layout)]),
          onKeyDown: function onKeyDown(event) {
            if ((event.metaKey || event.ctrlKey) && 80 === event.keyCode) {
              event.preventDefault();
              _this4.handlePreviewButtonClick();
            }
          },
          ref: this.rootRef
        },
        _react2.default.createElement(
          'div',
          { className: 'zacheditor-text-diagram-nav' },
          this.props.canChangeType ? this.renderDiagramSelect() : _react2.default.createElement(
            'span',
            { className: 'diagram-name' },
            this.getDiagramName(this.props.type)
          ),
          _react2.default.createElement(
            'div',
            { className: 'diagram-actions' },
            this.renderTemplates(),
            _react2.default.createElement(
              'a',
              { className: 'diagram-help', href: this.getHelpUrl(), target: '_blank' },
              locale.help
            ),
            _react2.default.createElement(
              _antd.Tooltip,
              { title: macos ? locale.macPreviewTooltip : locale.winPreviewTooltip },
              _react2.default.createElement(
                'button',
                {
                  size: 'small',
                  className: (0, _classnames2.default)({
                    'diagram-preview': true,
                    'diagram-preview-active': mode === EDITOR_MODE.PREVIEW
                  }),
                  onClick: this.handlePreviewButtonClick
                },
                _react2.default.createElement(_icons4.default, { component: _icons2.default.Preview }),
                locale.preview
              )
            )
          )
        ),
        _react2.default.createElement(
          'div',
          { className: 'zacheditor-text-diagram-editor-wrap', ref: this.bodyRef },
          _react2.default.createElement(
            'div',
            { className: 'zacheditor-text-diagram-editor' },
            _react2.default.createElement('textarea', {
              value: this.props.code,
              ref: this.editorRef,
              autoComplete: 'off',
              tabIndex: '1',
              readOnly: true
            })
          ),
          _react2.default.createElement(
            'div',
            {
              className: 'zacheditor-text-diagram-preview',
              style: { display: this.needShowPreviewPanel() ? 'block' : 'none' }
            },
            this.renderPreview()
          )
        )
      );
    }
  }]);

  return TextDiagramEditor;
}(_react2.default.PureComponent);

exports.default = TextDiagramEditor;