'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _tinycolor = require('tinycolor2');

var _tinycolor2 = _interopRequireDefault(_tinycolor);

var _classnames = require('classnames');

var _classnames2 = _interopRequireDefault(_classnames);

var _palette = require('../utils/palette');

var _palette2 = _interopRequireDefault(_palette);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var ColorPickerItem = function (_React$Component) {
  _inherits(ColorPickerItem, _React$Component);

  function ColorPickerItem() {
    _classCallCheck(this, ColorPickerItem);

    var _this = _possibleConstructorReturn(this, (ColorPickerItem.__proto__ || Object.getPrototypeOf(ColorPickerItem)).call(this));

    _this.toState = function (data, oldHue) {
      var color = data.hex ? (0, _tinycolor2.default)(data.hex) : (0, _tinycolor2.default)(data);
      var hsl = color.toHsl();
      var hsv = color.toHsv();
      var rgb = color.toRgb();
      var hex = color.toHex();

      if (hsl.s === 0) {
        hsl.h = oldHue || 0;
        hsv.h = oldHue || 0;
      }

      var transparent = hex === '000000' && rgb.a === 0;
      return {
        hsl: hsl,
        hex: transparent ? 'transparent' : '#'.concat(hex),
        rgb: rgb,
        hsv: hsv,
        oldHue: data.h || oldHue || hsl.h,
        source: data.source
      };
    };

    _this.getContrastingColor = function (data) {
      if (!data) {
        return '#fff';
      }
      var col = _this.toState(data);
      if (col.hex === 'transparent') {
        return 'rgba(0,0,0,0.4)';
      }

      var yiq = (col.rgb.r * 299 + col.rgb.g * 587 + col.rgb.b * 114) / 1000;
      return yiq >= 210 ? '#8C8C8C' : '#FFFFFF';
    };

    _this.onSelect = function (color, event) {
      event.preventDefault();
      event.stopPropagation();
      _this.props.onSelect && _this.props.onSelect(color);
    };
    return _this;
  }

  _createClass(ColorPickerItem, [{
    key: 'render',
    value: function render() {
      var _props = this.props,
          color = _props.color,
          activeColors = _props.activeColors,
          defaultColor = _props.defaultColor,
          setStroke = _props.setStroke;

      var needborder = ['#FFFFFF', '#FAFAFA'].indexOf(color) >= 0;
      var active = activeColors.indexOf(color) >= 0;
      var special = color === defaultColor;
      var styls = {
        check: {
          fill: this.getContrastingColor(color),
          display: active ? 'block' : 'none'
        },
        block: {
          backgroundColor: ''.concat(color)
        }
      };
      if (setStroke) {
        styls.block.border = '1px solid '.concat(_palette2.default.getStroke(color));
      }
      return _react2.default.createElement(
        'span',
        {
          className: (0, _classnames2.default)('zacheditor-colorboard-group-item', {
            'zacheditor-colorboard-group-item-border': needborder,
            'zacheditor-colorboard-group-item-active': active,
            'zacheditor-colorboard-group-item-special': special
          }),
          onClick: this.onSelect.bind(this, color)
        },
        _react2.default.createElement(
          'span',
          { style: styls.block },
          _react2.default.createElement(
            'svg',
            { style: styls.check, viewBox: '0 0 18 18' },
            _react2.default.createElement('path', { d: 'M21,7L9,19L3.5,13.5L4.91,12.09L9,16.17L19.59,5.59L21,7Z' })
          )
        )
      );
    }
  }]);

  return ColorPickerItem;
}(_react2.default.Component);

var ColorPickerGroup = function (_React$Component2) {
  _inherits(ColorPickerGroup, _React$Component2);

  function ColorPickerGroup() {
    _classCallCheck(this, ColorPickerGroup);

    return _possibleConstructorReturn(this, (ColorPickerGroup.__proto__ || Object.getPrototypeOf(ColorPickerGroup)).apply(this, arguments));
  }

  _createClass(ColorPickerGroup, [{
    key: 'render',
    value: function render() {
      var data = this.props.data === void 0 ? [] : this.props.data;
      var _props2 = this.props,
          activeColors = _props2.activeColors,
          defaultColor = _props2.defaultColor,
          onSelect = _props2.onSelect,
          setStroke = _props2.setStroke;


      return _react2.default.createElement(
        'span',
        { className: 'zacheditor-colorboard-group' },
        data.map(function (c) {
          return _react2.default.createElement(ColorPickerItem, {
            color: c,
            key: c,
            activeColors: activeColors,
            defaultColor: defaultColor,
            onSelect: onSelect,
            setStroke: setStroke
          });
        })
      );
    }
  }]);

  return ColorPickerGroup;
}(_react2.default.Component);

var ColorPicker = function (_React$Component3) {
  _inherits(ColorPicker, _React$Component3);

  function ColorPicker() {
    _classCallCheck(this, ColorPicker);

    return _possibleConstructorReturn(this, (ColorPicker.__proto__ || Object.getPrototypeOf(ColorPicker)).apply(this, arguments));
  }

  _createClass(ColorPicker, [{
    key: 'render',
    value: function render() {
      var _props3 = this.props,
          activeColors = _props3.activeColors,
          defaultColor = _props3.defaultColor,
          onSelect = _props3.onSelect,
          setStroke = _props3.setStroke;
      var colors = this.props.colors;

      if (!colors) colors = _palette2.default.getColors();
      return _react2.default.createElement(
        'div',
        { className: 'zacheditor-colorboard' },
        colors.map(function (data, index) {
          return _react2.default.createElement(ColorPickerGroup, {
            data: data,
            key: index,
            activeColors: activeColors,
            defaultColor: defaultColor,
            onSelect: onSelect,
            setStroke: setStroke
          });
        })
      );
    }
  }]);

  return ColorPicker;
}(_react2.default.Component);

exports.default = ColorPicker;