'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _get = require('lodash/get');

var _get2 = _interopRequireDefault(_get);

var _ajax = require('../utils/ajax');

var _ajax2 = _interopRequireDefault(_ajax);

var _antd = require('antd');

require('antd/lib/message/style');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var VideoUploader = function () {
  function VideoUploader(options) {
    _classCallCheck(this, VideoUploader);

    this.trigger = options.trigger;
    this.engine = options.engine;
    this.onToolBarClick = options.onToolBarClick;
    this.onToolBarError = options.onToolBarError;
    this.sectionRoot = null;
    this.selectedFile = null;
    this.data = {};
    this.initUploader();
  }

  _createClass(VideoUploader, [{
    key: 'initUploader',
    value: function initUploader() {
      var _this = this;

      var VodUploader = (0, _get2.default)(window, 'AliyunUpload.Vod');

      if (!VodUploader || !this.trigger) {
        _antd.message.error('Not found AliyunUpload.Vod');
        return;
      }

      var options = this.engine.options.video || {};
      var uploader = new VodUploader({
        timeout: options.timeout || 60 * 1000,
        partSize: options.partSize || 1048576,
        parallel: options.parallel || 5,
        retryCount: options.retryCount || 3,
        retryDuration: options.retryDuration || 2,
        region: options.region || '',
        userId: options.user_id || '',
        addFileSuccess: function addFileSuccess(uploadInfo) {
          // 触发外围 toolbar 的 click 事件， 用于选择文件后关闭菜单
          _this.onToolBarClick();
          var key = uploadInfo.key,
              file = uploadInfo.file;
          //debug('上传:文件选择: %j', file)

          _this.selectedFile = file;
          // 选定文件后，执行插入Section
          _this.insertSection(key, file);
          uploader.startUpload();
        },
        // 开始上传
        onUploadstarted: function onUploadstarted(uploadInfo) {
          var filename = uploadInfo.file.name;
          var filesize = uploadInfo.file.size;
          (0, _ajax2.default)({
            url: options.action.create,
            dataType: 'json',
            data: {
              filename: filename,
              filesize: filesize
            },
            success: function success(_ref) {
              var result = _ref.result,
                  data = _ref.data,
                  message = _ref.message;

              if (!result) {
                message.error(message);
                return;
              }
              var UploadAuth = data.UploadAuth || data.upload_auth;
              var UploadAddress = data.UploadAddress || data.upload_address;
              var VideoId = data.VideoId || data.video_id;
              uploadInfo.key = data.key;
              uploader.setUploadAuthAndAddress(uploadInfo, UploadAuth, UploadAddress, VideoId);
            },
            error: function error() {
              _this.onToolBarError({
                code: -100
              });
            },
            method: 'GET',
            processData: true
          });
        },
        // 文件上传成功
        onUploadSucceed: function onUploadSucceed(uploadInfo) {
          var videoId = uploadInfo.videoId,
              file = uploadInfo.file,
              key = uploadInfo.key;
          var name = file.name,
              size = file.size,
              type = file.type;

          var fileType = name.split('.').pop();
          if (fileType) {
            fileType = '.'.concat(fileType);
          }
          (0, _ajax2.default)({
            method: 'POST',
            url: options.action.save,
            dataType: 'json',
            data: {
              key: key,
              filename: name,
              filesize: size,
              filetype: type || fileType || 'unknow',
              video_id: videoId
            },
            success: function success() {
              _this.updateSection({
                status: 'uploaded',
                videoId: videoId,
                key: key
              });
            },
            error: function error() {
              _this.onToolBarError({
                code: -100
              });
            },
            processData: true
          });
        },
        // 文件上传进度，单位：字节, 可以在这个函数中拿到上传进度并显示在页面上
        onUploadProgress: function onUploadProgress(uploadInfo, totalSize, progress) {
          var progressPercent = Math.ceil(progress * 100);
          //debug('上传:进度: %j', progressPercent)
          // 不直接显示 100%，为截图服务和保存服务耗费的时间预留buffer

          if (progressPercent > 98) {
            return;
          }

          _this.updatePercent({
            percent: progressPercent
          });
        }
      });
      this.trigger.addEventListener('change', function (e) {
        var file = e.target.files[0];
        uploader.addFile(file, null, null, null, null);
        _this.trigger.value = '';
      }, false);
      window.uploader = uploader;
    }
  }, {
    key: 'insertSection',
    value: function insertSection(key, file) {
      var engine = this.engine;
      this.data = {
        key: key,
        status: 'uploading',
        name: file.name,
        size: file.size,
        percent: 0
      };
      engine.history.stop();
      this.sectionRoot = engine.change.insertSection('video', this.data);
      engine.history.start();
    }
  }, {
    key: 'updatePercent',
    value: function updatePercent(file) {
      this.sectionRoot.find('[data-role=percent]').html(''.concat(file.percent, '%'));
    }
  }, {
    key: 'updateSection',
    value: function updateSection() {
      var data = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
      this.data = Object.assign(this.data, data);
      this.engine.change.updateSection(this.sectionRoot, this.data);
    }
  }]);

  return VideoUploader;
}();

exports.default = VideoUploader;