'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _request = require('./request');

var _request2 = _interopRequireDefault(_request);

var _url2 = require('url');

var _url3 = _interopRequireDefault(_url2);

var _loadIframe = require('./loadIframe');

var _loadIframe2 = _interopRequireDefault(_loadIframe);

var _utils = require('./utils');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var TS = '_ts';
var CHARSET_KEY = '_input_charset';
var UTF8 = 'utf-8';
var globalSetupOptions = {
  iframeProxyUrl: '/proxy.html',
  useIframeProxy: false,
  withToken: true,
  withInputCharset: true,
  cache: true
};
var hostname = window.location.hostname; // 不考虑端口...
function patch(opt_) {
  var win = arguments.length <= 1 || arguments[1] === undefined ? window : arguments[1];
  var opt = opt_;
  var cache = globalSetupOptions.cache || opt.method === 'post';
  var token = (0, _utils.getTokenFromCookie)(win);
  var withToken = globalSetupOptions.withToken;
  var withInputCharset = globalSetupOptions.withInputCharset;

  var extraData = undefined;
  opt.data = opt.data || {};
  if (Array.isArray(opt.data)) {
    extraData = [];
    if (withToken && token) {
      extraData.push({
        name: _utils.TOKEN,
        value: encodeURIComponent(token)
      });
    }
    if (withInputCharset) {
      extraData.push({
        name: CHARSET_KEY,
        value: UTF8
      });
    }
    if (!cache) {
      extraData.push({
        name: TS,
        value: Date.now()
      });
    }
    opt.data = opt.data.concat(extraData);
  } else if (typeof opt.data === 'string') {
    if (withInputCharset && opt.data.indexOf(CHARSET_KEY) === -1) {
      opt.data += '&' + CHARSET_KEY + '=' + UTF8;
    }
    if (withToken && token && opt.data.indexOf(_utils.TOKEN) === -1) {
      // 字符串形式的 jquery 不会 encode
      opt.data += '&' + _utils.TOKEN + '=' + encodeURIComponent(token);
    }
    if (!cache) {
      opt.data += '&' + TS + '=' + Date.now();
    }
  } else if ((0, _utils.isAFormData)(opt.data)) {
    if (withInputCharset && !opt.data.get(CHARSET_KEY)) {
      opt.data.append(CHARSET_KEY, UTF8);
    }
    if (opt.url && withToken && token) {
      opt.url = (0, _utils.urlAppend)(opt.url, (0, _utils.toQueryString)(_defineProperty({}, _utils.TOKEN, token)));
    }
  } else {
    if (withInputCharset) {
      opt.data[CHARSET_KEY] = UTF8;
    }
    if (!cache) {
      opt.data[TS] = Date.now();
    }
    if (withToken && token && !opt.data[_utils.TOKEN]) {
      opt.data[_utils.TOKEN] = token;
    }
  }
  return opt;
}

function newAjax(_url, callback) {
  var opt = undefined;
  var url = _url;
  if (typeof url === 'string') {
    opt = {
      url: url,
      success: callback
    };
  } else {
    opt = url;
  }

  url = opt.url;
  var req = (0, _request2.default)();

  if ((0, _utils.startsWith)(url, '//')) {
    url = window.location.protocol + url;
  }

  if (opt.type !== 'jsonp') {
    var useIframeProxy = globalSetupOptions.useIframeProxy;
    if ('useIframeProxy' in opt) {
      useIframeProxy = opt.useIframeProxy;
    }

    if (useIframeProxy && (0, _utils.startsWith)(url, 'http:') || (0, _utils.startsWith)(url, 'https:')) {
      var myUrl = _url3.default.parse(url);
      var myHostname = myUrl.hostname;
      if (myHostname !== hostname) {
        var iframeProxyUrl = opt.iframeProxyUrl || globalSetupOptions.iframeProxyUrl;
        if ((0, _utils.endsWith)(myHostname, document.domain)) {
          (0, _loadIframe2.default)(myUrl.protocol + '//' + myUrl.host + iframeProxyUrl, function (error, iframe) {
            if (error) {
              req.setup(opt);
              req.triggerError(error);
            } else {
              opt.context = iframe.contentWindow;
              // 如果是 cdn 则不处理
              if ((0, _utils.urlNeedPatch)(url)) {
                opt = patch(opt, opt.context);
              }
              req.start(opt);
            }
          });
          return req;
        }
        // cors
      }
    }
  }

  // 如果是 cdn 则不处理
  if ((0, _utils.urlNeedPatch)(url)) {
    opt = patch(opt);
  }
  req.start(opt);
  return req;
}

newAjax.ajaxSetup = function (options) {
  for (var k in options) {
    if (options.hasOwnProperty(k) && globalSetupOptions.hasOwnProperty(k)) {
      globalSetupOptions[k] = options[k];
    }
  }
  _request2.default.ajaxSetup(options);
};
exports.default = newAjax;