'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var TOKEN = exports.TOKEN = 'token';
var isArray = Array.isArray;

var urlNeedPatch = exports.urlNeedPatch = function urlNeedPatch(url) {
  return !/^https?:\/\/cdn\.objects\.zacheditor\.com\//g.test(url);
};

var startsWith = exports.startsWith = function startsWith(str, prefix) {
  return str.slice(0, prefix.length) === prefix;
};

var endsWith = exports.endsWith = function endsWith(str, suffix) {
  return str.slice(-suffix.length) === suffix;
};

var getTokenFromCookie = exports.getTokenFromCookie = function getTokenFromCookie(win) {
  var cookieParts = win.document.cookie.split(/;\s/g);
  for (var i = 0, len = cookieParts.length; i < len; i++) {
    var cookieNameValue = cookieParts[i].match(/([^=]+)=/i);
    if (cookieNameValue && cookieNameValue[1] === TOKEN) {
      return cookieParts[i].substring(cookieNameValue[1].length + 1);
    }
  }
};

var buildParams = exports.buildParams = function buildParams(prefix, obj, traditional, add) {
  var name = undefined;
  var i = undefined;
  var v = undefined;
  var rbracket = /\[\]$/;

  if (isArray(obj)) {
    // Serialize array item.
    for (i = 0; obj && i < obj.length; i++) {
      v = obj[i];
      if (traditional || rbracket.test(prefix)) {
        // Treat each array item as a scalar.
        add(prefix, v);
      } else {
        buildParams(prefix + '[' + ((typeof v === 'undefined' ? 'undefined' : typeof v === 'undefined' ? 'undefined' : _typeof(v)) === 'object' ? i : '') + ']', v, traditional, add);
      }
    }
  } else if (obj && obj.toString() === '[object Object]') {
    // Serialize object item.
    for (name in obj) {
      if (obj.hasOwnProperty(name)) {
        buildParams(prefix + '[' + name + ']', obj[name], traditional, add);
      }
    }
  } else {
    // Serialize scalar item.
    add(prefix, obj);
  }
};

var urlAppend = exports.urlAppend = function urlAppend(url, s) {
  return url + (/\?/.test(url) ? '&' : '?') + s;
};

var toQueryString = exports.toQueryString = function toQueryString(o, trad) {
  var prefix = undefined;
  var i = undefined;
  var traditional = trad || false;
  var s = [];
  var enc = encodeURIComponent;

  var add = function add(key, value) {
    var v = value;
    // If value is a function, invoke it and return its value
    if (typeof value === 'function') {
      v = value();
    } else if (value === null || value === undefined) {
      v = '';
    }
    s[s.length] = enc(key) + '=' + enc(v);
  };

  // If an array was passed in, assume that it is an array of form elements.
  if (isArray(o)) {
    for (i = 0; o && i < o.length; i++) {
      add(o[i].name, o[i].value);
    }
  } else {
    // If traditional, encode the "old" way (the way 1.3.2 or older
    // did it), otherwise encode params recursively.
    for (prefix in o) {
      if (o.hasOwnProperty(prefix)) {
        buildParams(prefix, o[prefix], traditional, add);
      }
    }
  }
  // spaces should be + according to spec
  return s.join('&').replace(/%20/g, '+');
};

var isAFormData = exports.isAFormData = function isAFormData(data) {
  return typeof FormData !== 'undefined' && data instanceof FormData;
};