'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.loadScript = exports.namespaceDelete = exports.put = exports.post = exports.get = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _queryString = require('query-string');

var _queryString2 = _interopRequireDefault(_queryString);

var _jsCookie = require('js-cookie');

var _jsCookie2 = _interopRequireDefault(_jsCookie);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var getToken = function getToken() {
  return _jsCookie2.default.get('token');
};

var formatError = function formatError(message, options) {
  var resp = void 0;
  try {
    resp = JSON.parse(message);
  } catch (err) {
    resp = {
      message: message
    };
  }

  var err = new Error(resp.message);
  Object.assign(err, resp);
  if (!options || !options.ignoreGlobalMessage) {
    err.globalMessage = resp.message;
  }
  return err;
};

var parse = function parse(res, url, options) {
  if (res.status >= 200 && res.status < 300) {
    return res.json().catch(function () {});
  }

  if (res.status === 401) {
    if (window.location.pathname !== '/login') {
      var params = new URLSearchParams();
      params.set('goto', window.location.href);
      window.location.href = '/login?'.concat(params.toString());
    } else {
      window.location.href = '/login';
    }
    return {};
  }
  // 返回错误的消息体，有可能不是 JSON
  return res.text().then(function (message) {
    throw formatError(message, options);
  });
};

// 将 POST/PUT 的参数转换为 String
var JSONStringify = function JSONStringify(parameters) {
  if (!parameters) return parameters;
  return JSON.stringify(parameters);
};

var zacheditorFetch = function zacheditorFetch(url, options) {
  options = options || {};
  var opts = _extends({}, options, {
    credentials: 'same-origin',
    headers: _extends({
      'Content-Type': 'application/json'
    }, options.headers, {
      Accept: 'application/json',
      'X-Requested-With': 'XMLHttpRequest',
      'x-csrf-token': getToken(window, true)
    })
  });
  // 通过 FormData multipart 上传文件时，不能传递 `Content-Type`
  if (!opts.headers['Content-Type']) delete opts.headers['Content-Type'];

  return fetch(url, opts).then(function (res) {
    return parse(res, url, options);
  }, function (error) {
    throw 'network error: '.concat(error.message);
  });
};

exports.default = zacheditorFetch;
var get = exports.get = function get(url, parameters, options) {
  var realPath = parameters ? ''.concat(url, '?').concat(_queryString2.default.stringify(parameters)) : url;
  return zacheditorFetch(realPath, options);
};

var post = exports.post = function post(url, parameters, options) {
  return zacheditorFetch(url, _extends({
    method: 'POST',
    body: JSONStringify(parameters)
  }, options));
};

var put = exports.put = function put(url, parameters, options) {
  return zacheditorFetch(url, _extends({
    method: 'PUT',
    body: JSONStringify(parameters)
  }, options));
};

var namespaceDelete = exports.namespaceDelete = function namespaceDelete(url, parameters, options) {
  var realPath = parameters && Object.keys(parameters).length ? ''.concat(url, '?').concat(_queryString2.default.stringify(parameters)) : url;
  return zacheditorFetch(realPath, _extends({
    method: 'DELETE'
  }, options));
};

var loadScript = exports.loadScript = function loadScript(url, callback) {
  var script = document.createElement('script');
  script.src = url;
  document.head.appendChild(script);
  script.onload = callback;
};