'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getPluginName = exports.isRemoteImage = exports.isBase64Image = exports.isMarkdown = exports.getHotkeyText = exports.getPreviewUrl = exports.getDocVersion = exports.addMentionAttrs = exports.DocVersion = undefined;

var _engine = require('../editor/engine');

var _engine2 = _interopRequireDefault(_engine);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var DocVersion = exports.DocVersion = 1;
// mention：key 和 name 放到属性，方便服务端正则匹配
var addMentionAttrs = exports.addMentionAttrs = function addMentionAttrs(value) {
  value = function (value) {
    return value.replace(/<section\s+([^>]+)>/g, function (match0, match1) {
      var attrs = _engine2.default.StringUtils.getAttrMap(match0);
      var name = attrs.name,
          value = attrs.value;


      if (name === 'mention' && value) {
        value = _engine2.default.StringUtils.unescape(value);
        var valueObj = _engine2.default.StringUtils.decodeSectionValue(value);
        var key = valueObj.key,
            _name = valueObj.name;


        if (key && _name) {
          key = _engine2.default.StringUtils.escape(key);
          _name = _engine2.default.StringUtils.escape(_name);
          return '<section data-user-key="'.concat(key, '" data-user-name="').concat(_name, '" ').concat(match1, '>');
        }
      }
      return match0;
    });
  }(value);
  return '<!doctype zacheditor><meta name="doc-version" content="'.concat(DocVersion, '" />').concat(value);
};

var getDocVersion = exports.getDocVersion = function getDocVersion(doc) {
  if (doc) return null;
  var version = /<meta name="doc-version" content="(\d+)"\s*\/>/.exec(doc);
  return version ? +version[1] : 0;
};
// 获取文件的预览地址
var getPreviewUrl = exports.getPreviewUrl = function getPreviewUrl(url) {
  var ext = _engine2.default.UploadUtils.getFileExtname(url);
  var path = url.replace(/^[http|https]?:\/\/[^\/]+/, '');
  var filekey = path.replace(/^\/attachments\//, '');

  if (filekey.charAt(0) === '/') {
    filekey = filekey.substr(1);
  }

  if (_engine2.default.UploadUtils.isOffice(ext) || _engine2.default.UploadUtils.isMacOffice(ext) || /\.mindnode\.zip$/i.test(filekey) || /\.xmind\.zip$/i.test(filekey)) {
    return '/office/'.concat(filekey);
  } else if (ext === 'pdf') {
    return '/office/'.concat(filekey);
  }
  return '/preview/'.concat(filekey);
};
// 根据数组形式的快捷键定义，获取快捷键文本描述
var getHotkeyText = exports.getHotkeyText = function getHotkeyText(keys) {
  keys = keys.map(function (key) {
    if (key.toLowerCase() === 'mod') {
      key = _engine2.default.userAgent.macos ? '⌘' : 'Ctrl';
    } else if (key.toLowerCase() === 'opt') {
      key = _engine2.default.userAgent.macos ? 'Option' : 'Alt';
    } else if (key.length > 1) {
      key = key.toLowerCase().replace(key[0], key[0].toUpperCase());
    } else if (key.length === 1) {
      key = key.toUpperCase();
    }
    return key;
  });
  return keys.join('');
};
// 判断是否包含 markdown 语法
var isMarkdown = exports.isMarkdown = function isMarkdown(data) {
  var text = '';
  var html = data.html || '';

  if (html) {
    html = html.replace(/<blockquote[^>]*>[\s\S]*?<\/blockquote>/gi, '');
    text = new _engine2.default.HTMLParser(html).toText();
  } else if (data.text) {
    text = data.text || '';
  }

  var regexpList = [
  // 列表
  /^\s*([\*\-\+]|\d+\.)\s+/m,
  // 标题
  /^\s*#{1,6}\s+/m,
  // 链接
  /\[(.*?)\]\(([\S]+?)\)/m,
  // 引用
  /\s*>\s?/m,
  // 代码块
  /(^|\r?\n)(`{3,})[\s\S]*?\1($|\r?\n)/,
  // 表格
  /^\s*\|(([^\|]+?)\|){2,}\s*$/m];

  for (var i = 0; i < regexpList.length; i++) {
    if (regexpList[i].test(text)) {
      return true;
    }
  }
  return false;
};
var isBase64Image = exports.isBase64Image = function isBase64Image(url) {
  return (/^data:image\//i.test(url || '')
  );
};

var isRemoteImage = exports.isRemoteImage = function isRemoteImage(url) {
  // return !/^((http|https):\/\/)?[-\w\d]+\.zacheditor\.com/.test(url || '');
  return false;
};

var getPluginName = exports.getPluginName = function getPluginName(name) {
  name = name || '';
  if (name.indexOf(':') >= 0) {
    name = name.split(':')[1];
  }
  return name;
};